;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-args.el                    */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Apr 30 10:24:19 1998                          */
;*    Last change :  Sun Oct 25 07:50:53 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    This file implement the bdb args printing.                       */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-args)
(require 'bdb-config)
(require 'bdb)

;*---------------------------------------------------------------------*/
;*    Global control variables.                                        */
;*---------------------------------------------------------------------*/
;; The completion process filter indicates when it is finished.
(defvar bdb-args-in-progress)

;; Since output may arrive in fragments we accumulate partials strings here.
(defvar bdb-args-string)

;; this variable holds a state: are we currently inquirying for argss
(defvar bdb-args-command-armed-p nil)

;; the buffer for C args
(defvar bdb-args-buffer nil)

;; which args to args (Scheme or C)?
(defvar bdb-args 'scheme)

;; display in frame or window
(defvar bdb-args-in-frame-p t)

;*---------------------------------------------------------------------*/
;*    bdb-args-hook ...                                                */
;*---------------------------------------------------------------------*/
(defun bdb-args-hook (input)
  (if (and (not bdb-args-command-armed-p)
	   (or (not (stringp input))
	       ;; we invoke this hook on:
	       ;; next, step, continue, until, call, finish, run,
	       ;; return and breturn.
	       ;; it is important not to re-call the hook on `frame...'
	       (string-match "^[nscruf]\\|^b[esrcnuf]\\|^disp" input)))
      (progn
	(if (not (bufferp bdb-args-buffer))
	    (setq bdb-args-buffer (bdb-make-args-buffer "*Bdb args*"))
	  (bdb-popup-args bdb-args-buffer))
	(bdb-args-command bdb-args-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-args-command ...                                             */
;*    -------------------------------------------------------------    */
;*    This command refresh the args tracing.                           */
;*---------------------------------------------------------------------*/
(defun bdb-args-command (buffer)
  "Perform completion on the BDB command preceding point.
This is implemented using the BDB `args' command." 
  (interactive)
  (if (not bdb-args-command-armed-p)
      (progn 
	;; we mark that args is armed not to re-enter this hook
	(setq bdb-args-command-armed-p t)
	;; we wait for comint to be ready
	(bdb-wait-for-comint "bdb-args-command")
	;; Temporarily install our filter function.
	(let ((bdb-marker-filter 'bdb-args-filter))
	  ;; Issue the command to BDB.
	  (bdb-info-args-call)
	  (setq bdb-args-in-progress t)
	  (setq bdb-args-string "")
	  ;; Slurp the output.
	  (while bdb-args-in-progress
	    (if (not (accept-process-output
		      (get-buffer-process bdb-comint-buffer)
		      bdb-wait-output-timeout))
		(progn
		  (message "Args timeout...")
		  (setq bdb-args-in-progress nil)
		  (setq bdb-args-string nil)))))
	;; at this point bdb-args-string is the string of all
	;; the args, we have to parse in order to find which
	;; args are still actives.
	(if (stringp bdb-args-string)
	    (bdb-args-args buffer bdb-args-string))
	(setq bdb-args-command-armed-p nil)))
  "")

;*---------------------------------------------------------------------*/
;*    bdb-info-args-call ...                                           */
;*    -------------------------------------------------------------    */
;*    Sending a bdb command.                                           */
;*---------------------------------------------------------------------*/
(defun bdb-info-args-call ()
  (interactive)
  (let ((command "info bargs\n")
	(proc (get-buffer-process bdb-comint-buffer)))
    (or proc (error "Current buffer has no process"))
    ;; Arrange for the current prompt to get deleted.
    (save-excursion
      (set-buffer bdb-comint-buffer)
      (goto-char (process-mark proc))
      (while (not (looking-at comint-prompt-regexp))
	(goto-char (process-mark proc))
	(beginning-of-line))
      (process-send-string proc command))))

;*---------------------------------------------------------------------*/
;*    bdb-args-filter ...                                              */
;*    -------------------------------------------------------------    */
;*    The completion process filter is installed temporarily to slurp  */
;*    the output of BDB up to the next prompt and build the completion */
;*    list.                                                            */
;*---------------------------------------------------------------------*/
(defun bdb-args-filter (string)
  (setq bdb-marker-acc "")
  (setq bdb-args-string (concat bdb-args-string string))
  (if (string-match comint-prompt-regexp string)
      (setq bdb-args-in-progress nil))
  "")

;*---------------------------------------------------------------------*/
;*    The args mode map                                                */
;*---------------------------------------------------------------------*/
(defvar bdb-args-mode-map (make-sparse-keymap))
(define-key bdb-args-mode-map bdb-mouse-binding
  'bdb-args-mode-menu)
(defvar bdb-args-mouse-map (make-sparse-keymap))
(define-key bdb-args-mouse-map [(button2)]
  'bdb-args-menu)
(define-key bdb-args-mode-map "\C-x\C-c" 'bdb-args-quit)

;*---------------------------------------------------------------------*/
;*    bdb-args-args ...                                                */
;*    -------------------------------------------------------------    */
;*    Args a new args in a separate buffer.                            */
;*---------------------------------------------------------------------*/
(defun bdb-args-args (buffer args)
  (save-excursion
    (set-buffer buffer)
    (let ((buffer-read-only nil))
      (erase-buffer)
      (insert args)
      (save-excursion
	(goto-char (point-max))
	(re-search-backward comint-prompt-regexp))
      ;; we remove the bdb prompt
      (delete-region (match-beginning 0) (match-end 0))
      ;; we insert extent in all text
      (save-excursion
	(goto-char (point-min))
	(let ((keep t))
	  (while keep
	    (beginning-of-line)
	    (let ((start (point)))
	      (if (looking-at "^[0-9]+")
		  ;; this line is a frame line
		  (let ((extent (make-extent 1 1 buffer))
			(end    (progn (end-of-line) (point))))
		    (set-extent-property extent 'mouse-face 'highlight)
		    (set-extent-property extent 'keymap bdb-args-mouse-map)
		    (set-extent-endpoints extent start end)
		    (if (< end (point-max))
			(next-line 1)
		      (setq keep nil)))
		(progn
		  (end-of-line)
		  (if (< (point) (point-max))
		      (next-line 1)
		    (setq keep nil)))))))))))
		
;*---------------------------------------------------------------------*/
;*    bdb-make-args-buffer ...                                         */
;*---------------------------------------------------------------------*/
(defun bdb-make-args-buffer (name)
  (if bdb-args-in-frame-p
      (bdb-make-args-frame-buffer name)
    (bdb-make-args-window-buffer name)))

;*---------------------------------------------------------------------*/
;*    bdb-make-args-frame-buffer ...                                   */
;*---------------------------------------------------------------------*/
(defun bdb-make-args-frame-buffer (name)
  (let ((height bdb-args-height))
    (let* ((default-frame-alist (cons 'height
				      (cons height
					    default-frame-alist)))
	   (buffer (switch-to-buffer-other-frame name)))
      (set-buffer buffer)
      (bdb-args-mode)
      (current-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-make-args-window-buffer ...                                  */
;*---------------------------------------------------------------------*/
(defun bdb-make-args-window-buffer (name)
  (set-buffer bdb-comint-buffer)
  (let* ((window (selected-window))
	 (height (window-height window)))
    (split-window window)
    (let ((buffer (switch-to-buffer-other-window name)))
      (set-buffer buffer)
      (bdb-args-mode)
      (current-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-popup-args ...                                               */
;*    -------------------------------------------------------------    */
;*    If the frame argsing a args buffer has been killed, we           */
;*    popup a fresh one.                                               */
;*---------------------------------------------------------------------*/
(defun bdb-popup-args (buffer)
  (let ((window (get-buffer-window buffer 0)))
    (if (not (windowp window))
	(bdb-make-args-buffer (buffer-name buffer)))))
	
;*---------------------------------------------------------------------*/
;*    bdb-args-mode ...                                                */
;*---------------------------------------------------------------------*/
(defun bdb-args-mode ()
  "Major mode for bdb args.

\\{bdb-args-mode-map}"
  (setq major-mode 'bdb-args-mode)
  (setq mode-name "Bdb Args")
  (setq buffer-read-only t)
  (setq comint-prompt-regexp bdb-prompt-regexp)
  (setq paragraph-start comint-prompt-regexp)
  (use-local-map bdb-args-mode-map)
  (suppress-keymap bdb-args-mode-map)
  (font-lock-mode t)
  (setq font-lock-keywords-case-fold-search t)
  (setq font-lock-keywords (list
			    (list "^\\(\\S-+\\)\\s-+=" 1 bdb-c-frame-face))))
  
;*---------------------------------------------------------------------*/
;*    bdb-args-menu ...                                                */
;*---------------------------------------------------------------------*/
(defun bdb-args-menu (event)
  (interactive "e")
  (let (selection)
    (save-excursion
      (set-buffer (window-buffer (event-window event)))
      (save-excursion
	(goto-char (event-point event))
	(message "point: %S" (point))
	(setq selection (bdb-get-args-number))))
    (popup-menu (list (concat "args " selection)
		      (vector "disable"
			      (list 'bdb-remote-call
				    (concat "disable args " selection))
			      t)
		      (vector "unargs"
			      (list 'bdb-remote-call
				    (concat "unargs " selection))
			      t)))))

;*---------------------------------------------------------------------*/
;*    bdb-get-args-number ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-get-args-number ()
  (save-excursion
    (let ((pos (re-search-backward "^[0-9]+" nil t)))
      (or (and pos (buffer-substring (match-beginning 0) (match-end 0)))
	  "0"))))

;*---------------------------------------------------------------------*/
;*    bdb-args-mode-menu ...                                           */
;*---------------------------------------------------------------------*/
(defun bdb-args-mode-menu (event)
  (interactive "e")
  (popup-menu (list "bdb args"
		    (vector (format "Set args height (current: %d)"
				    bdb-args-height)
			    'bdb-args-set-height
			    t)
		    (vector "Current height + 3"
			    '(bdb-args-set-height (+ bdb-args-height 3))
			    t)
		    (vector "Current height - 3"
			    '(bdb-args-set-height (- bdb-args-height 3))
			    t)
		    "--:shadowEtchedOut"
		    (vector "Close args frame args"
			    '(bdb-args-quit)
			    t))))

;*---------------------------------------------------------------------*/
;*    bdb-args-set-height ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-args-set-height (height)
  (interactive "nHeight: ")
  (if (> height 0)
      (progn
	(setq bdb-args-height height)
	(set-frame-height (selected-frame) bdb-args-height))))

;*---------------------------------------------------------------------*/
;*    bdb-args-start ...                                               */
;*---------------------------------------------------------------------*/
(defun bdb-args-start (framep)
  (interactive)
  (setq bdb-args-in-frame-p framep)
  (if (not (bdb-installed-hook-p 'bdb-args-hook))
      (progn
	(bdb-add-send-input-hook 'bdb-args-hook)
	(bdb-args-hook t))))

;*---------------------------------------------------------------------*/
;*    bdb-args-quit ...                                                */
;*---------------------------------------------------------------------*/
(defun bdb-args-quit ()
  (interactive)
  (bdb-delete-window-or-frame bdb-args-buffer)
  (bdb-remove-send-input-hook 'bdb-args-hook))

;*---------------------------------------------------------------------*/
;*    bdb-args-toggle ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-args-toggle ()
  (interactive)
  (if (not (bdb-installed-hook-p 'bdb-args-hook))
      (bdb-args-start t)
    (bdb-args-quit)))
