;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-balloon.el                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Aug  6 16:00:36 1998                          */
;*    Last change :  Fri Aug 14 11:29:31 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    My own self made balloon implementation. This implementation is  */
;*    much less ambition than the official balloon but it is less      */
;*    instrusive because much less resource consuming...               */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-balloon)
(require 'bdb-config)

;*---------------------------------------------------------------------*/
;*    bdb-balloon-actions ...                                          */
;*---------------------------------------------------------------------*/
(defvar bdb-balloon-actions '())

;*---------------------------------------------------------------------*/
;*    bdb-add-balloon-action ...                                       */
;*---------------------------------------------------------------------*/
(defun bdb-add-balloon-action (pred action)
  (setq bdb-balloon-actions
	(cons (cons pred action) bdb-balloon-actions)))

;*---------------------------------------------------------------------*/
;*    bdb-balloon-mouse-point ...                                      */
;*---------------------------------------------------------------------*/
(defvar bdb-balloon-mouse-point nil)
(defvar bdb-balloon-mouse-buffer nil)
(defvar bdb-balloon-old-mouse-point nil)
(defvar bdb-balloon-old-mouse-buffer nil)

;*---------------------------------------------------------------------*/
;*    bdb-balloon-get-buffer ...                                       */
;*---------------------------------------------------------------------*/
(defun bdb-balloon-get-buffer ()
  bdb-balloon-mouse-buffer)

;*---------------------------------------------------------------------*/
;*    bdb-balloon-get-point ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-balloon-get-point ()
  bdb-balloon-mouse-point)

;*---------------------------------------------------------------------*/
;*    bdb-balloon-mouse-motion-hook ...                                */
;*---------------------------------------------------------------------*/
(defun bdb-balloon-mouse-motion-hook (event)
  (if (and (eventp event) (motion-event-p event))
      (progn
	(setq bdb-balloon-mouse-point (event-closest-point event))
	(setq bdb-balloon-mouse-buffer (event-buffer event)))))

;*---------------------------------------------------------------------*/
;*    bdb-balloon-timeout ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-balloon-timeout (dummy)
  (if (not (and (eq bdb-balloon-mouse-point
		    bdb-balloon-old-mouse-point)
		(eq bdb-balloon-mouse-buffer
		    bdb-balloon-old-mouse-buffer)))
      (let ((mouse-pos (mouse-pixel-position)))
	(setq bdb-balloon-old-mouse-point bdb-balloon-mouse-point)
	(setq bdb-balloon-old-mouse-buffer bdb-balloon-mouse-buffer)
	(let ((l     bdb-balloon-actions)
	      (found nil))
	  (while (and (not found) (consp l))
	    (if (funcall (car (car l)) mouse-pos)
		(setq found (car l))
	      (setq l (cdr l))))
	  (if found
	      (funcall (cdr found)))))))

;*---------------------------------------------------------------------*/
;*    bdb-balloon-timeout-id ...                                       */
;*---------------------------------------------------------------------*/
(defvar bdb-balloon-timeout-id nil)

;*---------------------------------------------------------------------*/
;*    bdb-balloon-start ...                                            */
;*---------------------------------------------------------------------*/
(defun bdb-balloon-start ()
  (add-hook 'mode-motion-hook 'bdb-balloon-mouse-motion-hook)
  (setq bdb-balloon-timeout-id
	(add-timeout (/ (float bdb-balloon-timeout) 1000.0)
		     'bdb-balloon-timeout
		     '() 
		     (/ (float bdb-balloon-timeout) 1000.0))))

;*---------------------------------------------------------------------*/
;*    bdb-balloon-stop ...                                             */
;*---------------------------------------------------------------------*/
(defun bdb-balloon-stop ()
  (remove-hook 'mouse-motion-handler 'bdb-balloon-mouse-motion-hook)
  (if (numberp bdb-balloon-timeout-id)
      (disable-timeout bdb-balloon-timeout-id)))
