;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-breakpoint.el              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Apr 22 06:12:31 1998                          */
;*    Last change :  Sun Oct 25 07:50:50 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Bdb breakpoint handling.                                         */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-breakpoint)
(require 'comint)
(require 'etags)
(require 'bdb-autoload)
(require 'bdb)
(require 'bdb-mode)
(require 'bdb-filter)

;*---------------------------------------------------------------------*/
;*    The breakpoint glyphs                                            */
;*---------------------------------------------------------------------*/
(defvar bdb-red-bp-glyph
  (make-glyph
   "/* XPM */
static char *mini-redlight[] = {
/* width height num_colors chars_per_pixel */
\"    21    14       13            1\",
/* colors */
\". c #000000\",
\"# c #202020\",
\"a c #303060\",
\"b c #6064c8\",
\"c c #800000\",
\"d c #808080\",
\"e c #9898f8\",
\"f c None s None\",
\"g c #f80000\",
\"h c #f86430\",
\"i c #f864c8\",
\"j c #f8fc00\",
\"k c #f8fcf8\",
/* pixels */
\"ffffbbbbbbbbabdffffff\",
\"ffabfkfa.afekbaa.afff\",
\"ffabkfcgccdekaee.afff\",
\"fffbkfighg.ekbfcgffff\",
\"fffbkcfggc#ekbbbcffff\",
\"ffabkeccc#eekaaaaafff\",
\"ffabkea...aekbee.afff\",
\"fffbkhddaaaekeb.aafff\",
\"fffbkbddabbekbaaaffff\",
\"ffabkedaaaeekbaaa.fff\",
\"ffdbkb#a.#aekee.aafff\",
\"fffbkbaaaa.ekdaaabdff\",
\"fffbeeeaa.dekbaabbfff\",
\"fffffaaebbbbababbdfff\",
};")
  "The glyph to be use for red breakpoint display.")

(defvar bdb-orange-bp-glyph
  (make-glyph
   "/* XPM */
static char *mini-redlight[] = {
/* width height num_colors chars_per_pixel */
\"    21    14       13            1\",
/* colors */
\". c #000000\",
\"# c #202020\",
\"a c #303060\",
\"b c #6064c8\",
\"c c #800000\",
\"d c #808080\",
\"e c #9898f8\",
\"f c None s None\",
\"g c #f80000\",
\"h c #f86430\",
\"i c #f864c8\",
\"j c #f8fc00\",
\"k c #f8fcf8\",
/* pixels */
\"ffffbbbbbbbbabdffffff\",
\"ffabkea..#eekaaaaafff\",
\"fffbkdddaaaekeb.aafff\",
\"fffbkbddabbekbaaaffff\",
\"ffabfkaa.afekbaa.afff\",
\"ffabkfhhccdekaee.afff\",
\"fffbkfghhh.ekbe.h.fff\",
\"fffbkfgghh.ekbfhhffff\",
\"fffbkchhhc#ekbbbhffff\",
\"ffabkedaaaeekbaaa.fff\",
\"ffdbkb#a.#aekee.aafff\",
\"fffbkbaaa#.ekddaadbff\",
\"fffbeeeaa.dekbaabbfff\",
\"fffffaaebbbbababbdfff\",
};")
  "The glyph to be use for red breakpoint display.")

(defvar bdb-green-bp-glyph
  (make-glyph
   "/* XPM */
static char *mini-redlight[] = {
/* width height num_colors chars_per_pixel */
\"    21    14       13            1\",
/* colors */
\". c #000000\",
\"# c #202020\",
\"a c #303060\",
\"b c #6064c8\",
\"c c #008000\",
\"d c #808080\",
\"e c #9898f8\",
\"f c None s None\",
\"g c #00f800\",
\"h c #64f830\",
\"i c #f864c8\",
\"j c #00fc00\",
\"k c #f8fcf8\",
/* pixels */
\"ffffbbbbbbbbabdffffff\",
\"ffabkea..#eekaaaaafff\",
\"ffabkea...aekbee.afff\",
\"fffbkdddaaaekeb.aafff\",
\"fffbkbdbbaaekefaaffff\",
\"ffabkedaaaeekbaaa.fff\",
\"ffdbkb#a.#aekee.aafff\",
\"fffbkbaaaa.ekdaaabdff\",
\"ffabfafa.afekbaa.afff\",
\"ffabkfhhccdekaee.afff\",
\"fffbkhjhhh.ekbe.c.fff\",
\"fffbkhgghh.ekbfchffff\",
\"fffbkcfghc#ekbbbcffff\",
\"fffffaaebbbbababfffff\",
};")
  "The glyph to be use for green breakpoint display.")

(defvar bdb-warning-glyph
  (make-glyph
   "/* XPM */
static char *mini-redlight[] = {
/* width height num_colors chars_per_pixel */
\"    21    14       4            1\",
/* colors */
\"  c None s None\",
\"x c #ff2020\",
\"o c black\",
\". c white\",
/* pixels */
\"         xxx         \",
\"         xxx         \",
\"        xx.xx        \",
\"        xx.xx        \",
\"       xx...xx       \",
\"       xx.o.xx       \",
\"      xx..o..xx      \",
\"      xx..o..xx      \",
\"     xx...o...xx     \",
\"     xx...o...xx     \",
\"    xx....o....xx    \",
\"    xx.........xx    \",
\"   xxxxxxxxxxxxxxx   \",
\"    xxxxxxxxxxxxx    \",
};")
  "The glyph to be used for assertions.")

(make-face 'bdb-breakpoint-face)
(set-face-background 'bdb-breakpoint-face bdb-margin-color)

;*---------------------------------------------------------------------*/
;*    breakpoint keymap                                                */
;*---------------------------------------------------------------------*/
(defvar bdb-breakpoint-mouse-map (make-sparse-keymap))
(define-key bdb-breakpoint-mouse-map [(button2)]
  (function bdb-breakpoint-update-menu))

;*---------------------------------------------------------------------*/
;*    Global control variables.                                        */
;*---------------------------------------------------------------------*/
;; The completion process filter indicates when it is finished.
(defvar bdb-breakpoint-in-progress)

;; Since output may arrive in fragments we accumulate partials strings here.
(defvar bdb-breakpoint-string)

;; this variable holds a state: are we currently inquirying for breakpoints
(defvar bdb-breakpoint-command-armed-p nil)

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-hook ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-hook (input)
  (if (and (not bdb-breakpoint-command-armed-p)
	   (or (not (stringp input))
	       (string-match "^[de]\\|^cond\\|^b[bt]\\|^brea\\|^as\\|^una\\|^b?ru" input)))
      (bdb-breakpoint-command)))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-command ...                                       */
;*    -------------------------------------------------------------    */
;*    This command refresh the breakpoint tracing.                     */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-command ()
  (interactive)
  (if (not bdb-breakpoint-command-armed-p)
      (progn
	;; we wait for comint to be ready
	(bdb-wait-for-comint "bdb-breakpoint-command")
	;; we mark that breakpoint is armed not to re-enter this hook
	(setq bdb-breakpoint-command-armed-p t)
	;; Temporarily install our filter function.
	(let ((bdb-marker-filter 'bdb-breakpoint-filter))
	  ;; Issue the command to BDB.
	  (bdb-info-breakpoint-call)
	  (setq bdb-breakpoint-in-progress t)
	  (setq bdb-breakpoint-string "")
	  ;; Slurp the output.
	  (while bdb-breakpoint-in-progress
	    (accept-process-output (get-buffer-process bdb-comint-buffer))))
	;; at this point bdb-breakpoint-string is the string of all
	;; the breakpoint, we have to parse in order to find which
	;; breakpoint are still actives.
	(bdb-update-breakpoints bdb-breakpoint-string)
	(setq bdb-breakpoint-command-armed-p nil)))
  "")

;*---------------------------------------------------------------------*/
;*    bdb-info-breakpoint-call ...                                     */
;*    -------------------------------------------------------------    */
;*    Sending a bdb command.                                           */
;*---------------------------------------------------------------------*/
(defun bdb-info-breakpoint-call ()
  (interactive)
  (let ((command "info breakpoint\n")
	(proc (get-buffer-process bdb-comint-buffer)))
    (or proc (error "Current buffer has no process"))
    ;; Arrange for the current prompt to get deleted.
    (save-excursion
      (set-buffer bdb-comint-buffer)
      (goto-char (process-mark proc))
      (process-send-string proc command))))

;*---------------------------------------------------------------------*/
;*    bdb-info-assert-call ...                                         */
;*    -------------------------------------------------------------    */
;*    Sending a bdb command.                                           */
;*---------------------------------------------------------------------*/
(defun bdb-info-assert-call (num)
  (interactive)
  (let ((command (format "info assert %S\n" num))
	(proc (get-buffer-process bdb-comint-buffer)))
    (or proc (error "Current buffer has no process"))
    ;; Arrange for the current prompt to get deleted.
    (save-excursion
      (set-buffer bdb-comint-buffer)
      (goto-char (process-mark proc))
      (process-send-string proc command))))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-filter ...                                        */
;*    -------------------------------------------------------------    */
;*    The completion process filter is installed temporarily to slurp  */
;*    the output of BDB up to the next prompt and build the completion */
;*    list.                                                            */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-filter (string)
  (setq bdb-marker-acc "")
  (setq bdb-breakpoint-string (concat bdb-breakpoint-string string))
  (if (string-match comint-prompt-regexp string)
      (setq bdb-breakpoint-in-progress nil))
  "")

;*---------------------------------------------------------------------*/
;*    bdb-assert-filter ...                                            */
;*    -------------------------------------------------------------    */
;*    This function is not a gdb filter. It is used to remove          */
;*    the leading and ending marker of the info assert command. This   */
;*    function is used after bdb-breakpoint-filter has been used to    */
;*    filter out the result of comint.                                 */
;*---------------------------------------------------------------------*/
(defun bdb-assert-filter (string)
  (if (bdb-gdb-io-start-p string)
      (let ((start (match-end 0)))
	(if (bdb-gdb-io-stop-p string)
	    (let ((end (- (match-beginning 0) 1)))
	      (substring string start end))
	  string))
    string))
  
;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-regexp ...                                        */
;*    -------------------------------------------------------------    */
;*    The regexp that matches a bdb breakpoint as printed by the       */
;*    `info breakpoint' command. This regular expression matches the   */
;*    first line of the breakpoint. It does not match the line         */
;*    describing the condition nor the hit rate of the breakpoint.     */
;*---------------------------------------------------------------------*/
(defvar bdb-breakpoint-regexp
  "\\([0-9]+\\)\\s-*\\S-*\\s-*\\S-*\\s-*\\(.\\)\\s-*0x[0-9a-fA-F]+\\s-+in\\s-+[a-zA-Z0-9_]+\\s-+at\\s-+\\([^:]+\\)[:]\\([0-9]+\\)")

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-assert-regexp ...                                 */
;*    -------------------------------------------------------------    */
;*    The regexp that matches a bdb breakpoint as printed by the       */
;*    `info breakpoint' command. This matches assert breakpoints.      */
;*---------------------------------------------------------------------*/
(defvar bdb-breakpoint-assert-regexp
  "^\\s-+stop only if bdb_assert[ ]*(\"\\([^\"]+\\)\"")
  
;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-condition-regexp ...                              */
;*    -------------------------------------------------------------    */
;*    The regular expression that match a breakpoint condition as      */
;*    reported by `info breakpoint'.                                   */
;*---------------------------------------------------------------------*/
(defvar bdb-breakpoint-condition-regexp
  "^\\s-+stop only if [a-zA-Z0-9_]+[ ]*(\"\\([^\"]+\\)\"")

;*---------------------------------------------------------------------*/
;*    bdb-registered-breakpoint-list ...                               */
;*    -------------------------------------------------------------    */
;*    This variable holds the list of the bdb registered breakpoint.   */
;*    This list is sorted. That is the breakpoint of this listed are   */
;*    sorted from the older to the newer.                              */
;*---------------------------------------------------------------------*/
(defvar bdb-registered-breakpoint-list '())

;*---------------------------------------------------------------------*/
;*    bdb-update-breakpoints ...                                       */
;*---------------------------------------------------------------------*/
(defun bdb-update-breakpoints (string)
  (save-excursion
    (let ((old bdb-registered-breakpoint-list)
	  (new '()))
      ;; as long as they are breakpoint to be matched we loop
      (while (string-match bdb-breakpoint-regexp string)
	(let ((num (string-to-number
		    (substring string (match-beginning 1) (match-end 1))))
	      (enablep (eq (aref
			    (substring string
				       (match-beginning 2)
				       (match-end 2))
			    0)
			   ?y))
	      (file (substring string (match-beginning 3) (match-end 3)))
	      (line (string-to-number
		     (substring string (match-beginning 4) (match-end 4)))))
	  ;; we are done with this breakpoint
	  (setq string (substring string (match-end 0)))
	  (let* ((condp (numberp (string-match
				  bdb-breakpoint-condition-regexp string)))
		 (assertp (and condp
			       (numberp
				(string-match
				 bdb-breakpoint-assert-regexp string))))
		 (msg     (cond
			   (assertp
			    ;; we wait for comint to be ready
			    (bdb-wait-for-comint "bdb-update-breakpoints")
			    ;; we mark that breakpoint is armed not
			    ;; to re-enter this hook
			    (setq bdb-breakpoint-command-armed-p t)
			    ;; Temporarily install our filter function.
			    (let ((bdb-marker-filter 'bdb-breakpoint-filter))
			      ;; Issue the command to BDB.
			      (bdb-info-assert-call num)
			      (setq bdb-breakpoint-in-progress t)
			      (setq bdb-breakpoint-string "")
			      ;; Slurp the output.
			      (while bdb-breakpoint-in-progress
				(accept-process-output
				 (get-buffer-process bdb-comint-buffer)))
			      (bdb-assert-filter bdb-breakpoint-string)))
			   (condp
			    (substring string
				       (match-beginning 1)
				       (match-end 1)))
			   (t
			    nil)))
		 (kind    (cond
			   (assertp 'assert)
			   (condp 'cond)
			   (t nil))))
	    ;; we search in the old list the breakpoint number num. in this
	    ;; search we unregister all unreferenced breakpoint (breakpoints
	    ;; that have been deleted since the last bdb-update-breakpoints).
	    (while (and (consp old) (< (bdb-breakpoint-num (car old)) num))
	      ;; this breakpoint is not used anymore
	      (bdb-delete-breakpoint (car old))
	      (setq old (cdr old)))
	    (if (consp old)
		(let ((cur (car old)))
		  (if (= (bdb-breakpoint-num cur) num)
		      (progn
			(setq old (cdr old))
			;; we test if we have to refresh the breakpoint
			(if (or (not (eq (bdb-breakpoint-enablep cur) enablep))
				(not (eq (bdb-breakpoint-kind cur) kind)))
			    ;; this is the same breakpoint but is has changed
			    (progn
			      (bdb-delete-breakpoint cur)
			      (let ((nbp (bdb-make-breakpoint num
							      enablep
							      file
							      line
							      condp
							      assertp
							      msg)))
				(setq new (cons nbp new))))
			  ;; the breakpoint is still used
			  ;; and has not been changed
			  (setq new (cons cur new))))))
	      (let ((nbp (bdb-make-breakpoint num
					      enablep
					      file
					      line
					      condp
					      assertp
					      msg)))
		(setq new (cons nbp new)))))))
      ;; we now have to remove all the remaining breakpoints
      (while (consp old)
	(bdb-delete-breakpoint (car old))
	(setq old (cdr old)))
      ;; we are now done we just have to register the new breakpoints
      (setq bdb-registered-breakpoint-list (reverse new)))))
      
;*---------------------------------------------------------------------*/
;*    bdb-delete-breakpoint ...                                        */
;*    -------------------------------------------------------------    */
;*    We make a breakpoint to disapear from the screen.                */
;*---------------------------------------------------------------------*/
(defun bdb-delete-breakpoint (bp)
  (if bp (delete-extent (bdb-breakpoint-extent bp))))

;*---------------------------------------------------------------------*/
;*    bdb-make-breakpoint ...                                          */
;*    -------------------------------------------------------------    */
;*    a bdb breakpoint is a vector that holds the following values:    */
;*      - num     : an integer    : the breakpoint number              */
;*      - enablep : a bool        : is the breakpoint enabled          */
;*      - file    : a string      : the file name of this breakpoint   */
;*      - line    : an integer    : the line number of this breakpoint */
;*      - condp   : a bool        : is the breakpoint conditional      */
;*      - assertp : a bool        : is the breakpoint an assertion     */
;*      - msg     : a string      : the expr of the breakpoint         */
;*---------------------------------------------------------------------*/
(defun bdb-make-breakpoint (num enablep file line condp assertp msg)
  (save-excursion
    (let ((buffer (bdb-find-file file)))
      (if (bufferp buffer)
	  (let* ((extent (make-extent 1 1 buffer))
		 (bp (vector 'breakpoint
			     num
			     enablep
			     file
			     line
			     (cond (assertp 'assert) (condp 'cond) (t nil))
			     extent
			     msg)))
	    ;; we set the left margin for the bdb source buffer
	    (bdb-connect-buffer buffer)
	    (set-extent-face extent 'bdb-breakpoint-face)
	    (set-extent-begin-glyph extent (cond
					    (assertp bdb-warning-glyph)
					    ((not enablep) bdb-green-bp-glyph)
					    (condp bdb-orange-bp-glyph)
					    (t bdb-red-bp-glyph)))
	    (set-extent-begin-glyph-layout extent 'outside-margin)
	    (set-extent-priority extent 1000)
	    (set-extent-property extent 'keymap bdb-breakpoint-mouse-map)
	    (set-extent-property extent 'mouse-face 'highlight)
	    (set-extent-property extent 'bdb-breakpoint bp)
	    (set-buffer buffer)
	    (goto-line line)
	    (beginning-of-line)
	    (set-extent-endpoints extent (point) (+ 1 (point)))
	    bp)))))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-balloon-message ...                               */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-balloon-message (object)
  (if (extentp object)
      (let ((bp (extent-property object 'bdb-breakpoint)))
	(if (bdb-breakpoint-p bp)
	    (let ((str (concat (if (eq (bdb-breakpoint-kind bp) 'assert)
				   "assertion: "
				 "breakpoint: ")
			       (number-to-string (bdb-breakpoint-num bp)))))
	      (put-text-property 0 10
				 'face
				 bdb-prompt-face str)
	      str)
	  nil))
    nil))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-p ...                                             */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-p (bp)
  (and (vectorp bp) (= (length bp) 8) (eq (aref bp 0) 'breakpoint)))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-extent ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-extent (bp)
  (aref bp 6))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-kind ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-kind (bp)
  (aref bp 5))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-line ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-line (bp)
  (aref bp 4))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-file ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-file (bp)
  (aref bp 3))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-enablep ...                                       */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-enablep (bp)
  (aref bp 2))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-num ...                                           */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-num (bp)
  (aref bp 1))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-msg ...                                           */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-msg (bp)
  (aref bp 7))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-update-menu ...                                   */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-update-menu (event &rest arg)
  (interactive "e")
  (let ((extent (event-glyph-extent event)))
    (bdb-breakpoint-update-menu/extent extent)))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-update-menu/extent ...                            */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-update-menu/extent (extent)
  (if (extentp extent)
      (let ((bp (extent-property extent 'bdb-breakpoint)))
	(if (bdb-breakpoint-p bp)
	    (save-excursion
	      (popup-menu
	       (bdb-breakpoint-update-popup-menu bp
						 (extent-end-position extent)
						 (current-buffer))))))))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-update-popup-menu ...                             */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-update-popup-menu (breakpoint pos buffer)
  "The popup menu to print when the cursor hit a breakpoint glyph."
  (let* ((msg (bdb-breakpoint-msg breakpoint))
	 (kind (if (eq (bdb-breakpoint-kind breakpoint) 'assert)
		   "assertion"
		 "breakpoint"))
	 (rest (if (eq (bdb-breakpoint-kind breakpoint) 'assert)
		   (list (vector "Unassert"
				 (list 'bdb-remote-unassert-breakpoint
				       breakpoint)
				 t))
		 (cons (if (bdb-breakpoint-enablep breakpoint)
			   (vector (concat "Disable " kind)
				   (list 'bdb-remote-disable-breakpoint
					 breakpoint)
				   t)
			 (vector (concat "Enable " kind)
				 (list 'bdb-remote-enable-breakpoint
				       breakpoint)
				 t))
		       (list (vector (concat "Delete " kind)
				     (list 'bdb-remote-delete-breakpoint
					   breakpoint)
				     t))))))
    (if (stringp msg)
	(cons (format "%s:%d (%s %d)"
		      (bdb-breakpoint-file breakpoint)
		      (bdb-breakpoint-line breakpoint)
		      kind
		      (bdb-breakpoint-num breakpoint))
	      (append (bdb-unnewline msg) (cons "--:shadowEtchedIn" rest)))
      (cons (format "%s:%d (%s %d)"
		    (bdb-breakpoint-file breakpoint)
		    (bdb-breakpoint-line breakpoint)
		    kind
		    (bdb-breakpoint-num breakpoint))
	    rest))))

;*---------------------------------------------------------------------*/
;*    bdb-unnewline ...                                                */
;*    -------------------------------------------------------------    */
;*    This function takes a string that contains newline characters.   */
;*    It returns a list of string splitted on each newline.            */
;*---------------------------------------------------------------------*/
(defun bdb-unnewline (string)
  (let ((res '())
	(i    0)
	(len  (length string)))
    (while (string-match "\n" string i)
      (setq res (cons (substring string i (- (match-end 0) 1)) res))
      (setq i (match-end 0)))
    (reverse (cons (substring string i len) res))))

;*---------------------------------------------------------------------*/
;*    bdb-remote-disable-breakpoint ...                                */
;*---------------------------------------------------------------------*/
(defun bdb-remote-disable-breakpoint (bp)
  (interactive)
  (save-excursion
    (bdb-remote-call (format "disable %d" (bdb-breakpoint-num bp)))))

;*---------------------------------------------------------------------*/
;*    bdb-remote-enable-breakpoint ...                                 */
;*---------------------------------------------------------------------*/
(defun bdb-remote-enable-breakpoint (bp)
  (interactive)
  (save-excursion
    (bdb-remote-call (format "enable %d" (bdb-breakpoint-num bp)))))

;*---------------------------------------------------------------------*/
;*    bdb-remote-delete-breakpoint ...                                 */
;*---------------------------------------------------------------------*/
(defun bdb-remote-delete-breakpoint (bp)
  (interactive)
  (save-excursion
    (bdb-remote-call (format "delete %d" (bdb-breakpoint-num bp)))))

;*---------------------------------------------------------------------*/
;*    bdb-remote-unassert-breakpoint ...                               */
;*---------------------------------------------------------------------*/
(defun bdb-remote-unassert-breakpoint (bp)
  (interactive)
  (save-excursion
    (bdb-remote-call (format "unassert %d" (bdb-breakpoint-num bp)))))

;*---------------------------------------------------------------------*/
;*    bdb-breakpoint-menu ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-breakpoint-menu (event)
  (save-excursion
    (let* ((buffer (event-buffer event))
	   (file   (buffer-file-name buffer))
	   (fname  (file-name-nondirectory file))
	   (pos    (event-closest-point event))
	   (line   (bdb-line-number buffer pos)))
      (popup-menu
       (list (format "%s, line %s" fname line)
	     (vector "Set breakpoint"
		     (list 'bdb-remote-set-breakpoint "bbreak " fname line)
		     t)
	     (vector "Set temporary breakpoint"
		     (list 'bdb-remote-set-breakpoint "tbbreak " fname line)
		     t)
	     "-"
	     (vector "Continue until here"
		     (list 'bdb-remote-continue fname line)
		     t))))))

;*---------------------------------------------------------------------*/
;*    bdb-remote-set-breakpoint ...                                    */
;*---------------------------------------------------------------------*/
(defun bdb-remote-set-breakpoint (break fname line)
  (let ((cmd (concat break fname ":" (int-to-string line))))
    (bdb-remote-call cmd)))
  
;*---------------------------------------------------------------------*/
;*    bdb-remote-continue ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-remote-continue (fname line)
  (let ((cmd (concat "btbreak " fname ":" (int-to-string line))))
    (bdb-remote-call cmd)
    (bdb-remote-call "bcontinue")))

    
  
