;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bdb/emacs/bdb-complete.el            */
;*    -------------------------------------------------------------    */
;*    Author      :  Rick Sladkey <jrs@world.std.com>                  */
;*    Creation    :  Sun Apr 19 07:19:00 1998                          */
;*    Last change :  Thu May  7 08:28:03 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bdb completion.                                              */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-complete)
(require 'comint)
(require 'etags)
(require 'bdb)
(require 'bdb-mode)
(require 'bdb-filter)

;*---------------------------------------------------------------------*/
;*    Global control variables.                                        */
;*---------------------------------------------------------------------*/
;; The completion process filter indicates when it is finished.
(defvar bdb-complete-in-progress)

;; Since output may arrive in fragments we accumulate partials strings here.
(defvar bdb-complete-string)

;; We need to know how much of the completion to chop off.
(defvar bdb-complete-break)

;; The completion list is constructed by the process filter.
(defvar bdb-complete-list)

;*---------------------------------------------------------------------*/
;*    bdb-complete-command ...                                         */
;*    -------------------------------------------------------------    */
;*    This command is invoked when TAB is pressed. We here use the     */
;*    bdb completion to find what to do.                               */
;*---------------------------------------------------------------------*/
(defun bdb-complete-command ()
  "Perform completion on the BDB command preceding point.
This is implemented using the BDB `complete' command." 
  (interactive)
  (let* ((end (point))
	 (command (save-excursion
		    (beginning-of-line)
		    (and (looking-at comint-prompt-regexp)
			 (goto-char (match-end 0)))
		    (buffer-substring (point) end)))
	 command-word)
    ;; Find the word break.  This match will always succeed.
    (string-match "\\(\\`\\| \\)\\([^ ]*\\)\\'" command)
    (setq bdb-complete-break (match-beginning 2))
    (setq command-word (substring command bdb-complete-break))
    (message (format "completing %s..." command-word))
    ;; Temporarily install our filter function.
    (let ((bdb-marker-filter 'bdb-complete-filter))
      ;; Issue the command to BDB.
      (bdb-basic-call (concat "complete " command))
      (setq bdb-complete-in-progress t)
      (setq bdb-complete-string nil)
      (setq bdb-complete-list nil)
      ;; Slurp the output.
      (while bdb-complete-in-progress
	(accept-process-output (get-buffer-process bdb-comint-buffer))))
    ;; Sort the list like readline.
    (setq bdb-complete-list (sort bdb-complete-list (function string-lessp)))
    ;; Remove duplicates.
    (let ((first bdb-complete-list)
	  (second (cdr bdb-complete-list)))
      (while second
	(if (string-equal (car first) (car second))
	    (setcdr first (setq second (cdr second)))
	  (setq first second
		second (cdr second)))))
    ;; Add a trailing single quote if there is a unique completion
    ;; and it contains an odd number of unquoted single quotes.
    (and (= (length bdb-complete-list) 1)
	 (let ((str (car bdb-complete-list))
	       (pos 0)
	       (count 0))
	   (while (string-match "\\([^'\\]\\|\\\\'\\)*'" str pos)
	     (setq count (1+ count)
		   pos (match-end 0)))
	   (and (= (mod count 2) 1)
		(setq bdb-complete-list (list (concat str "'"))))))
    (let ((complete-list (mapcar 'downcase bdb-complete-list)))
      ;; Let comint handle the rest.
      (comint-dynamic-simple-complete command-word complete-list))))

;*---------------------------------------------------------------------*/
;*    bdb-complete-filter ...                                          */
;*    -------------------------------------------------------------    */
;*    The output filter for the completion.                            */
;*    The completion process filter is installed temporarily to slurp  */
;*    the output of BDB up to the next prompt and build the completion */
;*    list.                                                            */
;*---------------------------------------------------------------------*/
(defun bdb-complete-filter (string)
  (setq string (concat bdb-complete-string string))
  (while (string-match "\n" string)
    (setq bdb-complete-list
	  (cons (substring string bdb-complete-break (match-beginning 0))
		bdb-complete-list))
    (setq string (substring string (match-end 0))))
  (if (string-match comint-prompt-regexp string)
      (progn
	(setq bdb-complete-in-progress nil)
	string)
    (progn
      (setq bdb-complete-string string)
      "")))

