;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-connect.el                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Aug  6 11:25:49 1998                          */
;*    Last change :  Sun Oct 25 08:23:24 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The connection between bdb and the source buffers.               */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-connect)
(require 'bdb-config)
(require 'bdb-autoload)
(require 'bee-autoload)

;*---------------------------------------------------------------------*/
;*    bdb-connected-buffers ...                                        */
;*    -------------------------------------------------------------    */
;*    The list of connected buffers.                                   */
;*---------------------------------------------------------------------*/
(defvar bdb-connected-buffers '())

;*---------------------------------------------------------------------*/
;*    bdb-connected-buffer ...                                         */
;*    -------------------------------------------------------------    */
;*    A buffer local variable that help the implementation of the      */
;*    bdb-connected-buffer-p predicate.                                */
;*---------------------------------------------------------------------*/
(defvar bdb-connected-buffer nil)
(make-variable-buffer-local 'bdb-connected-buffer)

;*---------------------------------------------------------------------*/
;*    bdb-connected-buffer-p ...                                       */
;*    -------------------------------------------------------------    */
;*    Is the current buffer connected?                                 */
;*---------------------------------------------------------------------*/
(defun bdb-connected-buffer-p ()
  bdb-connected-buffer)

;*---------------------------------------------------------------------*/
;*    bdb-connect-buffer ...                                           */
;*    -------------------------------------------------------------    */
;*    This function connect a buffer, that is, some buffer bindings    */
;*    are changed and the margin is setup.                             */
;*---------------------------------------------------------------------*/
(defun bdb-connect-buffer (buffer)
  (interactive "BBuffer: ")
  (if (stringp buffer)
      (setq buffer (get-buffer buffer)))
  (if (buffer-live-p buffer)
      (progn
	(set-buffer buffer)
	(if (not bdb-connected-buffer)
	    (progn
	      ;; we register this buffer as connected to bdb
	      (setq bdb-connected-buffers (cons buffer bdb-connected-buffers))
	      ;; we mark this buffer as connected to bdb
	      (setq bdb-connected-buffer t)
	      ;; we draw the margin
	      (bdb-set-buffer-margin buffer)
	      ;; if the current mode is not bee we have to setup the generic
	      ;; mouse binding first
	      (if (not (eq major-mode 'bee-mode))
		  (define-key (current-global-map)
		    bdb-mouse-binding 'bee-predicate-mouse-event))
	      ;; we set bdb mouse bindings
	      (bee-add-menu '(lambda (event)
			       (and (> (window-left-margin-pixel-width
					(event-window event))
				       (event-x-pixel event))))
			    'bdb-breakpoint-menu)
	      ;; mode specific initialization
	      (cond
	       ((eq major-mode 'bee-mode)
		;; we actived Scheme source balloon help
		(bdb-bee-balloon-start)
		;; the s-expression menu when clicking on the active region
		(bee-add-region-popup-entry 'bdb-bee-source-menu-entries)
		;; we have to refresh the breakpoints
		(bdb-breakpoint-command))
	       ((eq major-mode 'c-mode)
		;; the region popup menu
		(bee-add-menu '(lambda (event)
				 (and (< (window-left-margin-pixel-width
					  (event-window event))
					 (event-x-pixel event))
				      (region-active-p)
				      (region-exists-p)))
			      'bdb-c-source-menu-entries)
		;; we actived C source balloon help
		(bdb-c-balloon-start)
		;; we have to refresh the breakpoints
		(bdb-breakpoint-command)))
	      t)))
    nil))

;*---------------------------------------------------------------------*/
;*    bdb-connect-file ...                                             */
;*---------------------------------------------------------------------*/
(defun bdb-connect-file (file)
  (interactive "fConnect to file: ")
  (let ((buffer (find-file-noselect file)))
    (if (bufferp buffer)
	(let ((pop-up-frames t))
	  (pop-to-buffer (buffer-name buffer))
	  (bdb-connect-buffer buffer)))))
  
;*---------------------------------------------------------------------*/
;*    bdb-disconnect-buffer ...                                        */
;*    -------------------------------------------------------------    */
;*    This function connect a buffer, that is, some buffer bindings    */
;*    are changed and the margin is setup.                             */
;*---------------------------------------------------------------------*/
(defun bdb-disconnect-buffer (buffer)
  (interactive "BBuffer: ")
  (if (buffer-live-p buffer)
      (progn
	(set-buffer buffer)
	(if bdb-connected-buffer
	    (progn
	      ;; we remove the bdb region popup entry
	      (cond
	       ((eq major-mode 'bee-mode)
		(bee-remove-region-popup-entry 'bdb-bee-source-menu-entries))
	       ((eq major-mode 'c-mode)
		(global-unset-key bdb-mouse-binding)))
	      ;; we mark this buffer as connected to bdb
	      (setq bdb-connected-buffer nil)
	      ;; we draw the margin
	      (bdb-unset-buffer-margin buffer)
	      ;; we set bdb mouse bindings
	      (local-unset-key [(button2)])
	      ;; we desactivate balloon hook
	      (bdb-balloon-stop))))))
	    
;*---------------------------------------------------------------------*/
;*    bdb-toggle-connect-buffer ...                                    */
;*---------------------------------------------------------------------*/
(defun bdb-toggle-connect-buffer ()
  (interactive)
  (let ((buffer (current-buffer)))
    (if (bdb-connected-buffer-p)
	(bdb-disconnect-buffer buffer)
      (bdb-connect-buffer buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-disconnect-all-buffers ...                                   */
;*    -------------------------------------------------------------    */
;*    Disconnect all currently connected buffers.                      */
;*---------------------------------------------------------------------*/
(defun bdb-disconnect-all-buffers ()
  (mapcar 'bdb-disconnect-buffer bdb-connected-buffers)
  (setq bdb-connected-buffers '()))
  
