;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-filter.el                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Apr 20 16:21:44 1998                          */
;*    Last change :  Fri Oct  9 19:22:12 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bdb output filter.                                           */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-filter)
(require 'bdb-source)
(require 'bdb-breakpoint)

;*---------------------------------------------------------------------*/
;*    Global control variables.                                        */
;*---------------------------------------------------------------------*/
(defvar bdb-filter-defer-flag nil
  "*Non-nil means don't process anything from the debugger right now.
It is saved for when this flag is not set.")

(defvar bdb-filter-pending-text nil
  "Non-nil means this is text that has been saved for later in `bdb-filter'.")

(defvar bdb-delete-prompt-marker nil)

(defvar bdb-last-frame nil)

(defvar bdb-last-last-frame nil)

(defvar bdb-marker-acc "")

(defvar bdb-marker-filter nil)

(defconst bdb-file:linespec-regexp "\\(\\S-+\\):\\([0-9]+\\)$")
(defconst bdb-file:linespec2-regexp "file \\(\\S-+\\) line \\([0-9]+\\).")
(defconst bdb-@-regexp "[(][@][^)]+[)]")
(defconst bdb-procedure-regexp "[#][<]procedure:\\([^.]+\\)[.][-]?[0-9]+[>]")

;*---------------------------------------------------------------------*/
;*    keymaps ...                                                      */
;*---------------------------------------------------------------------*/
(defvar bdb-linespec-mouse-map (make-sparse-keymap))
(define-key bdb-linespec-mouse-map [(button2)] 'bdb-extent-display-linespec)

(defvar bdb-@-expr-mouse-map (make-sparse-keymap))
(define-key bdb-@-expr-mouse-map [(button2)] 'bdb-extent-display-@-expr)

(defvar bdb-procedure-mouse-map (make-sparse-keymap))
(define-key bdb-procedure-mouse-map [(button2)] 'bdb-extent-display-procedure)

;*---------------------------------------------------------------------*/
;*    bdb-marker-regexp ...                                            */
;*---------------------------------------------------------------------*/
(defvar bdb-marker-regexp
  (concat "\032\032\\([^" path-separator "\n]*\\)" path-separator
	  "\\([0-9]*\\)" path-separator ".*\n"))

(defvar bdb-default-marker-bdb-io-p nil)

;*---------------------------------------------------------------------*/
;*    bdb-toggle-hilit-output ...                                      */
;*---------------------------------------------------------------------*/
(defun bdb-toggle-hilit-output ()
  (interactive)
  (setq bdb-hilit-io-p (not bdb-hilit-io-p)))

(defvar *string* '())
(defvar *buffer* '())
(defvar *filter* '())
(defvar *internal* '())
(defvar *filter2* '())

;*---------------------------------------------------------------------*/
;*    bdb-split-buffer ...                                             */
;*---------------------------------------------------------------------*/
(defvar bdb-split-buffer "")

;*---------------------------------------------------------------------*/
;*    bdb-split-output ...                                             */
;*---------------------------------------------------------------------*/
(defun bdb-split-output (string)
  (setq *string* (cons string *string*))
  (setq bdb-split-buffer (concat bdb-split-buffer string))
  (if (string-match bdb-prompt-eol-regexp bdb-split-buffer)
      (let ((buffer bdb-split-buffer))
	(setq *buffer* (cons bdb-split-buffer *buffer*))
	(setq bdb-split-buffer "")
	(bdb-split-output-internal buffer))
    (progn
      (message "bdb waiting for output...")
      "")))

;*---------------------------------------------------------------------*/
;*    bdb-io-state ...                                                 */
;*---------------------------------------------------------------------*/
(defvar bdb-io-state 'scheme)

;*---------------------------------------------------------------------*/
;*    bdb-gdb-io-start-p ...                                           */
;*    -------------------------------------------------------------    */
;*    Is a string a gdb IO starter?                                    */
;*---------------------------------------------------------------------*/
(defun bdb-gdb-io-start-p (string)
  (string-match "^gdb-io-start[\n]" string))

;*---------------------------------------------------------------------*/
;*    bdb-gdb-io-stop-p ...                                            */
;*---------------------------------------------------------------------*/
(defun bdb-gdb-io-stop-p (string)
  (string-match "gdb-io-stop[\n]" string))

;*---------------------------------------------------------------------*/
;*    bdb-scheme-io-text ...                                           */
;*---------------------------------------------------------------------*/
(defun bdb-scheme-io-text (string start end)
  (if (and bdb-hilit-io-p
	   (or (symbolp bdb-scheme-io-face)
	       (facep bdb-scheme-io-face)))
      (put-text-property start end 'face bdb-scheme-io-face string))
  (if (string-match comint-prompt-regexp string)
      (put-text-property (match-beginning 0) (match-end 0)
			 'face bdb-prompt-face
			 string)
    ;; we will walk thru the entire output string
    (let ((len    (length string))
	  (buffer (current-buffer))
	  (i      0))
      (while (< i len)
	(cond
	 ;; we search for a <procedure:????.--> expression
	 ((string-match bdb-procedure-regexp string i)
	  (let ((beg (match-beginning 0))
		(end (match-end 0)))
	    ;; prepare the extent in the bdb buffer
	    (put-text-property beg end 'mouse-face 'highlight string)
	    (put-text-property beg end 'keymap bdb-procedure-mouse-map string)
	    (put-text-property beg end
			       'procedure
			       (substring string
					  (match-beginning 1)
					  (match-end 1))
			       string)
	    (setq i end)))
	 (t
	  (setq i len))))))
  string)

;*---------------------------------------------------------------------*/
;*    bdb-bdb-io-text ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-bdb-io-text (string start end)
  (if (and bdb-hilit-io-p
	   (or (symbolp bdb-bdb-io-face)
	       (facep bdb-bdb-io-face)))
      (put-text-property start end 'face bdb-bdb-io-face string))
  ;; we will walk thru the entire output string
  (let ((len    (length string))
	(buffer (current-buffer))
	(i      0))
    (while (< i len)
      (cond
       ;; we search for LINE-SPEC (...:...)
       ((string-match bdb-file:linespec-regexp string i)
	(let ((beg (match-beginning 0))
	      (end (match-end 0)))
	  ;; prepare the extent in the bdb buffer
	  (put-text-property beg end 'mouse-face 'highlight string)
	  (put-text-property beg end 'keymap bdb-linespec-mouse-map string)
	  (put-text-property beg end
			     'filename
			     (substring string
					(match-beginning 1)
					(match-end 1))
			     string)
	  (put-text-property beg end
			     'linespec
			     (string-to-int (substring string
						       (match-beginning 2)
						       (match-end 2)))
			     string)
	  (setq i end)))
       ;; we search for LINE-SPEC (file ..., line ...)
       ((string-match bdb-file:linespec2-regexp string i)
	(let ((beg (match-beginning 0))
	      (end (match-end 0)))
	  ;; prepare the extent in the bdb buffer
	  (put-text-property beg end 'mouse-face 'highlight string)
	  (put-text-property beg end 'keymap bdb-linespec-mouse-map string)
	  (put-text-property beg end
			     'filename
			     (substring string
					(match-beginning 1)
					(- (match-end 1) 1))
			     string)
	  (put-text-property beg end
			     'linespec
			     (string-to-int (substring string
						       (match-beginning 2)
						       (match-end 2)))
			     string)
	  (setq i end)))
       ;; we search for a (@ id module) expression
       ((string-match bdb-@-regexp string i)
	(let ((beg (match-beginning 0))
	      (end (match-end 0)))
	  ;; prepare the extent in the bdb buffer
	  (put-text-property beg end 'mouse-face 'highlight string)
	  (put-text-property beg end 'keymap bdb-@-expr-mouse-map string)
	  (put-text-property beg end
			     '@-expr
			     (substring string beg end)
			     string)
	  (setq i end)))
       (t
	(setq i len)))))
  string)
  
;*---------------------------------------------------------------------*/
;*    bdb-split-output-internal ...                                    */
;*---------------------------------------------------------------------*/
(defun bdb-split-output-internal (string)
  (setq *internal* (cons string *internal*))
  (if (= (length string) 0)
      ""
    (let ((start-beg nil)
	  (start-end nil)
	  (stop-beg  nil)
	  (stop-end  nil))
      (if (bdb-gdb-io-start-p string)
	  (progn
	    (setq start-beg (match-beginning 0))
	    (setq start-end (match-end 0))))
      (if (bdb-gdb-io-stop-p string)
	  (progn
	    (setq stop-beg (match-beginning 0))
	    (setq stop-end (match-end 0))))
      (cond
       ((not (or start-beg stop-beg))
	;; this is a regular string, no marker at all
	(if (eq bdb-io-state 'scheme)
	    (bdb-scheme-io-text string 0 (length string))
	  (bdb-bdb-io-text string 0 (length string))))
       ((or (not start-beg)
	    (and stop-beg (< stop-beg start-beg)))
	;; we have found a stop marker, we mark until the mark
	(let ((pre-str (substring string 0 stop-beg))
	      (ante-str (substring string stop-end (length string))))
	  (setq bdb-io-state 'scheme)
	  (concat (bdb-bdb-io-text pre-str 0 stop-beg)
		  (bdb-split-output-internal ante-str))))
       (t
	;; we have found a start marker, we mark until the mark
	(setq bdb-io-state 'io)
	(let ((pre-str (substring string 0 start-beg))
	      (ante-str (substring string start-end (length string))))
	  (concat (bdb-scheme-io-text pre-str 0 start-beg)
		  (bdb-split-output-internal ante-str))))))))

;*---------------------------------------------------------------------*/
;*    bdb-default-marker-filter ...                                    */
;*---------------------------------------------------------------------*/
(defun bdb-default-marker-filter (string)
  (setq *filter* (cons string *filter*))
  (setq bdb-marker-acc (concat bdb-marker-acc string))
  (let ((output ""))
    ;; Process all the complete markers in this chunk.
    (while (string-match bdb-marker-regexp bdb-marker-acc)
      (message "bdb-default-marker-filter:match: %S" (match-beginning 0))
      ;; Extract the frame position from the marker.
      (setq bdb-last-frame
	    (cons (substring bdb-marker-acc (match-beginning 1) (match-end 1))
		  (string-to-int (substring bdb-marker-acc
					    (match-beginning 2)
					    (match-end 2)))))
      ;; Append any text before the marker to the output we're going
      ;; to return - we don't include the marker in this text.
      (setq output (concat output
			   (substring bdb-marker-acc 0 (match-beginning 0))))
      ;; Set the accumulator to the remaining text.
      (setq bdb-marker-acc (substring bdb-marker-acc (match-end 0))))
    ;; Does the remaining text look like it might end with the
    ;; beginning of another marker?  If it does, then keep it in
    ;; bdb-marker-acc until we receive the rest of it.  Since we
    ;; know the full marker regexp above failed, it's pretty simple to
    ;; test for marker starts.
    (cond
     ((string-match "\032.*\\'" bdb-marker-acc)
      ;; Everything before the potential marker start can be output.
      (setq output (concat output (substring bdb-marker-acc
					     0 (match-beginning 0))))
      ;; Everything after, we save, to combine with later input.
      (setq bdb-marker-acc (substring bdb-marker-acc (match-beginning 0))))
     (t
      (setq output (concat output bdb-marker-acc))
      (setq bdb-marker-acc "")))
    (bdb-split-output output)))

;*---------------------------------------------------------------------*/
;*    bdb-filter ...                                                   */
;*---------------------------------------------------------------------*/
(defvar *debug-proc* nil)
(defvar *debug-output* nil)
(defun bdb-filter (proc string)
  (setq *filter2* (cons string *filter2*))
  ;; Here's where the actual buffer insertion is done
  (let (output process-window)
    (if (buffer-name (process-buffer proc))
	(if bdb-filter-defer-flag
	    ;; If we can't process any text now,
	    ;; save it for later.
	    (setq bdb-filter-pending-text
		  (concat (or bdb-filter-pending-text "") string))
	  ;; If we have to ask a question during the processing,
	  ;; defer any additional text that comes from the debugger
	  ;; during that time.
	  (let ((bdb-filter-defer-flag t))
	    ;; Process now any text we previously saved up.
	    (if bdb-filter-pending-text
		(progn
		  (setq string (concat bdb-filter-pending-text string))
		  (setq bdb-filter-pending-text nil)))
	    (save-excursion
	      (set-buffer (process-buffer proc))
	      ;; If we have been so requested, delete the debugger prompt.
	      (if (marker-buffer bdb-delete-prompt-marker)
		  (progn
		    (delete-region (process-mark proc)
				   bdb-delete-prompt-marker)
		    (set-marker bdb-delete-prompt-marker nil)))
	      ;; Save the process output, checking for source file markers.
	      (setq output (funcall bdb-marker-filter string))
	      ;; Check for a filename-and-line number.
	      ;; Don't display the specified file
	      ;; unless (1) point is at or after the
	      ;; position where output appears
	      ;; and (2) this buffer is on the screen.
	      (setq process-window
		    (and bdb-last-frame
			 (>= (point) (process-mark proc))
			 (get-buffer-window (current-buffer))))
	      (setq *debug-proc* proc)
	      (setq *debug-output* output)
	      ;; Let the comint filter do the actual insertion.
	      ;; That lets us inherit various comint features.
	      (comint-output-filter proc output)))
	  ;; Put the arrow on the source line.
	  ;; This must be outside of the save-excursion
	  ;; in case the source file is our current buffer.
	  (if process-window
	      (save-selected-window
		(select-window process-window)
		(bdb-display-frame))
	    ;; We have to be in the proper buffer, (process-buffer proc),
	    ;; but not in a save-excursion, because that would restore point.
	    (let ((old-buf (current-buffer)))
	      (set-buffer (process-buffer proc))
	      (unwind-protect
		  (bdb-display-frame)
		(set-buffer old-buf))))
	  ;; If we deferred text that arrived during this processing,
	  ;; handle it now.
	  (if bdb-filter-pending-text
	      (bdb-filter proc ""))))))

;*---------------------------------------------------------------------*/
;*    bdb-display-frame ...                                            */
;*    -------------------------------------------------------------    */
;*    This function is called when a new source line is hitted and     */
;*    as to be prompted.                                               */
;*---------------------------------------------------------------------*/
(defun bdb-display-frame ()
  (interactive)
  (if (and bdb-last-frame (not (equal bdb-last-frame bdb-last-last-frame)))
      (progn
	(setq bdb-last-last-frame bdb-last-frame)
	(bdb-display-source-line (car bdb-last-frame) (cdr bdb-last-frame))
	(setq bdb-last-frame nil))))

;*---------------------------------------------------------------------*/
;*    bdb-extent-display-linespec ...                                  */
;*---------------------------------------------------------------------*/
(defun bdb-extent-display-linespec (event)
  (interactive "e")
  (let ((point  (event-point event))
	(buffer (event-buffer event)))
    (if (numberp point)
	(save-excursion
	  (let ((file (get-text-property point 'filename buffer))
		(linespec (get-text-property point 'linespec buffer)))
	    (if (and (stringp file) (numberp linespec))
		(bdb-display-source-line file linespec)))))))

;*---------------------------------------------------------------------*/
;*    bdb-extent-display-@-expr ...                                    */
;*---------------------------------------------------------------------*/
(defun bdb-extent-display-@-expr (event)
  (interactive "e")
  (let ((point  (event-point event))
	(buffer (event-buffer event)))
    (if (numberp point)
	(save-excursion
	  (let ((@-expr (get-text-property point '@-expr buffer)))
	    (if (stringp @-expr)
		(bdb-silent-remote-call (concat "binfo line " @-expr))))))))

;*---------------------------------------------------------------------*/
;*    bdb-extent-display-procedure ...                                 */
;*---------------------------------------------------------------------*/
(defun bdb-extent-display-procedure (event)
  (interactive "e")
  (let ((point  (event-point event))
	(buffer (event-buffer event)))
    (if (numberp point)
	(save-excursion
	  (let ((proc (get-text-property point 'procedure buffer)))
	    (if (stringp proc)
		(bdb-silent-remote-call (concat "info line *0x" proc))))))))
	    
	    
  
