;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-locals.el                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Apr 30 10:24:19 1998                          */
;*    Last change :  Sun Oct 25 07:51:02 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    This file implement the bdb locals printing.                     */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-locals)
(require 'bdb-config)
(require 'bdb)

;*---------------------------------------------------------------------*/
;*    Global control variables.                                        */
;*---------------------------------------------------------------------*/
;; The completion process filter indicates when it is finished.
(defvar bdb-locals-in-progress)

;; Since output may arrive in fragments we accumulate partials strings here.
(defvar bdb-locals-string)

;; this variable holds a state: are we currently inquirying for localss
(defvar bdb-locals-command-armed-p nil)

;; the buffer for C locals
(defvar bdb-locals-buffer nil)

;; which locals to locals (Scheme or C)?
(defvar bdb-locals 'scheme)

;; display in frame or window
(defvar bdb-locals-in-frame-p t)

;*---------------------------------------------------------------------*/
;*    bdb-locals-hook ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-locals-hook (input)
  (if (and (not bdb-locals-command-armed-p)
	   (or (not (stringp input))
	       ;; we invoke this hook on:
	       ;; next, step, continue, until, call, finish, run,
	       ;; return and breturn.
	       ;; it is important not to re-call the hook on `frame...'
	       (string-match "^[nscruf]\\|^b[dsrcnuf]\\|^disp" input)))
      (progn
	(if (not (bufferp bdb-locals-buffer))
	    (setq bdb-locals-buffer (bdb-make-locals-buffer "*Bdb locals*"))
	  (bdb-popup-locals bdb-locals-buffer))
	(bdb-locals-command bdb-locals-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-locals-command ...                                           */
;*    -------------------------------------------------------------    */
;*    This command refresh the locals tracing.                         */
;*---------------------------------------------------------------------*/
(defun bdb-locals-command (buffer)
  "Perform completion on the BDB command preceding point.
This is implemented using the BDB `locals' command." 
  (interactive)
  (if (not bdb-locals-command-armed-p)
      (progn
	;; we mark that locals is armed not to re-enter this hook
	(setq bdb-locals-command-armed-p t)
	;; we wait for comint to be ready
	(bdb-wait-for-comint "bdb-locals-command")
	;; Temporarily install our filter function.
	(let ((bdb-marker-filter 'bdb-locals-filter))
	  ;; Issue the command to BDB.
	  (bdb-info-locals-call)
	  (setq bdb-locals-in-progress t)
	  (setq bdb-locals-string "")
	  ;; Slurp the output.
	  (while bdb-locals-in-progress
	    (if (not (accept-process-output
		      (get-buffer-process bdb-comint-buffer)
		      bdb-wait-output-timeout))
		(progn
		  (message "Locals timeout...")
		  (setq bdb-locals-in-progress nil)
		  (setq bdb-locals-string nil)))))
	;; at this point bdb-locals-string is the string of all
	;; the locals, we have to parse in order to find which
	;; locals are still actives.
	(if (stringp bdb-locals-string)
	    (bdb-locals-locals buffer bdb-locals-string))
	(setq bdb-locals-command-armed-p nil)))
  "")

;*---------------------------------------------------------------------*/
;*    bdb-info-locals-call ...                                         */
;*    -------------------------------------------------------------    */
;*    Sending a bdb command.                                           */
;*---------------------------------------------------------------------*/
(defun bdb-info-locals-call ()
  (interactive)
  (let ((command "info blocals\n")
	(proc (get-buffer-process bdb-comint-buffer)))
    (or proc (error "Current buffer has no process"))
    ;; Arrange for the current prompt to get deleted.
    (save-excursion
      (set-buffer bdb-comint-buffer)
      (goto-char (process-mark proc))
      (while (not (looking-at comint-prompt-regexp))
	(goto-char (process-mark proc))
	(beginning-of-line))
      (process-send-string proc command))))

;*---------------------------------------------------------------------*/
;*    bdb-locals-filter ...                                            */
;*    -------------------------------------------------------------    */
;*    The completion process filter is installed temporarily to slurp  */
;*    the output of BDB up to the next prompt and build the completion */
;*    list.                                                            */
;*---------------------------------------------------------------------*/
(defun bdb-locals-filter (string)
  (setq bdb-marker-acc "")
  (setq bdb-locals-string (concat bdb-locals-string string))
  (if (string-match comint-prompt-regexp string)
      (setq bdb-locals-in-progress nil))
  "")

;*---------------------------------------------------------------------*/
;*    The locals mode map                                              */
;*---------------------------------------------------------------------*/
(defvar bdb-locals-mode-map (make-sparse-keymap))
(define-key bdb-locals-mode-map bdb-mouse-binding
  'bdb-locals-mode-menu)
(defvar bdb-locals-mouse-map (make-sparse-keymap))
(define-key bdb-locals-mouse-map [(button2)]
  'bdb-locals-menu)
(define-key bdb-locals-mode-map "\C-x\C-c" 'bdb-locals-quit)

;*---------------------------------------------------------------------*/
;*    bdb-locals-locals ...                                            */
;*    -------------------------------------------------------------    */
;*    Locals a new locals in a separate buffer.                        */
;*---------------------------------------------------------------------*/
(defun bdb-locals-locals (buffer locals)
  (save-excursion
    (set-buffer buffer)
    (let ((buffer-read-only nil))
      (erase-buffer)
      (insert locals)
      (save-excursion
	(goto-char (point-max))
	(re-search-backward comint-prompt-regexp))
      ;; we remove the bdb prompt
      (delete-region (match-beginning 0) (match-end 0))
      ;; we insert extent in all text
      (save-excursion
	(goto-char (point-min))
	(let ((keep t))
	  (while keep
	    (beginning-of-line)
	    (let ((start (point)))
	      (if (looking-at "^[0-9]+")
		  ;; this line is a frame line
		  (let ((extent (make-extent 1 1 buffer))
			(end    (progn (end-of-line) (point))))
		    (set-extent-property extent 'mouse-face 'highlight)
		    (set-extent-property extent 'keymap bdb-locals-mouse-map)
		    (set-extent-endpoints extent start end)
		    (if (< end (point-max))
			(next-line 1)
		      (setq keep nil)))
		(progn
		  (end-of-line)
		  (if (< (point) (point-max))
		      (next-line 1)
		    (setq keep nil)))))))))))
		
;*---------------------------------------------------------------------*/
;*    bdb-make-locals-buffer ...                                       */
;*---------------------------------------------------------------------*/
(defun bdb-make-locals-buffer (name)
  (if bdb-locals-in-frame-p
      (bdb-make-locals-frame-buffer name)
    (bdb-make-locals-window-buffer name)))

;*---------------------------------------------------------------------*/
;*    bdb-make-locals-frame-buffer ...                                 */
;*---------------------------------------------------------------------*/
(defun bdb-make-locals-frame-buffer (name)
  (let ((height bdb-locals-height))
    (let* ((default-frame-alist (cons 'height
				      (cons height
					    default-frame-alist)))
	   (buffer (switch-to-buffer-other-frame name)))
      (set-buffer buffer)
      (bdb-locals-mode)
      (current-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-make-locals-window-buffer ...                                */
;*---------------------------------------------------------------------*/
(defun bdb-make-locals-window-buffer (name)
  (set-buffer bdb-comint-buffer)
  (let* ((window (selected-window))
	 (height (window-height window)))
    (split-window window)
    (let ((buffer (switch-to-buffer-other-window name)))
      (set-buffer buffer)
      (bdb-locals-mode)
      (current-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-popup-locals ...                                             */
;*    -------------------------------------------------------------    */
;*    If the frame localsing a locals buffer has been killed, we       */
;*    popup a fresh one.                                               */
;*---------------------------------------------------------------------*/
(defun bdb-popup-locals (buffer)
  (let ((window (get-buffer-window buffer 0)))
    (if (not (windowp window))
	(bdb-make-locals-buffer (buffer-name buffer)))))
	
;*---------------------------------------------------------------------*/
;*    bdb-locals-mode ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-locals-mode ()
  "Major mode for bdb locals.

\\{bdb-locals-mode-map}"
  (setq major-mode 'bdb-locals-mode)
  (setq mode-name "Bdb Locals")
  (setq buffer-read-only t)
  (setq comint-prompt-regexp bdb-prompt-regexp)
  (setq paragraph-start comint-prompt-regexp)
  (use-local-map bdb-locals-mode-map)
  (suppress-keymap bdb-locals-mode-map)
  (font-lock-mode t)
  (setq font-lock-keywords-case-fold-search t)
  (setq font-lock-keywords (list
			    (list "^\\(\\S-+\\)\\s-+=" 1 bdb-c-frame-face))))
  
;*---------------------------------------------------------------------*/
;*    bdb-locals-menu ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-locals-menu (event)
  (interactive "e")
  (let (selection)
    (save-excursion
      (set-buffer (window-buffer (event-window event)))
      (save-excursion
	(goto-char (event-point event))
	(message "point: %S" (point))
	(setq selection (bdb-get-locals-number))))
    (popup-menu (list (concat "locals " selection)
		      (vector "disable"
			      (list 'bdb-remote-call
				    (concat "disable locals " selection))
			      t)
		      (vector "unlocals"
			      (list 'bdb-remote-call
				    (concat "unlocals " selection))
			      t)))))

;*---------------------------------------------------------------------*/
;*    bdb-get-locals-number ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-get-locals-number ()
  (save-excursion
    (let ((pos (re-search-backward "^[0-9]+" nil t)))
      (or (and pos (buffer-substring (match-beginning 0) (match-end 0)))
	  "0"))))

;*---------------------------------------------------------------------*/
;*    bdb-locals-mode-menu ...                                         */
;*---------------------------------------------------------------------*/
(defun bdb-locals-mode-menu (event)
  (interactive "e")
  (popup-menu (list "bdb locals"
		    (vector (format "Set locals height (current: %d)"
				    bdb-locals-height)
			    'bdb-locals-set-height
			    t)
		    (vector "Current height + 3"
			    '(bdb-locals-set-height (+ bdb-locals-height 3))
			    t)
		    (vector "Current height - 3"
			    '(bdb-locals-set-height (- bdb-locals-height 3))
			    t)
		    "--:shadowEtchedOut"
		    (vector "Close locals frame locals"
			    '(bdb-locals-quit)
			    t))))

;*---------------------------------------------------------------------*/
;*    bdb-locals-set-height ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-locals-set-height (height)
  (interactive "nHeight: ")
  (if (> height 0)
      (progn
	(setq bdb-locals-height height)
	(set-frame-height (selected-frame) bdb-locals-height))))

;*---------------------------------------------------------------------*/
;*    bdb-locals-start ...                                             */
;*---------------------------------------------------------------------*/
(defun bdb-locals-start (framep)
  (interactive)
  (setq bdb-locals-in-frame-p framep)
  (if (not (bdb-installed-hook-p 'bdb-locals-hook))
      (progn
	(bdb-add-send-input-hook 'bdb-locals-hook)
	(bdb-locals-hook t))))

;*---------------------------------------------------------------------*/
;*    bdb-locals-quit ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-locals-quit ()
  (interactive)
  (bdb-delete-window-or-frame bdb-locals-buffer)
  (bdb-remove-send-input-hook 'bdb-locals-hook))

;*---------------------------------------------------------------------*/
;*    bdb-locals-toggle ...                                            */
;*---------------------------------------------------------------------*/
(defun bdb-locals-toggle ()
  (interactive)
  (if (not (bdb-installed-hook-p 'bdb-local-hook))
      (bdb-locals-start t)
    (bdb-locals-quit)))
