;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-mode.el                    */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Sun Apr 19 15:12:44 1998                          */
;*    Last change :  Fri Aug 21 12:56:36 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bdb mode (not to be confused with the bdb function).         */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-mode)
(require 'comint)
(require 'bdb-config)
(require 'bdb-filter)
(require 'bdb-complete)
(require 'bdb-toolbar)
(require 'bdb-autoload)
(require 'bee-autoload)

;*---------------------------------------------------------------------*/
;*    bdb-history ...                                                  */
;*    -------------------------------------------------------------    */
;*    History of argument lists passed to bdb                          */
;*---------------------------------------------------------------------*/
(defvar bdb-history nil)

;*---------------------------------------------------------------------*/
;*    bdb-marker-regexp ...                                            */
;*    -------------------------------------------------------------    */
;*    This matches the lines bdb print out when invoked in --emacs     */
;*    mode.                                                            */
;*---------------------------------------------------------------------*/
(defvar bdb-marker-regexp
  (concat "\032\032\\([^" path-separator "\n]*\\)" path-separator
	  "\\([0-9]*\\)" path-separator ".*\n"))

;*---------------------------------------------------------------------*/
;*    minibuffer local map                                             */
;*---------------------------------------------------------------------*/
(defvar bdb-minibuffer-local-map nil
  "Keymap for minibuffer prompting of bdb startup command.")

(if (not bdb-minibuffer-local-map)
    (progn
      (setq bdb-minibuffer-local-map (copy-keymap minibuffer-local-map))
      (define-key
	bdb-minibuffer-local-map "\C-i" 'comint-dynamic-complete-filename)))

;*---------------------------------------------------------------------*/
;*    bdb-output-buffer ...                                            */
;*    -------------------------------------------------------------    */
;*    The buffer for process output.                                   */
;*---------------------------------------------------------------------*/
(defvar bdb-output-buffer nil)

;*---------------------------------------------------------------------*/
;*    bdb-mode ...                                                     */
;*---------------------------------------------------------------------*/
(defun bdb-mode ()
  "Major mode for running Bdb withing XEmacs.
The following command are available:

\\{comint-mode-map}."
  (interactive)
  (comint-mode)
  (setq major-mode 'bdb-mode)
  (setq mode-name "bdb")
  (setq mode-line-process '(":%s"))
  (use-local-map comint-mode-map)
  ;; local bindings
  (local-set-key "\C-i" 'bdb-complete-command)
  (local-set-key "\C-m" 'bdb-send-input)
  (local-set-key "\C-l" 'bdb-refresh)
  (local-set-key bdb-mouse-binding 'bee-predicate-mouse-event)
  ;; the popup menu
  (bee-add-menu '(lambda (event) t)
		'bdb-popup-menu)
  (bee-add-menu '(lambda (event) (and (region-active-p) (region-exists-p)))
		'bee-region-popup-menu)
  ;; the bdb prompt
  (setq comint-prompt-regexp bdb-prompt-regexp)
  (setq paragraph-start comint-prompt-regexp)
  (make-local-variable 'comint-prompt-regexp)
  (make-local-variable 'paragraph-start)
  ;; the prompt marker is defined in bdb-filter file.
  (make-local-variable 'bdb-delete-prompt-marker)
  (setq bdb-delete-prompt-marker (make-marker))
  ;; same for bdb-last-frame
  (make-variable-buffer-local 'bdb-last-frame)
  (setq bdb-last-frame nil)
  ;; and so it is for bdb-marker-acc
  (make-variable-buffer-local 'bdb-marker-acc)
  ;; the mouse shape when flying over non text
  (set-glyph-image nontext-pointer-glyph "left_ptr")
  ;; we set up kill buffer hook
  (add-hook 'kill-buffer-hook 'bdb-quit)
  ;; the toolbar
  (bdb-toolbar-init)
  ;; the modline
  (bdb-modeline-init)
  ;; we are done thus we run bdb hooks
  (run-hooks 'bdb-mode-hook))

;*---------------------------------------------------------------------*/
;*    bdb-quit ...                                                     */
;*---------------------------------------------------------------------*/
(defun bdb-quit ()
  ;; we have to disconnect all connected buffer
  (bdb-disconnect-all-buffers)
  ;; we now close all additional frames
  (if (bdb-installed-hook-p 'bdb-stack-hook)
      (bdb-stack-quit))
  (if (bdb-installed-hook-p 'bdb-args-hook)
      (bdb-args-quit))
  (if (bdb-installed-hook-p 'bdb-locals-hook)
      (bdb-locals-quit))
  (if (bdb-installed-hook-p 'bdb-display-hook)
      (bdb-display-quit))
  ;; if bdb was the only one in its frame, we close the frame
  (let* ((frame (selected-frame))
	 (wins  (get-buffer-window-list bdb-comint-buffer)))
    (while (consp wins)
      (if (null (cdr wins))
	  ;; we have to delete the whole frame
	  (progn
	    (setq wins '())
	    (delete-frame frame))
	(progn
	  (delete-window (car wins))
	  (setq wins (cdr wins)))))))

;*---------------------------------------------------------------------*/
;*    bdb-popup-menu ...                                               */
;*    ------------------------------------------------------------     */
;*    The main window bdb popup menu.                                  */
;*---------------------------------------------------------------------*/
(defun bdb-popup-menu (event)
  (interactive "e")
  (popup-menu
   (list "bdb"
	 ;; clearance
	 ["Clear line" bdb-clear-line t]
	 ["Clear window" bdb-clear-window t]
	 "--:shadowEtchedOut"
	 ;; buffer connection
	 ["Connect buffer" bdb-connect-buffer t]
	 ["Disconnect buffer" bdb-disconnect-buffer t]
	 ["Connect file" bdb-connect-file t]
	 "--:shadowEtchedOut"
	 ;; gdb hooking
	 ["Show stack"
	 bdb-stack-toggle
	 :style toggle
	 :selected (bdb-installed-hook-p 'bdb-stack-hook)]
	 ["Show args"
	 bdb-args-toggle
	 :style toggle
	 :selected (bdb-installed-hook-p 'bdb-args-hook)]
	 ["Show locals"
	 bdb-locals-toggle
	 :style toggle
	 :selected (bdb-installed-hook-p 'bdb-locals-hook)]
	 ["Show display"
	 bdb-display-toggle
	 :style toggle
	 :selected (bdb-installed-hook-p 'bdb-display-hook)]
	 ;; console logs
	 "--:shadowEtchedOut"
	 ["Console log" bdb-pop-console t]
	 ["Erase console log" bdb-erase-console t]
	 ["Recenter console log" bdb-recenter-console t])))

;*---------------------------------------------------------------------*/
;*    bdb-modeline-glyph ...                                           */
;*---------------------------------------------------------------------*/
(defvar bdb-modeline-glyph
  (let ((glyph (make-glyph "/* XPM */
static char *gnus-pointer[] = {
/* width height num_colors chars_per_pixel */
\"16 14 5 1\",
\"       c None s None\",
\".      c black\",
\"X      c gray50\",
\"o      c gray85\",
\"#      c white\",
\"      .   .     \",
\"       ...      \",
\"     X.....X    \",
\"    XX.....XX   \",
\"    ..#...X..   \",
\"   X.#.XXX...X  \",
\"  ...#..o...... \",
\"   .....o.....  \",
\"   .....o.....  \",
\"  .X....o....X. \",
\"    ....o....   \",
\"   .X...o...X.  \",
\"     X..o..X    \",
\"       XoX      \"};
")))
    (set-glyph-face glyph 'modeline-buffer-id)
    glyph))

;*---------------------------------------------------------------------*/
;*    bdb-modeline-init ...                                            */
;*    -------------------------------------------------------------    */
;*    This function change the left glyph for the modeline.            */
;*---------------------------------------------------------------------*/
(defun bdb-modeline-init ()
  (let ((extra (cons (copy-extent modeline-buffer-id-left-extent)
		     bdb-modeline-glyph)))
    (setq mode-line-buffer-identification
	  (cons extra
		(mapcar '(lambda (prop)
			   (let ((name (cdr prop)))
			     (if (string= "XEmacs%N:" (cdr prop))
				 (cons (car prop) "%N:")
			       prop)))
			mode-line-buffer-identification)))))
  

