;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-source.el                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Sun Apr 19 15:15:26 1998                          */
;*    Last change :  Sun Oct 25 07:15:04 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    This module takes in charge the hilit of the current source      */
;*    code line.                                                       */
;*    -------------------------------------------------------------    */
;*    This module implements the function `bdb-display-source-line'.   */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-source)
(require 'bdb-config)
(require 'bdb-autoload)
(require 'bee-autoload)
(require 'bdb)

;*---------------------------------------------------------------------*/
;*    The arrow glyph, extent and face.                                */
;*---------------------------------------------------------------------*/
(defvar bdb-arrow-glyph
  (make-glyph
   "/* XPM */
static char *arrow[] = {
/* width height num_colors chars_per_pixel */
\"    14    12        2            1\",
/* colors */
\". c None s None\",
\"# c #32cd32\",
/* pixels */
\"..............\",
\".........#....\",
\".........##...\",
\"#.##.#######..\",
\"#.##.########.\",
\"#.##.##########\",
\"#.##.########.\",
\"#.##.#######..\",
\".........##...\",
\".........#....\",
\"..............\",
\"..............\",
};")
  "The glyph to be use for arrow display.")

(make-face 'bdb-arrow-face)
(set-face-background 'bdb-arrow-face bdb-margin-color)

(defvar bdb-stop-glyph
  (make-glyph
   "/* XPM */
static char *arrow[] = {
/* width height num_colors chars_per_pixel */
\"    14    12        2            1\",
/* colors */
\". c None s None\",
\"# c #ff2222\",
/* pixels */
\"..............\",
\".........#....\",
\".........##...\",
\"#.##.#######..\",
\"#.##.########.\",
\"#.##.##########\",
\"#.##.########.\",
\"#.##.#######..\",
\".........##...\",
\".........#....\",
\"..............\",
\"..............\",
};")
  "The glyph to be use when the arrow reaches a breakpoint.")

(make-face 'bdb-stop-face)
(set-face-background 'bdb-stop-face bdb-margin-color)

(defvar bdb-source-line-extent nil)

;*---------------------------------------------------------------------*/
;*    arrow keymap ...                                                 */
;*    -------------------------------------------------------------    */
;*    This keymap is somekind of hack. What we want to do is that      */
;*    when the arrow is in front of a breakpoint the arrow inherit     */
;*    from the breakpoint popup menu. What we do is that, when         */
;*    displaying an arrow we check if there is a previous extent.      */
;*    If we found one, we set the variable bdb-bp-extent to that       */
;*    extent and we set the extent with a keymap that will call        */
;*    bdb-arrow-mouse-map. This last function simply call the          */
;*    breakpoint popup with the bdb-bp-extent value as parameter.      */
;*---------------------------------------------------------------------*/
(defvar bdb-arrow-mouse-map (make-sparse-keymap))
(defvar bdb-bp-extent nil)
(define-key bdb-arrow-mouse-map [(button2)]
  (function bdb-breakpoint-update-menu))

;*---------------------------------------------------------------------*/
;*    bdb-display-source-line ...                                      */
;*    -------------------------------------------------------------    */
;*    This function display the current source line.                   */
;*---------------------------------------------------------------------*/
(defun bdb-display-source-line (true-file line)
  ;; true-file is the file name to display
  ;; and line is number to highlight and make visible
  (let* ((buffer (save-excursion
		   (or (eq (current-buffer) bdb-comint-buffer)
		       (set-buffer bdb-comint-buffer))
		   (bdb-find-file true-file)))
	 (window (and buffer
		      (if bdb-display-source-in-frame-p
			  (let ((pop-up-frames t))
			    (display-buffer buffer))
			  (display-buffer buffer))))
	 (extent bdb-source-line-extent)
	 (pos))
    (if buffer
	(progn
	  ;; we set the left margin for the bdb source buffer
	  (bdb-connect-buffer buffer)
	  (if extent
	      (progn
		(not (eq (extent-object extent) buffer))
		(setq extent (delete-extent extent))))
	  (if (not extent)
	      (progn
		(setq extent (make-extent 1 1 buffer))
		(set-extent-face extent 'bdb-arrow-face)
		(set-extent-begin-glyph extent bdb-arrow-glyph)
		(set-extent-begin-glyph-layout extent 'outside-margin)
		(setq bdb-source-line-extent extent)))
	  (save-excursion
	    (set-buffer buffer)
	    (save-restriction
	      (widen)
	      (goto-line line)
	      (set-window-point window (point))
	      (setq pos (progn
			  (beginning-of-line)
			  (point)))
	      (if (consp (extent-list (current-buffer) (point) (point)))
		  (progn
		    ;; the arrow is shadowing a breakpoint
		    (setq bdb-bp-extent
			  (car (extent-list (current-buffer) (point) (point))))
		    (set-extent-face extent 'bdb-stop-face)
		    (set-extent-begin-glyph extent bdb-stop-glyph)
		    (set-extent-begin-glyph-layout extent 'outside-margin)
		    (set-extent-property extent 'mouse-face 'highlight)
		    (set-extent-property extent
					 'bdb-breakpoint
					 (extent-property bdb-bp-extent
							  'bdb-breakpoint))
		    (set-extent-property extent 'keymap bdb-arrow-mouse-map)))
	      (end-of-line)
	      (set-extent-endpoints extent pos (point)))
	    (if (or (< pos (point-min)) (> pos (point-max)))
		(progn
		  (widen)
		  (goto-char pos))))
	  (set-buffer bdb-comint-buffer)
	  (let ((height (window-height)))
	    (if (and (not bdb-display-source-in-frame-p)
		     (> height bdb-window-height)
		     (consp (window-list))
		     (consp (cdr (window-list))))
		(shrink-window (- height bdb-window-height))))
	  (set-buffer buffer)))))

;*---------------------------------------------------------------------*/
;*    bdb-scheme-source-menu ...                                       */
;*---------------------------------------------------------------------*/
(defun bdb-scheme-source-menu (event &rest arg)
  (interactive "e")
  (if (and (region-active-p) (region-exists-p))
      (let* ((buffer (event-buffer event))
	     (file   (buffer-file-name buffer)))
	(if (stringp file)
	    (let* ((beg    (region-beginning))
		   (end    (region-end))
		   (fname  (file-name-nondirectory file)))
	      (save-excursion
		(popup-menu
		 (bdb-source-popup-menu buffer fname beg end))))))))

;*---------------------------------------------------------------------*/
;*    bdb-eval-region ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-eval-region (region)
  (if (and (> (length region) 0) (not (eq (aref region 0) ?\()))
      (bdb-remote-call (format "(begin %s)" region))
    (bdb-remote-call region)))

;*---------------------------------------------------------------------*/
;*    bdb-command-region ...                                           */
;*---------------------------------------------------------------------*/
(defun bdb-command-region (command region)
  (bdb-remote-call (concat command region)))

;*---------------------------------------------------------------------*/
;*    bdb-bee-source-menu-entries ...                                  */
;*---------------------------------------------------------------------*/
(defun bdb-bee-source-menu-entries (event)
  (let* ((region (buffer-substring (region-beginning) (region-end)))
	 (buffer (event-buffer event))
	 (file   (file-name-nondirectory (buffer-file-name buffer)))
	 (line   (count-lines 1 (+ (region-beginning) 1))))
    (list (vector "(bdb) eval"
		  (list 'bdb-eval-region region)
		  t)
	  (vector "(bdb) bprint"
		  (list 'bdb-command-region "bprint " region)
		  t)
	  "-"
	  (vector "(bdb) bwhatis"
		  (list 'bdb-command-region "bwhatis " region)
		  t)
	  "-"
	  (vector "(bdb) break"
		  (list 'bdb-remote-set-breakpoint "break "
			file
			line)
		  t)
	  (vector "(bdb) temporary break"
		  (list 'bdb-remote-set-breakpoint "tbreak "
			file
			line)
		  t)
	  "--:shadowDoubleEtchedOut")))

;*---------------------------------------------------------------------*/
;*    bdb-c-source-menu-entries ...                                    */
;*    -------------------------------------------------------------    */
;*    This function is not bound to a region-menu (at the opposite     */
;*    from bdb-bee-source-menu-entries). It is directly bound to a     */
;*    mouse-predicate menu. That's the reason why it needs to          */
;*    expliclty call `popup-menu'.                                     */
;*---------------------------------------------------------------------*/
(defun bdb-c-source-menu-entries (event)
  (let* ((region (buffer-substring (region-beginning) (region-end)))
	 (buffer (event-buffer event))
	 (file   (buffer-file-name buffer))
	 (line   (count-lines 1 (+ (region-beginning) 1))))
    (popup-menu
     (cons (concat
	    "`"
	    (bee-region-excerpt (region-beginning) (region-end))
	    "'")
	   (list (vector "(bdb) print"
			 (list 'bdb-command-region "print " region)
			 t)
		 (vector "(bdb) print *"
			 (list 'bdb-command-region "print *" region)
			 t)
		 "-"
		 (vector "(bdb) whatis"
			 (list 'bdb-command-region "whatis " region)
			 t)
		 "-"
		 (vector "(bdb) break"
			 (list 'bdb-remote-set-breakpoint "break "
			       file
			       line)
			 t)
		 (vector "(bdb) temporary break"
			 (list 'bdb-remote-set-breakpoint "tbreak "
			       file
			       line)
			 t))))))

;*---------------------------------------------------------------------*/
;*    bdb-set-buffer-margin ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-set-buffer-margin (buffer)
  ;; we set the correct left margin width
  (add-spec-to-specifier left-margin-width bdb-margin-width buffer)
  ;; and the correct color
  (set-face-background 'left-margin bdb-margin-color))

;*---------------------------------------------------------------------*/
;*    bdb-unset-buffer-margin ...                                      */
;*---------------------------------------------------------------------*/
(defun bdb-unset-buffer-margin (buffer)
  ;; we set the correct left margin width
  (remove-specifier left-margin-width buffer)
  (sit-for 0))

;*---------------------------------------------------------------------*/
;*    bdb-get-buffer-margin ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-get-buffer-margin (buffer)
  (specifier-instance left-margin-width))

;*---------------------------------------------------------------------*/
;*    bdb-bee-balloon-start ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-bee-balloon-start ()
  ;; we start balloon
  (bdb-balloon-start)
  ;; the action on breakpoint (margin balloon)
  (bdb-add-balloon-action
   '(lambda (mouse-pos)
      (let ((win (car mouse-pos))
	    (x   (car (cdr mouse-pos))))
	(and (windowp win)
	     (numberp x)
	     (> (window-left-margin-pixel-width win) x))))
   '(lambda ()
      (let* ((buffer (bdb-balloon-get-buffer))
	     (pos    (bdb-balloon-get-point)))
	(if (and (bufferp buffer) (numberp pos))
	    (let* ((ex  (extent-at pos buffer 'bdb-breakpoint))
		   (msg (bdb-breakpoint-balloon-message ex)))
	      (if (stringp msg)
		  (display-message 'no-log msg))))))))

;*---------------------------------------------------------------------*/
;*    bdb-c-balloon-start ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-c-balloon-start ()
  ;; we start balloon
  (bdb-balloon-start)
  ;; the action on breakpoint (margin balloon)
  (bdb-add-balloon-action
   '(lambda (mouse-pos)
      (let ((win (car mouse-pos))
	    (x   (car (cdr mouse-pos))))
	(and (windowp win)
	     (numberp x)
	     (> (window-left-margin-pixel-width win) x))))
   '(lambda ()
      (let* ((buffer (bdb-balloon-get-buffer))
	     (pos    (bdb-balloon-get-point)))
	(if (and (bufferp buffer) (numberp pos))
	    (let* ((ex  (extent-at pos buffer 'bdb-breakpoint))
		   (msg (bdb-breakpoint-balloon-message ex)))
	      (if (stringp msg)
		  (display-message 'no-log msg))))))))


