;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-stack.el                   */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Apr 30 10:24:19 1998                          */
;*    Last change :  Sun Oct 25 07:32:54 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    This file implement the bdb stack printing.                      */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-stack)
(require 'bdb-config)
(require 'bdb)

;*---------------------------------------------------------------------*/
;*    Global control variables.                                        */
;*---------------------------------------------------------------------*/
;; The completion process filter indicates when it is finished.
(defvar bdb-stack-in-progress)

;; Since output may arrive in fragments we accumulate partials strings here.
(defvar bdb-stack-string)

;; this variable holds a state: are we currently inquirying for stacks
(defvar bdb-stack-command-armed-p nil)

;; the buffer for C stack
(defvar bdb-stack-buffer nil)

;; which stack to display (Scheme or C)?
(defvar bdb-stack 'scheme)

;; display in frame or window
(defvar bdb-stack-in-frame-p t)

;*---------------------------------------------------------------------*/
;*    bdb-stack-hook ...                                               */
;*---------------------------------------------------------------------*/
(defun bdb-stack-hook (input)
  (if (and (not bdb-stack-command-armed-p)
	   (or (not (stringp input))
	       ;; we invoke this hook on:
	       ;; next, step, continue, until, call, finish, run,
	       ;; return and breturn.
	       ;; it is important not to re-call the hook on `frame...'
	       (string-match "^[nscru]\\|^fi\\|^b[srcnuf]" input)))
      (progn
	(if (not (bufferp bdb-stack-buffer))
	    (setq bdb-stack-buffer (bdb-make-stack-buffer "*Bdb stack*"))
	  (bdb-popup-stack bdb-stack-buffer))
	(bdb-stack-command (if (eq bdb-stack 'scheme) "binfo" "info")
			   bdb-stack-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-stack-command ...                                            */
;*    -------------------------------------------------------------    */
;*    This command refresh the stack tracing.                          */
;*---------------------------------------------------------------------*/
(defun bdb-stack-command (info buffer)
  "Perform completion on the BDB command preceding point.
This is implemented using the BDB `stack' command." 
  (interactive)
  (if (not bdb-stack-command-armed-p)
      (progn
	;; we mark that stack is armed not to re-enter this hook
	(setq bdb-stack-command-armed-p t)
	;; we wait for comint to be ready
	(bdb-wait-for-comint "bdb-stack-command")
	;; Temporarily install our filter function.
	(let ((bdb-marker-filter 'bdb-stack-filter))
	  ;; Issue the command to BDB.
	  (bdb-info-stack-call info)
	  (setq bdb-stack-in-progress t)
	  (setq bdb-stack-string "")
	  ;; Slurp the output.
	  (while bdb-stack-in-progress
	    (if (not (accept-process-output
		      (get-buffer-process bdb-comint-buffer)
		      bdb-wait-output-timeout))
		;; we have exhausted wait time because we have
		;; raised the timeout
		(progn
		  (message "Stack timeout...")
		  (setq bdb-stack-in-progress nil)
		  (setq bdb-stack-string nil)))))
	;; at this point bdb-stack-string is the string of all
	;; the stack, we have to parse in order to find which
	;; stack are still actives.
	(if (stringp bdb-stack-string)
	    (bdb-display-stack buffer bdb-stack-string))
	(setq bdb-stack-command-armed-p nil)))
  "")

;*---------------------------------------------------------------------*/
;*    bdb-info-stack-call ...                                          */
;*    -------------------------------------------------------------    */
;*    Sending a bdb command.                                           */
;*---------------------------------------------------------------------*/
(defun bdb-info-stack-call (info)
  (interactive)
  (let ((command (format "%s stack %s\n"
			 info 
			 (if (numberp bdb-stack-depth)
			     (int-to-string bdb-stack-depth)
			   "")))
	(proc (get-buffer-process bdb-comint-buffer)))
    (or proc (error "Current buffer has no process"))
    ;; Arrange for the current prompt to get deleted.
    (save-excursion
      (set-buffer bdb-comint-buffer)
      (goto-char (process-mark proc))
      (while (not (looking-at comint-prompt-regexp))
	(goto-char (process-mark proc))
	(beginning-of-line))
      (process-send-string proc command))))

;*---------------------------------------------------------------------*/
;*    bdb-stack-filter ...                                             */
;*    -------------------------------------------------------------    */
;*    The completion process filter is installed temporarily to slurp  */
;*    the output of BDB up to the next prompt and build the completion */
;*    list.                                                            */
;*---------------------------------------------------------------------*/
(defun bdb-stack-filter (string)
  (setq bdb-marker-acc "")
  (setq bdb-stack-string (concat bdb-stack-string string))
  (if (string-match comint-prompt-regexp string)
      (setq bdb-stack-in-progress nil))
  "")

;*---------------------------------------------------------------------*/
;*    The stack mode map                                               */
;*---------------------------------------------------------------------*/
(defvar bdb-stack-mode-map (make-sparse-keymap))
(define-key bdb-stack-mode-map bdb-mouse-binding
  'bdb-stack-mode-menu)
(defvar bdb-stack-mouse-map (make-sparse-keymap))
(define-key bdb-stack-mouse-map [(button2)]
  'bdb-frames-select-by-mouse)
(define-key bdb-stack-mode-map "\C-x\C-c" 'bdb-stack-quit)

;*---------------------------------------------------------------------*/
;*    bdb-display-stack ...                                            */
;*    -------------------------------------------------------------    */
;*    Display a new stack in a separate buffer.                        */
;*---------------------------------------------------------------------*/
(defun bdb-display-stack (buffer stack)
  (save-excursion
    (set-buffer buffer)
    (let ((buffer-read-only nil))
      (erase-buffer)
      (insert stack)
      (save-excursion
	(goto-char (point-max))
	(re-search-backward comint-prompt-regexp))
      ;; we remove the bdb prompt
      (delete-region (match-beginning 0) (match-end 0))
      ;; we insert extent in all text
      (save-excursion
	(goto-char (point-min))
	(let ((keep t))
	  (while keep
	    (beginning-of-line)
	    (let ((start (point)))
	      (if (looking-at "^#[0-9]+")
		  ;; this line is a frame line
		  (let ((extent (make-extent 1 1 buffer))
			(end    (progn (end-of-line) (point))))
		    (set-extent-property extent 'mouse-face 'highlight)
		    (set-extent-property extent 'keymap bdb-stack-mouse-map)
		    (set-extent-endpoints extent start end)
		    (if (< end (point-max))
			(next-line 1)
		      (setq keep nil)))
		(progn
		  (end-of-line)
		  (if (< (point) (point-max))
		      (next-line 1)
		    (setq keep nil)))))))))))
		
;*---------------------------------------------------------------------*/
;*    bdb-make-stack-buffer ...                                        */
;*---------------------------------------------------------------------*/
(defun bdb-make-stack-buffer (name)
  (if bdb-stack-in-frame-p
      (bdb-make-stack-frame-buffer name)
    (bdb-make-stack-window-buffer name)))

;*---------------------------------------------------------------------*/
;*    bdb-make-stack-frame-buffer ...                                  */
;*---------------------------------------------------------------------*/
(defun bdb-make-stack-frame-buffer (name)
  (let ((height (if (numberp bdb-stack-depth)
		    (+ bdb-stack-depth 3)
		  10)))
    (let* ((default-frame-alist (cons 'height
				      (cons height
					    default-frame-alist)))
	   (buffer (switch-to-buffer-other-frame name)))
      (set-buffer buffer)
      (bdb-stack-mode)
      (current-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-make-stack-window-buffer ...                                 */
;*---------------------------------------------------------------------*/
(defun bdb-make-stack-window-buffer (name)
  (set-buffer bdb-comint-buffer)
  (let* ((window (selected-window))
	 (height (window-height window)))
    (split-window window)
    (let ((buffer (switch-to-buffer-other-window name)))
      (set-buffer buffer)
      (bdb-stack-mode)
      (current-buffer))))

;*---------------------------------------------------------------------*/
;*    bdb-popup-stack ...                                              */
;*    -------------------------------------------------------------    */
;*    If the frame/window displaying a stack buffer has been killed,   */
;*    we popup a fresh one.                                            */
;*---------------------------------------------------------------------*/
(defun bdb-popup-stack (buffer)
  (let ((window (get-buffer-window buffer 0)))
    (if (not (windowp window))
	(bdb-make-stack-buffer (buffer-name buffer)))))
	
;*---------------------------------------------------------------------*/
;*    bdb-stack-mode ...                                               */
;*---------------------------------------------------------------------*/
(defun bdb-stack-mode ()
  "Major mode for bdb frames.

\\{bdb-stack-mode-map}"
  (setq major-mode 'bdb-stack-mode)
  (setq mode-name "Bdb Stack")
  (setq buffer-read-only t)
  (setq comint-prompt-regexp bdb-prompt-regexp)
  (setq paragraph-start comint-prompt-regexp)
  (use-local-map bdb-stack-mode-map)
  (suppress-keymap bdb-stack-mode-map)
  (font-lock-mode t)
  (setq font-lock-keywords-case-fold-search t)
  (setq font-lock-keywords (list
			    (list "#[0-9]+\\s-+\\([^0(]\\S-*\\)\\s-+\""
				  1
				  bdb-scheme-frame-face)
			    (list "#[0-9]+\\s-+\\([(][@][^)]+[)]\\)\\s-+\""
				  1
				  bdb-scheme-frame-face)
			    (list "in\\s-+\\([^0]\\S-*\\)\\s-+\""
				  1
				  bdb-scheme-frame-face)
			    (list "in\\s-+\\([(][@][^)]+[)]\\)\\s-+"
				  1
				  bdb-scheme-frame-face)
			    (list "#[0-9]+\\s-+\\([^0(]\\S-*\\)\\s-+[(]"
				  1
				  bdb-c-frame-face)
			    (list "in\\s-+\\([^0(]\\S-*\\)\\s-+[(]"
				  1
				  bdb-c-frame-face)
			    (list "#[0-9]+\\s-+\\([(][@][^)]+[)]\\)\\s-+[(]"
				  1
				  bdb-c-frame-face)
			    (list "in\\s-+\\([^0(]\\S-*\\)"
				  1
				  bdb-c-frame-face))))
  
;*---------------------------------------------------------------------*/
;*    bdb-frames-select-by-mouse ...                                   */
;*---------------------------------------------------------------------*/
(defun bdb-frames-select-by-mouse (event)
  (interactive "e")
  (let (selection)
    (save-excursion
      (set-buffer (window-buffer (event-window event)))
      (save-excursion
	(goto-char (event-point event))
	(setq selection (bdb-get-frame-number))))
    (select-window (event-window event))
    (bdb-remote-call (format "frame %d" selection))))

;*---------------------------------------------------------------------*/
;*    bdb-get-frame-number ...                                         */
;*---------------------------------------------------------------------*/
(defun bdb-get-frame-number ()
  (save-excursion
    (let ((pos (re-search-backward "^#\\([0-9]\\)" nil t)))
      (or (and pos
	       (string-to-int
		(buffer-substring (match-beginning 1) (match-end 1))))
	  0))))

;*---------------------------------------------------------------------*/
;*    bdb-stack-mode-menu ...                                          */
;*---------------------------------------------------------------------*/
(defun bdb-stack-mode-menu (event)
  (interactive "e")
  (popup-menu (list "bdb stack"
		    (if (eq bdb-stack 'scheme)
			(vector "Show C frames"
				'(progn (setq bdb-stack 'c)
					(bdb-stack-hook t))
				t)
		      (vector "Hidden C frames"
			      '(progn (setq bdb-stack 'scheme)
				      (bdb-stack-hook t))
			      t))
		    "--:shadowEtchedOut"
		    (vector "Close stack frame display"
			    '(bdb-stack-quit)
			    t))))

;*---------------------------------------------------------------------*/
;*    bdb-stack-start ...                                              */
;*---------------------------------------------------------------------*/
(defun bdb-stack-start (framep)
  (interactive)
  (setq bdb-stack-in-frame-p framep)
  (if (not (bdb-installed-hook-p 'bdb-stack-hook))
      (progn
	(bdb-add-send-input-hook 'bdb-stack-hook)
	(bdb-stack-hook t))))

;*---------------------------------------------------------------------*/
;*    bdb-stack-quit ...                                               */
;*---------------------------------------------------------------------*/
(defun bdb-stack-quit ()
  (interactive)
  (bdb-delete-window-or-frame bdb-stack-buffer)
  (bdb-remove-send-input-hook 'bdb-stack-hook))

;*---------------------------------------------------------------------*/
;*    bdb-stack-toggle ...                                             */
;*---------------------------------------------------------------------*/
(defun bdb-stack-toggle ()
  (interactive)
  (if (not (bdb-installed-hook-p 'bdb-stack-hook))
      (bdb-stack-start t)
    (bdb-stack-quit)))
