;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bdb-toolbar.el                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Jun  5 21:03:27 1998                          */
;*    Last change :  Fri Oct 23 08:18:48 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bdb toolbar system.                                          */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bdb-toolbar)
(require 'bdb-config)
(require 'bee-autoload)
(require 'comint)

;*---------------------------------------------------------------------*/
;*    bdb toolbars ...                                                 */
;*---------------------------------------------------------------------*/
(defvar bdb-opened-toolbar
  '(;; close button
    [bdb-close-toolbar-button bdb-close-toolbar  t "bdb toolbar"]
    [:style 2d :size 2]
    
    ;; the find buttom
    [bdb-btag-button bee-tags-find t "Find definition"]
    ;; the doc buttom
    [bdb-usage-button bee-usage-find t "Definition documentation"]
    [:style 2d :size 2]
    
    ;; the file button
    [bdb-file-button (bdb-remote-call "file") t "Reload file"]
    [:style 2d :size 2]
    
    ;; the stop button
    [bdb-stop-button comint-interrupt-subjob t "Stop execution"]
    [:style 2d :size 2]
    
    ;; the run button
    [bdb-run-button (bdb-remote-call "brun") t "Run"]
    ;; the step button
    [bdb-step-button (bdb-remote-call "bstep") t "Step"]
    ;; the continue button
    [bdb-continue-button (bdb-remote-call "bcont") t "Continue"]
    ;; the next button
    [bdb-next-button (bdb-remote-call "bnext") t "Next"]
    ;; the until button
    [bdb-until-button (bdb-remote-call "buntil") t "until"]
    ;; the finish button
    [bdb-finish-button (bdb-remote-call "bfinish") t "Finish"]
    [:style 2d :size 2]

    ;; the show button
    [bdb-show-button bdb-show t "Show value"]
    [:style 2d :size 2]
    
    ;; the bdb connect button
    [bdb-connect-button bdb-connect-buffer t "Bdb connect buffer"]
    [:style 2d :size 2]

    nil
    ;; the help button
    [:style 2d :size 2]
    [bdb-help-button describe-mode t "Describe mode"]
    ;; the info button
    [bdb-info-button bdb-docline t "The online documentation for Bdb"]))
 
;*---------------------------------------------------------------------*/
;*    bdb-closed-toolbar ...                                           */
;*---------------------------------------------------------------------*/
(defvar bdb-closed-toolbar
  '(;;open button
    [bdb-open-toolbar-button bdb-open-toolbar t "bdb toolbar"]))

;*---------------------------------------------------------------------*/
;*    Opening/closing the toolbar ...                                  */
;*---------------------------------------------------------------------*/
(defun bdb-close-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bdb-closed-toolbar-height)))
    (set-specifier default-toolbar bdb-closed-toolbar (current-buffer))
    (set-specifier default-toolbar-height new-height) 
    (bdb-shrink-frame-for-toolbar (- old-height new-height))))

(defun bdb-open-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bdb-opened-toolbar-height)))
    (set-specifier default-toolbar bdb-opened-toolbar (current-buffer))
    (set-specifier default-toolbar-height new-height)
    (bdb-shrink-frame-for-toolbar (- old-height new-height))))

;*---------------------------------------------------------------------*/
;*    bdb-shrink-frame-for-toolbar ...                                 */
;*---------------------------------------------------------------------*/
(defun bdb-shrink-frame-for-toolbar (delta)
  (let* ((frame (selected-frame))
	 (fr-height (frame-pixel-height frame)))
    (if (< delta 0)
	(while (> (- (frame-pixel-height frame) fr-height) delta)
	  (set-frame-height frame (- (frame-height frame) 1)))
      (while (< (- (frame-pixel-height frame) fr-height) delta)
	(set-frame-height frame (+ (frame-height frame) 1))))))

;*---------------------------------------------------------------------*/
;*    bdb-toolbar-visible-p ...                                        */
;*---------------------------------------------------------------------*/
(defvar bdb-toolbar-visible-p '())

;*---------------------------------------------------------------------*/
;*    bdb-toolbar-init ...                                             */
;*---------------------------------------------------------------------*/
(defun bdb-toolbar-init ()
  (set-specifier default-toolbar-visible-p t)
  (set-specifier default-toolbar bdb-opened-toolbar (current-buffer)))

;*---------------------------------------------------------------------*/
;*    Toolbar icons ...                                                */
;*---------------------------------------------------------------------*/
(defvar bdb-close-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *on[] = {
/* width height num_colors chars_per_pixel */
\"    8    30        9            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #808080\",
\"d c #c0c0c0\",
\"e c #e0e4e0\",
\"f c #f8e8c8\",
\"g c #f8fcf8\",
/* pixels */
\"eddddddd\",
\"eddddddd\",
\"e#####cd\",
\"ed#aagdd\",
\"edd#gddd\",
\"eddddddd\",
\"eddddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"eddddddd\",
};"))

(defvar bdb-file-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 12 1\",
\" 	c #F7F7F3F3F7F7\",
\".	c #B6B6B2B2B6B6\",
\"X	c #B6B6B6B6B6B6\",
\"o	c #000000001010\",
\"O	c #A6A6A2A2A6A6\",
\"+	c #969692929696\",
\"@	c #717182829696\",
\"#	c #000000000000\",
\"$	c #A6A69E9EA6A6\",
\"%	c #696971718E8E\",
\"&	c #AEAEA6A6AEAE\",
\"*	c #FFFF14144141\",
\".................................\",
\".................................\",
\".................................\",
\".@.@@@@@.@..###################..\",
\"...........###################...\",
\"...........###################...\",
\".@@@@@....##################.....\",
\"..........#################......\",
\".@@@@....###################.....\",
\".........########..#########.....\",
\"........########....#######......\",
\"........########...########......\",
\"..@@@%@@########..########.......\",
\"....&..###################.......\",
\"......########..###.#####........\",
\".%@@..########...########........\",
\".&...#######...#########.........\",
\".....########. #########.........\",
\".@@.########..#########..........\",
\"....###############...##.........\",
\"...###############.....#.####....\",
\"..........#..#....###.####**##...\",
\".....####..#..#.....#.###**#..#..\",
\"....#X#*####..#.....##..#*.#X.o..\",
\"..@.#.**#.....#.@@.#...#*.o..X#..\",
\"....#X***#...#.....#..#..#X. #...\",
\"....o.**#####.......##.##..##....\",
\".@@..#.***#....@@.@@.##.X.oX.....\",
\".....oX.***#.........#.Xo#.......\",
\".....##X.**#....@@@@@.#o.........\",
\"..@@..#oX.*#.....................\",
\".......#o##......................\",
\".................................\",
};"))

(defvar bdb-open-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *off[] = {
/* width height num_colors chars_per_pixel */
\"    27    5        8            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #c0c0c0\",
\"d c #e0e4e0\",
\"e c #f8e8c8\",
\"f c #f8fcf8\",
/* pixels */
\"ccc#ccccccfccfccfccfccfcccc\",
\"ccc##cccccc#cc#cc#cc#cc#ccc\",
\"ccc#a#cccccfccfccfccfccfccc\",
\"ccc#afcccccc.cc.cc.cc.cc.cc\",
\"ccc#fcccccccccccccccccccccc\",
};"))

(defvar bdb-help-button
  (toolbar-make-button-list
   "/* XPM */
static char *foo[] = {
/* width height num_colors chars_per_pixel */
\"    28    28        2            1\",
/* colors */
\"_ c #b0b0b0\",
\"# c #000000\",
/* pixels */
\"____________________________\",
\"____________________________\",
\"____________________________\",
\"____________________________\",
\"________##########__________\",
\"_______##_#_#_#_#_#_________\",
\"______##_#_#_#_#_#_#________\",
\"_____##_#_#######_#_#_______\",
\"_____#_#_#########_#_#______\",
\"_____##_###########_#_______\",
\"_____#_#_###_#_###_#_#______\",
\"_____##_#####_#####_#_#_____\",
\"_____#_#_###_#_###_#_#_#____\",
\"_____##_#_#_#_#####_#_#_#___\",
\"_____#_#_#_#_#####_#_####___\",
\"_____##_#_#_#####_#_#_______\",
\"______##_#_#####_#_#_#______\",
\"_______##_#_###_#_#_#_______\",
\"________##_#####_#_#________\",
\"_________##_#_#_#_#_#_______\",
\"_________#_#_#_#_#_#________\",
\"_________##_###_#___________\",
\"_________#_#####_#__________\",
\"_________##_###_#___________\",
\"________##_#_#_#_#__________\",
\"________#_#_#_#_#_#_________\",
\"____________________________\",
\"____________________________\"
};"))

(defvar bdb-info-button
  (toolbar-make-button-list
   "/* XPM */
static char * info[] = {
\"33 33 3 1\",
\"_      c Gray75 s backgroundToolBarColor\",
\"o      c #330099\",
\"#      c black\",
\"_________________________________\",
\"_______________ooo_______________\",
\"______________ooooo______________\",
\"_____________ooooooo_____________\",
\"_____________ooooooo_____________\",
\"_____________ooooooo_____________\",
\"______________ooooo______________\",
\"_______________ooo_______________\",
\"_________________________________\",
\"_____________oooooo______________\",
\"____________ooooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"_____________ooooooo_____________\",
\"____________ooooooooo____________\",
\"_________________________________\",
\"_________________________________\",
\"___________________#_____________\",
\"__________#_______#______________\",
\"__________#_###__###_##__________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#__##__________\",
\"_________________________________\",
\"_________________________________\"};"))

(defvar bdb-run-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 5 1\",
\"_	c #C8C8C8C8C8C8 s backgroundToolBarColor\",
\".	s FgColor c #000000000000\",
\"X	c #836fff\",
\"+	c #7a67ee\",
\"@	c #6959cd\",
\"_________________________________\",
\"____________________.......______\",
\"_________________________________\",
\"________.....____________________\",
\"________.X+@._______.......______\",
\"________.X+@.____________________\",
\"________.X+@.____________________\",
\"________.X+@._______.......______\",
\"________.X+@.____________________\",
\"________.X+@.____________________\",
\"________.X+@._______.......______\",
\"________.X+@.____________________\",
\"________.X+@.____________________\",
\"_____....X+@....____.......______\",
\"______.XXX++++.__________________\",
\"_______.XX++@.___________________\",
\"________.@+@._______.......______\",
\"_________.@._____________________\",
\"__________.______________________\",
\"____________________.......______\",
\"_________________________________\",
\"_________________________________\",
\"____________________.......______\",
\"_________________________________\",
\"________...______________________\",
\"________.__._____________________\",
\"________.__._.__._...____________\",
\"________...._.__._.__.___________\",
\"________.__._.__._.__.___________\",
\"________.__._.__._.__.___________\",
\"________.__.__..__.__.___________\",
\"_________________________________\",
\"_________________________________\"};")
  "*The glyph for the `run' icon.")

(defvar bdb-continue-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 6 1\",
\"_	c #C8C8C8C8C8C8 s backgroundToolBarColor\",
\".	s FgColor c #000000000000\",
\"X	c #836fff\",
\"+	c #7a67ee\",
\"@	c #6959cd\",
\"o     c #ff0000\",
\"_________________________________\",
\"_________________________________\",
\"_________.....___......._________\",
\"________.XXXX.___________________\",
\"_______.X++++.___________________\",
\"_______.X+....___......._________\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+.______......._________\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+.______......._________\",
\"_______.X+._.____________________\",
\"_______.X+._..___________________\",
\"_______.X+._.X.__________________\",
\"_______.X+...X@._____..__________\",
\"_______.X++XXX+@.___.oo._________\",
\"_______.X+++++++@._.oooo.________\",
\"________.++++++@.__.oooo.________\",
\"_________....X@.____.oo._________\",
\"____________.X.______..__________\",
\"____________..___________________\",
\"____________.____________________\",
\"_________________________________\",
\"______________________.__________\",
\"________...___________.__________\",
\"_______._____..__...__...________\",
\"_______.____.__._.__._.__________\",
\"_______.____.__._.__._.__________\",
\"_______.____.__._.__._.__________\",
\"________...__..__.__.__..._______\",
\"_________________________________\",
\"_________________________________\"};")
  "*The glyph for the `continue' icon.")

(defvar bdb-step-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 6 1\",
\"_	c #C8C8C8 s backgroundToolBarColor\",
\".	s FgColor c #000000\",
\"X	c #836fff\",
\"+	c #7a67ee\",
\"@	c #6959cd\",
\"o     c #ff0000\",
\"_________________________________\",
\"_________________________________\",
\"_________.....___......._________\",
\"________.XXXX.___________________\",
\"_______.X++++.___________________\",
\"_______.X+....___......._________\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+.______......._________\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+.______......._________\",
\"_______.X+._.____________________\",
\"_______.X+._..___________________\",
\"_______.X+._.X.__......._________\",
\"_______.X+...X@._________________\",
\"_______.X++XXX+@.________________\",
\"_______.X+++++++@.__ooooooo______\",
\"________.++++++@.________________\",
\"_________....X@._________________\",
\"____________.X._____.......______\",
\"____________..___________________\",
\"____________.____________________\",
\"_________________________________\",
\"_____________.___________________\",
\"________...__.___________________\",
\"_______._____...___..__..._______\",
\"________...__.____.__._.__.______\",
\"___________._.____...._.__.______\",
\"_______.___._.____.____.__.______\",
\"________...___...__..._..._______\",
\"_______________________._________\",
\"_______________________._________\"};")
  "*The glyph for the step icon.")

(defvar bdb-next-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 6 1\",
\"_	c #C8C8C8 s backgroundToolBarColor\",
\".	s FgColor c #000000\",
\"X	c #836fff\",
\"+	c #7a67ee\",
\"@	c #6959cd\",
\"o     c #ff0000\",
\"_________________________________\",
\"_________________________________\",
\"_________.....___________________\",
\"________.XXXX.______.......______\",
\"_______.X++++.___________________\",
\"_______.X+....___________________\",
\"_______.X+._________.......______\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+._________.......______\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+._________.......______\",
\"_______.X+._.____________________\",
\"_______.X+._..___________________\",
\"_______.X+._.X._____.......______\",
\"_______.X+...X@._________________\",
\"_______.X++XXX+@.________________\",
\"_______.X+++++++@.__ooooooo______\",
\"________.++++++@.________________\",
\"_________....X@._________________\",
\"____________.X._____.......______\",
\"____________..___________________\",
\"____________.____________________\",
\"_______________________._________\",
\"_______.___.___________._________\",
\"_______..__.__..__.__._..._______\",
\"_______._._._.__._.__._._________\",
\"_______.__.._....__..__._________\",
\"_______.___._.____.__._._________\",
\"_______.___.__..._.__.__...______\",
\"_________________________________\",
\"_________________________________\"};")
  "*The glyph for the `next' icon.")

(defvar bdb-until-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 7 1\",
\"_	c #C8C8C8 s backgroundToolBarColor\",
\".	s FgColor c #000000\",
\"-	c #777777\",
\"X	c #836fff\",
\"+	c #7a67ee\",
\"@	c #6959cd\",
\"o     c #ff0000\",
\"_________________________________\",
\"_________________________________\",
\"_________.....___________________\",
\"________.XXXX.______.......______\",
\"_______.X++++.___________________\",
\"_______.X+....___________________\",
\"_______.X+._________.......______\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+._________.......______\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+._________.......______\",
\"_______.X+._.____________________\",
\"_______.X+._..___________________\",
\"_-----_.X+._.X._----------------_\",
\"_______.X+...X@._________________\",
\"_______.X++XXX+@.________________\",
\"_______.X+++++++@.__ooooooo______\",
\"________.++++++@.________________\",
\"_________....X@._________________\",
\"____________.X._____.......______\",
\"____________..___________________\",
\"____________.____________________\",
\"_________________.______.________\",
\"______.___.______.______.________\",
\"______.___._...__...__._.________\",
\"______.___._.__._.____._.________\",
\"______.___._.__._.____._.________\",
\"______.___._.__._.____._.________\",
\"_______...._.__.__..._.__..______\",
\"_________________________________\",
\"_________________________________\"};")
  "*The glyph for the `until' icon.")

(defvar bdb-finish-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 6 1\",
\"_	c #C8C8C8 s backgroundToolBarColor\",
\".	s FgColor c #000000\",
\"X	c #836fff\",
\"+	c #7a67ee\",
\"@	c #6959cd\",
\"o     c #ff0000\",
\"_________________________________\",
\"_________________________________\",
\"_________.....___________________\",
\"________.XXXX.________.......____\",
\"_______.X++++.___________________\",
\"_______.X+....___________________\",
\"_______.X+.___________.......____\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+.___________.......____\",
\"_______.X+.______________________\",
\"_______.X+.______________________\",
\"_______.X+.___________.......____\",
\"_______.X+._.____________________\",
\"_______.X+._..___________________\",
\"_______.X+._.X._______.......____\",
\"_______.X+...X@._________________\",
\"_______.X++XXX+@.________________\",
\"_______.X+++++++@._ooooooo_______\",
\"________.++++++@.________________\",
\"_________....X@._________________\",
\"____________.X.____......._______\",
\"____________..___________________\",
\"____________.____________________\",
\"_____....______________._________\",
\"_____._________________._________\",
\"_____.____._...__._..._..._______\",
\"_____...._._.__._._.___.__.______\",
\"_____.____._.__._._..._.__.______\",
\"_____.____._.__._.___._.__.______\",
\"_____.____._.__._._..._.__.______\",
\"_________________________________\",
\"_________________________________\"};")
  "*The glyph for the `finish' icon.")

(defvar bdb-stack-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 7 1\",
\"_     c #C8C8C8 s backgroundToolBarColor\",
\"#     c #000000\",
\"X     c #836fff\",
\"o     c #6959cd\",
\"O	c #FFFF00\",
\"+	c #FFCC33\",
\"@	c #00FF00\",
\"_________________________________\",
\"________________#______#_________\",
\"__##__###_______#______#_________\",
\"_#____#__#___#__########_________\",
\"__##__###__####_#XXXXXX#_________\",
\"____#_#______#__#XXXXXX#_________\",
\"_###__#_________#XXoooo#___###___\",
\"________________#XXoooo#___###___\",
\"________________#XXoooo#___###___\",
\"________________#XXoooo#___###___\",
\"________________#XXoooo#___###___\",
\"________________#XXoooo#___###___\",
\"________________#XXoooo#___###___\",
\"________________#XXoooo#___###___\",
\"________________#XXoooo#_#######_\",
\"________________#XXoooo#__#####__\",
\"_###__###_______#XXoooo#___###___\",
\"_#____#__#___#__#XXoooo#____#____\",
\"_###__###__####_#Xooooo#_________\",
\"_#____#______#__########_________\",
\"_#____#_________#______#_________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"__________#_____________#________\",
\"_____###__#_____________#__#_____\",
\"____#_____###___##__###_#_#______\",
\"_____###__#____#__#_#___##_______\",
\"________#_#_____###_#___#_#______\",
\"____#___#_#____#__#_#___#__#_____\",
\"_____###___###__##_#_##_#___#____\",
\"_________________________________\",
\"_________________________________\"};")
  "*The glyph for the `stack' icon.")

(defvar bdb-show-button
  (toolbar-make-button-list
   "/* XPM */
static char *show[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        9            1\",
/* colors */
\". c black\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #808080\",
\"c c #9898f8\",
\"_ c none s backgroundToolBarColor\",
\"e c #f86430\",
\"f c #f8fc00\",
\"g c #f8fcf8\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_________________g_g_g___________\",
\"________________f_gfg_g_f________\",
\"_______________g_fgggfgg_g_______\",
\"______________g_ggfffgfgff_______\",
\"_______________gfffefefff_g______\",
\"______________b____fefefgg_______\",
\"______________caaaacfbfffgf______\",
\"______________acga#.cfefgf_______\",
\"_____________acgcaa#a_fffgg______\",
\"__________##_.aaa##a#_ffgf_______\",
\"_________a_#ac_a##aa#_ffgg_______\",
\"_________#acg_a#.#.#a_g_f_f______\",
\"________#cggcaa#..#abg_g_g_______\",
\"______#aggcaaa#.abbb_cbbcbc______\",
\"_____acgcaaa#.a______abbbbb______\",
\"_____#.aaa#.a____ca#a#aabbc______\",
\"_____a#.a.ab__caaa#abc___________\",
\"_____c.#.bbbbbbbc________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"___________._____________________\",
\"______...__._____________________\",
\"_____._____...___..__.___._______\",
\"______...__.__._.__._.___._______\",
\"_________._.__._.__._._._._______\",
\"_____.___._.__._.__._._._._______\",
\"______...__.__.__..___._.________\",
\"_________________________________\",
\"_________________________________\"
};")
  "The `show' button.")

(defvar bdb-args-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 8 1\",
\"       c #C8C8C8C8C8C8 s backgroundToolBarColor\",
\".      c #000000000000\",
\"X      c #CCCC9999FFFF\",
\"o      c #99996666CCCC\",
\"O	c #FFFFFFFF0000\",
\"+	c #FFFFCCCC3333\",
\"@	c #0000FFFF0000\",
\"#	c #000077770000\",
\"                                 \",
\"                .       .        \",
\"                .       .        \",
\"                .@######.        \",
\"                .@######.        \",
\"                .@######.        \",
\"                .@######.        \",
\"       ..       .@######.        \",
\"      .  .      .@######.   ...  \",
\"      ....      .@######.   ...  \",
\"      .  .      .@######.   ...  \",
\"      .  .      .@######.   ...  \",
\"                .@######.   ...  \",
\"      ...       .@######.   ...  \",
\"      .  .      .@######.   ...  \",
\"      ...       .@######.   ...  \",
\"      . .       .@######.   ...  \",
\"      .  .      .@######.   ...  \",
\"                .@######. .......\",
\"       ..       .@######.  ..... \",
\"      .         .@######.   ...  \",
\"      . ..      .@######.    .   \",
\"      .  .      .@######.        \",
\"       ..       .@######.        \",
\"                .@######.        \",
\"       ..       .@######.        \",
\"      .         .@######.        \",
\"       ..       .@######.        \",
\"         .      .       .        \",
\"      ...       .       .        \",
\"                                 \",
\"                                 \",
\"                                 \"};")
  "*The glyph for the `args' icon.")

(defvar bdb-locals-button
  (toolbar-make-button-list
   "/* XPM */
static char * file[] = {
\"33 33 8 1\",
\"       c #C8C8C8C8C8C8 s backgroundToolBarColor\",
\".      c #000000000000\",
\"X      c #CCCC9999FFFF\",
\"o      c #99996666CCCC\",
\"O	c #FFFFFFFF0000\",
\"+	c #FFFFCCCC3333\",
\"@	c #0000FFFF0000\",
\"#	c #000077770000\",
\"                                 \",
\"      .         .       .        \",
\"      .         .       .        \",
\"      .         .@######.        \",
\"      .         .@######.        \",
\"      ....      .@######.        \",
\"                .@######.        \",
\"       ..       .@######.        \",
\"      .  .      .@######.   ...  \",
\"      .  .      .@######.   ...  \",
\"      .  .      .@######.   ...  \",
\"       ..       .@######.   ...  \",
\"                .@######.   ...  \",
\"       ..       .@######.   ...  \",
\"      .         .@######.   ...  \",
\"      .         .@######.   ...  \",
\"      .         .@######.   ...  \",
\"       ..       .@######.   ...  \",
\"                .@######. .......\",
\"       ..       .@######.  ..... \",
\"      .  .      .@######.   ...  \",
\"      ....      .@######.    .   \",
\"      .  .      .@######.        \",
\"      .  .      .@######.        \",
\"                .@######.        \",
\"      .         .@######.        \",
\"      .         .@######.        \",
\"      .         .@######.        \",
\"      .         .       .        \",
\"      ....      .       .        \",
\"                                 \",
\"                                 \",
\"                                 \"};")
  "*The glyph for the `show local' icon.")

(defvar bdb-display-button
  (toolbar-make-button-list
   "/* XPM */
static char *eye[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        6            1\",
/* colors */
\"_ c #c8c8c8 s backgroundToolBarColor\",
\"# c #000000\",
\"a c #787c78\",
\"b c #d8d8d8\",
\"c c #f8fcf8\",
\". c black\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"____________####_____####________\",
\"__________##bccb##_##bccb##______\",
\"_________#bbccccbb#bccccccb#_____\",
\"________a#cccccccc#cccccccc#_____\",
\"________a#cccccccc#cccccccc#_____\",
\"________a#cccccccc#cccccccc#_____\",
\"________a#cccc###c#cccc###c#_____\",
\"________a#bbcc###b#bccc###b#_____\",
\"________aa##bccbb#a##bccbb#______\",
\"_________aaa####__aaa#####_______\",
\"__________aaaaa____aaaaa_________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"__...._____________._____________\",
\"__.___.____________._____________\",
\"__.___._._..._...__.____..__.__._\",
\"__.___._._.___.__._.___.__._.__._\",
\"__.___._._..._.__._.____..._.__._\",
\"__.___._.___._.__._.___.__.__..._\",
\"__....__._..._...__...__.._.___._\",
\"______________.______________..._\",
\"______________.__________________\"
};")
  "*The glyph for the `display local' icon.")

(defvar bdb-btag-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",                  
\"       c #b6b6b2b2b6b6 s backgroundToolBarColor\",      
\".      c #B6B6B2B2B6B6\",      
\"X      c #000000001010\",      
\"o      c #A6A6A2A2A6A6\",      
\"O      c #000000000000\",      
\"+      c #FFFFFBFBCFCF\",      
\"@      c #D7D7B6B68E8E\",      
\"#      c #86867D7D7171\",      
\"%      c #FFFF00000000\",                   
\"                                   \",
\"       OOOOOO                      \",
\"                                   \",
\"       OOOOOO                      \",
\"                                   \",
\"       OOOOOO  %%%  %   %%         \",
\"                %  % % %           \",
\"       OOOOOO   %  %%% % %         \",
\"                %  % %  %%         \",
\"       OOOOOO                      \",
\"                     O             \",
\"       OOOOOO        O             \",
\"                     O             \",
\"       OOOOOO   O    O             \",
\"               O    O              \",
\"       %%%%%% OOOOOO               \",
\"               O                   \",
\"       OOOOOO   O                  \",
\"                                   \",
\"       OOOOOO                      \",
\"                                   \",
\"       OOOOOO                      \",
\"                                   \",
\"                                   \"
\"        OOOO O         O           \",
\"        O              O           \",
\"        O    O O O   OOO           \",
\"        OOOO O OO O O  O           \",
\"        O    O O  O O  O           \",
\"        O    O O  O O  O           \",
\"        O    O O  O  OOO           \",
\"                                   \",
\"                                   \",
};")
  "*The btag button.")

(defvar bdb-usage-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",                  
\"       c #b6b6b2b2b6b6 s backgroundToolBarColor\",      
\".      c #B6B6B2B2B6B6\",      
\"X      c #000000001010\",      
\"o      c #A6A6A2A2A6A6\",      
\"O      c #000000000000\",      
\"+      c #FFFFFBFBCFCF\",      
\"@      c #D7D7B6B68E8E\",      
\"#      c #86867D7D7171\",      
\"%      c #ff0000\",                   
\"                                 \",
\"     OOOOOO       %%             \",
\"                 %%%%            \",
\"     OOOOOO      %%%%            \",
\"                  %%             \",
\"     OOOOOO                      \",
\"                %%%%%            \",
\"     OOOOOO      %%%%            \",
\"                 %%%%            \",
\"     OOOOOO      %%%%            \",
\"                 %%%%            \",
\"     OOOOOO      %%%%            \",
\"                %%%%%%           \",
\"     OOOOOO                      \",
\"                   O             \",
\"     OOOOOO        O             \",
\"                   O             \",
\"     OOOOOO   O    O             \",
\"             O    O              \",
\"     %%%%%% OOOOOO               \",
\"             O                   \",
\"     OOOOOO   O                  \",
\"                                 \",
\"                                 \",
\"          OOO                    \",
\"          O  O                   \",
\"          O  O  OO   OOO         \",
\"          O  O O  O O            \",
\"          O  O O  O O            \",
\"          O  O O  O O            \",
\"          OOO   OO   OOO         \",
\"                                 \",
\"                                 \",
};")
  "*The doc button.")

(defvar bdb-connect-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 3 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\"O	c #5949bd\",
\"#      c black\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"________________________#__OO____\",
\"_____________________#__#_OOOO___\",
\"___________#__________#_#O_OOO___\",
\"___________##__________#OOOOO____\",
\"__############_____###_OOOOO_____\",
\"__#############______#O_OOO###___\",
\"__############______O_OOOO#______\",
\"___________##______OOOOOO__#_____\",
\"___________#_______OOOOO#___#____\",
\"___________________OOOOOO#_______\",
\"___________________OOOOO_#_______\",
\"____________________OOO__#_______\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_____________________________#___\",
\"__##_________________________#___\",
\"_#____##__###__###___##___##_###_\",
\"_#___#__#_#__#_#__#_#__#_#___#___\",
\"_#___#__#_#__#_#__#_####_#___#___\",
\"_#___#__#_#__#_#__#_#____#___#___\",
\"__##__##__#__#_#__#__###__##__##_\",
\"_________________________________\",
\"_________________________________\"};")
  "*The connect button")
  
(defvar bdb-stop-button
  (toolbar-make-button-list
   "/* XPM */
static char *stop[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        4            1\",
/* colors */
\"_ c #c0c0c0 s backgroundToolBarColor\",
\". c #cc0000\",
\"a c #f0f4f0\",
\"# c black\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"____________.........____________\",
\"___________...........___________\",
\"__________.............__________\",
\"_________..............._________\",
\"________.................________\",
\"________.aaa.aaa.aaa.aaa.________\",
\"________.a....a..a.a.a.a.________\",
\"________.aaa..a..a.a.a.a.________\",
\"________...a..a..a.a.aaa.________\",
\"________.aaa..a..aaa.a...________\",
\"________.................________\",
\"_________..............._________\",
\"__________.............__________\",
\"___________...........___________\",
\"____________.........____________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_____________#___________________\",
\"________###__#___________________\",
\"_______#_____###___##__###_______\",
\"________###__#____#__#_#__#______\",
\"___________#_#____#__#_#__#______\",
\"_______#___#_#____#__#_#__#______\",
\"________###___###__##__###_______\",
\"_______________________#_________\",
\"_______________________#_________\"
};")
  "*The stop button")
  
;*---------------------------------------------------------------------*/
;*    bdb-opened-toolbar-height ...                                    */
;*---------------------------------------------------------------------*/
(defun bdb-opened-toolbar-height ()
  (+ 8 (glyph-height (car bdb-close-toolbar-button))))

;*---------------------------------------------------------------------*/
;*    bdb-closed-toolbar-height ...                                    */
;*---------------------------------------------------------------------*/
(defun bdb-closed-toolbar-height ()
  (+ 8 (glyph-height (car bdb-open-toolbar-button))))

