;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-bmake.el                   */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Sat May 30 19:00:12 1998                          */
;*    Last change :  Sun Sep 27 06:48:41 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Bee makefile generation.                                     */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-bmake)
(require 'bee-config)
(require 'bee-tools)

;*---------------------------------------------------------------------*/
;*    bee-makefile ...                                                 */
;*    -------------------------------------------------------------    */
;*    The makefile for compiling.                                      */
;*---------------------------------------------------------------------*/
(defvar bee-makefile-basename "Makefile"
  "*The basename of the Makefile file.")
(make-local-variable 'bee-makefile-basename)

(defvar bee-makefile bee-makefile-basename
  "*The Makefile absolute name for the current buffer.")
(make-local-variable 'bee-makefile)

(defvar bee-makefile-directory nil
  "*The relative directory of the current buffer Makefile.")
(make-local-variable 'bee-makefile-directory)

(defvar bee-makefile-entries nil
  "*The Makefile entries.")
(make-local-variable 'bee-makefile-entries)

(defvar bee-makefile-locked nil
  "*A flag that denotes a Makefile creation/update.")

;*---------------------------------------------------------------------*/
;*    bee-lock-makefile                                                */
;*---------------------------------------------------------------------*/
(defun bee-lock-makefile ()
  (setq bee-makefile-locked t))

;*---------------------------------------------------------------------*/
;*    bee-unlock-makefile ...                                          */
;*---------------------------------------------------------------------*/
(defun bee-unlock-makefile ()
  (setq bee-makefile-locked nil))

;*---------------------------------------------------------------------*/
;*    bee-makefile-locked-p ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-makefile-locked-p ()
  bee-makefile-locked)

;*---------------------------------------------------------------------*/
;*    bee-makefile-init ...                                            */
;*---------------------------------------------------------------------*/
(defun bee-makefile-init ()
  ;; we seek the Makefile and we setup the path to load it.
  (bee-set-makefile-path (bee-find-makefile)))
  
;*---------------------------------------------------------------------*/
;*    bee-set-makefile-path ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-set-makefile-path (makefile)
  (interactive "FMakefile: ")
  (if (stringp makefile)
      (progn
	(setq bee-makefile makefile)
	(setq bee-makefile-basename (file-name-nondirectory makefile))
	(setq bee-makefile-directory (file-name-directory
				      (expand-file-name makefile)))
	;; we setup the Makefile entries
	(bee-set-makefile-entries)
	;; we setup the compile command
	(bee-set-compile-command))))

;*---------------------------------------------------------------------*/
;*    bee-find-makefile ...                                            */
;*    -------------------------------------------------------------    */
;*    This find seek for a Makefile. If search two kind of files:      */
;*     1- a couple of file, a Makefile and a .afile, the .afile        */
;*        must contain the buffer file name.                           */
;*     2- a file called Makefile.bufname                               */
;*---------------------------------------------------------------------*/
(defun bee-find-makefile ()
  (if (stringp (buffer-file-name))
      (let* ((name (buffer-file-name))
	     (name-nondirectory (file-name-nondirectory name))
	     (name-sans-extension (file-name-sans-extension
				   name-nondirectory)))
	(if (stringp name)
	    (let ((fname name-nondirectory)
		  (dname (file-name-directory name))
		  (res   nil))
	      (while (and (not res) (stringp dname) (> (length dname) 1))
		;; we look for all bee-afile until the root file
		(let ((aname (concat dname bee-afile-name))
		      (mname (concat dname bee-makefile-name)))
		  (cond
		   ((and (file-exists-p aname)
			 (file-exists-p mname)
			 (let* ((cmd (concat "grep " fname " " aname))
				(res (exec-to-string cmd)))
			   (> (length res) 0)))
		    (setq res mname))
		   (t
		    (let ((dir (directory-file-name dname)))
		      (if (stringp dir)
			  (let ((pname (file-name-nondirectory dir)))
			    (setq fname (concat pname "/" fname))
			    (setq dname (file-name-directory dir)))
			(setq res nil)))))))
	      (if (stringp res) res "Makefile"))
	  "Makefile"))))

;*---------------------------------------------------------------------*/
;*    bee-set-makefile-entries ...                                     */
;*    -------------------------------------------------------------    */
;*    Find all the entries of a Makefile.                              */
;*---------------------------------------------------------------------*/
(defun bee-set-makefile-entries ()
  (if (and (stringp bee-makefile) (file-exists-p bee-makefile))
      (let* ((cmd     (concat "grep -E -e \""
			      bee-makefile-entry-regexp
			      ":\" " bee-makefile))
	     (entries (exec-to-string cmd))
	     (pat     (concat "^\\(" bee-makefile-entry-regexp "\\):"))
	     (index   0))
	(setq bee-makefile-entries '())
	(while (string-match pat entries index)
	  (let ((entry (substring entries
				  (match-beginning 1)
				  (match-end 1)))
		(end   (match-end 0)))
	    (if (not (string-match bee-makefile-entry-exclude-regexp entry))
		(setq bee-makefile-entries (cons entry bee-makefile-entries)))
	    (setq index (+ 1 end))))
	(setq bee-makefile-entries
	      (if bee-sort-makefile-entries
		  (sort bee-makefile-entries 'string<)
		(nreverse bee-makefile-entries))))))
 
;*---------------------------------------------------------------------*/
;*    bee-edit-makefile ...                                            */
;*---------------------------------------------------------------------*/
(defun bee-edit-makefile ()
  (interactive)
  (if (stringp bee-makefile)
      (find-file-other-frame bee-makefile)
    (error "No Makefile selected.")))
		
;*---------------------------------------------------------------------*/
;*    bee-makefile-fetch-binary-entry ...                              */
;*---------------------------------------------------------------------*/
(defun bee-makefile-fetch-binary-entry ()
  (interactive)
  (if (and (stringp bee-makefile) (file-exists-p bee-makefile))
      (let* ((cmd (format "%s -f %s -s getbinary" bee-make bee-makefile))
	     (exec (exec-to-string cmd)))
	(cond
	 ((string= exec "")
	  nil)
	 ((string-match "getbinary\\|stop" exec)
	  nil)
	 ((eq (aref exec (- (length exec) 1)) ?\n)
	  (substring exec 0 (- (length exec) 1)))
	 (t
	  exec)))))


