;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-c.el                       */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Aug 13 07:29:23 1998                          */
;*    Last change :  Thu Aug 13 08:11:40 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Small C hooking and configuration.                               */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-c)
(require 'bee-autoload)

;*---------------------------------------------------------------------*/
;*    bee-c-mode-init ...                                              */
;*---------------------------------------------------------------------*/
(defun bee-c-mode-init ()
  (add-hook 'c-mode-common-hook 'bee-c-mode-hook))

;*---------------------------------------------------------------------*/
;*    bee-c-mode-hook ...                                              */
;*---------------------------------------------------------------------*/
(defun bee-c-mode-hook ()
  (bee-c-toolbar-init))
  
;*---------------------------------------------------------------------*/
;*    bee opened toolbar ...                                           */
;*---------------------------------------------------------------------*/
(defvar bee-c-opened-toolbar
  '(;;close button
    [bee-c-close-toolbar-button bee-close-toolbar t "bee-c toolbar"]
			      
    [:style 2d :size 2]
    ;; the tag button
    [bee-c-btag-button bee-c-tags-find t "Find definition"]
    [:style 2d :size 2]

    ;; the indent button
    [bee-c-indent-button bee-c-external-indent t "Indent whole buffer"]
    [:style 2d :size 2]
    
    ;; the compile button
    [bee-compile-button bee-compile t "Compile"]
    ;; the root button
    [bee-c-root-button bee-set-root-directory t "Set Bee root directory"]
    [:style 2d :size 2]
    
    ;; the bdb button
    [bee-c-bdb-button bdb-other-frame t "Bdb other frame"]
    ;; the bdb connect button
    [bee-connect-button bdb-toggle-connect-buffer t "Bdb connect buffer"]
    [:style 2d :size 2]
    
    nil
    ;; the help button
    [:style 2d :size 2]
    [bee-c-help-button bee-c-compiler-help t
    "Summary of the compiler invokation options"]
    ;; the info button
    [bee-c-info-button bee-c-docline t
    "The online documentation for Bee-C"]))

;*---------------------------------------------------------------------*/
;*    bee-c-closed-toolbar ...                                         */
;*---------------------------------------------------------------------*/
(defvar bee-c-closed-toolbar
  '(;;open button
    [bee-c-open-toolbar-button bee-c-open-toolbar t "bee-c toolbar"]))

;*---------------------------------------------------------------------*/
;*    Opening/closing the toolbar ...                                  */
;*---------------------------------------------------------------------*/
(defun bee-c-close-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bee-c-closed-toolbar-height)))
    (set-specifier default-toolbar bee-c-closed-toolbar (current-buffer))
    (set-specifier default-toolbar-height new-height) 
    (bee-c-shrink-frame-for-toolbar (- old-height new-height))))

(defun bee-c-open-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bee-c-opened-toolbar-height)))
    (set-specifier default-toolbar bee-c-opened-toolbar (current-buffer))
    (set-specifier default-toolbar-height new-height)
    (bee-c-shrink-frame-for-toolbar (- old-height new-height))))

;*---------------------------------------------------------------------*/
;*    bee-c-shrink-frame-for-toolbar ...                               */
;*---------------------------------------------------------------------*/
(defun bee-c-shrink-frame-for-toolbar (delta)
  (let* ((frame (selected-frame))
	 (fr-height (frame-pixel-height frame)))
    (if (< delta 0)
	(while (> (- (frame-pixel-height frame) fr-height) delta)
	  (set-frame-height frame (- (frame-height frame) 1)))
      (while (< (- (frame-pixel-height frame) fr-height) delta)
	(set-frame-height frame (+ (frame-height frame) 1))))))

;*---------------------------------------------------------------------*/
;*    bee-c-toolbar-init ...                                           */
;*---------------------------------------------------------------------*/
(defun bee-c-toolbar-init ()
  (set-specifier default-toolbar-visible-p t)
  (set-specifier default-toolbar bee-c-opened-toolbar (current-buffer)))

;*---------------------------------------------------------------------*/
;*    The bee-c toolbar buttons                                        */
;*---------------------------------------------------------------------*/
(defvar bee-c-close-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *on[] = {
/* width height num_colors chars_per_pixel */
\"    8    30        9            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #808080\",
\"d c #c0c0c0\",
\"e c #e0e4e0\",
\"f c #f8e8c8\",
\"g c #f8fcf8\",
/* pixels */
\"eddddddd\",
\"eddddddd\",
\"e#####cd\",
\"ed#aagdd\",
\"edd#gddd\",
\"eddddddd\",
\"eddddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"eddddddd\",
};"))

(defvar bee-c-open-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *off[] = {
/* width height num_colors chars_per_pixel */
\"    27    5        8            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #c0c0c0\",
\"d c #e0e4e0\",
\"e c #f8e8c8\",
\"f c #f8fcf8\",
/* pixels */
\"ccc#ccccccfccfccfccfccfcccc\",
\"ccc##cccccc#cc#cc#cc#cc#ccc\",
\"ccc#a#cccccfccfccfccfccfccc\",
\"ccc#afcccccc.cc.cc.cc.cc.cc\",
\"ccc#fcccccccccccccccccccccc\",
};"))

(defvar bee-c-btag-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",                  
\"       c #b6b6b2b2b6b6 s backgroundToolBarColor\",      
\".      c #B6B6B2B2B6B6\",      
\"X      c #000000001010\",      
\"o      c #A6A6A2A2A6A6\",      
\"O      c #000000000000\",      
\"+      c #FFFFFBFBCFCF\",      
\"@      c #D7D7B6B68E8E\",      
\"#      c #86867D7D7171\",      
\"%      c #FFFF00000000\",                   
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"     OOOOOO  %%%  %   %%         \",
\"              %  % % %           \",
\"     OOOOOO   %  %%% % %         \",
\"              %  % %  %%         \",
\"     OOOOOO                      \",
\"                   O             \",
\"     OOOOOO        O             \",
\"                   O             \",
\"     OOOOOO   O    O             \",
\"             O    O              \",
\"     %%%%%% OOOOOO               \",
\"             O                   \",
\"     OOOOOO   O                  \",
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"                                 \"
\"      OOOO O         O           \",
\"      O              O           \",
\"      O    O O O   OOO           \",
\"      OOOO O OO O O  O           \",
\"      O    O O  O O  O           \",
\"      O    O O  O O  O           \",
\"      O    O O  O  OOO           \",
\"                                 \",
\"                                 \",
};")
  "*The btag button.")

(defvar bee-c-indent-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",
\"       c #c0c0c0c0c0c0 s backgroundToolBarColor\",
\".      c #B6B6B2B2B6B6\",
\"X      c #000000001010\",
\"o      c #A6A6A2A2A6A6\",
\"O      c #000000000000\",
\"_      c #FFFFFBFBCFCF\",
\"L      c #FFFF00000000\",
\"@      c #D7D7B6B68E8E\",
\"#      c #86867D7D7171\",
\"                                 \",
\"     OOOOOOOOOOOOOOOOOOO         \",
\"     O_________________OO        \",
\"     O________LL_______O@O       \",
\"     O_______L__L______O@@O      \",
\"     O_______L_________O@@@O     \",
\"     O_______L_________O@@@@O    \",
\"     O_______L__L______OOOOOO#   \",
\"     O________LL________@@@@O#   \",
\"     O__________________@@@@O#   \",
\"     O______________________O#   \",
\"     O_OOOOO_O______________O#   \",
\"     O______________________O#   \",
\"     O_OOOOOO_______________O#   \",
\"     O______________________O#   \",
\"     O_OO____OO___OO________O#   \",
\"     O______________________O#   \",
\"     O__OO___OO___OO________O#   \",
\"     O______________________O#   \",
\"     OOOO____OO__OO_________O#   \",
\"     O______________________O#   \",
\"     O__OOOO___OO___O_______O#   \",
\"     O______________________O#   \",
\"     OOOO_OO_O______________O#   \",
\"     O______________________O#   \",
\"     O_OOOO__OOO__OO________O#   \",
\"     O______________________O#   \",
\"     O_______OOOO___________O#   \",
\"     O______________________O#   \",
\"     O_OOOO___OOOO__________O#   \",
\"     O______________________O#   \",
\"     OOOOOOOOOOOOOOOOOOOOOOOO#   \",
\"      ########################   \",
\"                                 \"
};"
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",
\"       c #c0c0c0c0c0c0 s backgroundToolBarColor\",
\".      c #B6B6B2B2B6B6\",
\"X      c #000000001010\",
\"o      c #A6A6A2A2A6A6\",
\"O      c #000000000000\",
\"_      c #FFFFFBFBCFCF\",
\"L      c #FFFF00000000\",
\"@      c #D7D7B6B68E8E\",
\"#      c #86867D7D7171\",
\"                                 \",
\"     OOOOOOOOOOOOOOOOOOO         \",
\"     O_________________OO        \",
\"     O________LL_______O@O       \",
\"     O_______L__L______O@@O      \",
\"     O_______L_________O@@@O     \",
\"     O_______L_________O@@@@O    \",
\"     O_______L__L______OOOOOO#   \",
\"     O________LL________@@@@O#   \",
\"     O__________________@@@@O#   \",
\"     O______________________O#   \",
\"     O_OOOOO_O______________O#   \",
\"     O______________________O#   \",
\"     O_OOOOOO_______________O#   \",
\"     O______________________O#   \",
\"     O___OO_OOO_O___________O#   \",
\"     O______________________O#   \",
\"     O____OO_OO_OO__________O#   \",
\"     O______________________O#   \",
\"     O___OOO_OO__OO_________O#   \",
\"     O______________________O#   \",
\"     O____OOOO_OO_O_________O#   \",
\"     O______________________O#   \",
\"     O_____OOO_OO__O________O#   \",
\"     O______________________O#   \",
\"     O___OOOO_OOO_OO________O#   \",
\"     O______________________O#   \",
\"     O__OOOO________________O#   \",
\"     O______________________O#   \",
\"     O_OOOO_OOOO____________O#   \",
\"     O______________________O#   \",
\"     OOOOOOOOOOOOOOOOOOOOOOOO#   \",
\"      ########################   \",
\"                                 \"
};")
  "*The indent button.")

(defvar bee-c-compile-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 6 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\".	c #836fff\",
\"o	c #000010\",
\"+	c #000000\",
\"@	c #9898f8\",
\"#      c black\",
\"_________________________________\",
\"_________________________________\",
\"____________+++__________________\",
\"__________+++_+++________________\",
\"__________+..+..+________________\",
\"_________++.o++.o+_______________\",
\"_________o..+....+_______________\",
\"_________++..o..o+_______________\",
\"__________o..._.++++_____________\",
\"__________+o+.++++@+__+__________\",
\"____________+++@+@@@++@+_________\",
\"____________+@@@@@@@@@@++________\",
\"_____________+@@@+++@@@+_________\",
\"_____________+@@+@+_+@@+_________\",
\"___________++@@+@+___+@@++_______\",
\"___________+@@@++____+@@@+_______\",
\"___________+++@+@+___+@+++_______\",
\"_____________+@@+@+_+@@+_________\",
\"_____________+@@@+++@@@+_________\",
\"____________+@@@@@@@@@@@+________\",
\"____________++@++@@+++++_________\",
\"_____________++_+@@+__+__________\",
\"________________++++_____________\",
\"_________________________________\",
\"________________________#________\",
\"__###___________________#________\",
\"_#_____##___#_#__###__#_#___##___\",
\"_#____#__#_#_#_#_#__#_#_#__#__#__\",
\"_#____#__#_#_#_#_#__#_#_#__####__\",
\"_#____#__#_#_#_#_#__#_#_#__#_____\",
\"__##_#_##__#_#_#_###__#__##_###__\",
\"_________________#_______________\",
\"_________________#_______________\",
};\")")
  "*The compile button.")

(defvar bee-c-root-button
  (toolbar-make-button-list
   "/* XPM */
static char *dir[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        5            1\",
/* colors */
\"_ c #c0c0c0 s backgroundToolBarColor\",
\"# c black\",
\"b c #836fff\",
\"c c #080408\",
\"d c #9898f8\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________######__________\",
\"________________#dddddd#_________\",
\"_______________#dbbbbbbd#________\",
\"________#################________\",
\"_______cdddddddddddddddddc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______ccccccccccccccccccc_______\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_______###_____________#_________\",
\"_______#__#____________#_________\",
\"_______###____##___##__#_________\",
\"_______#__#__#__#_#__#_###_______\",
\"_______#___#_#__#_#__#_#_________\",
\"_______#___#_#__#_#__#_#_________\",
\"_______#___#__##___##___##_______\",
\"_________________________________\",
\"_________________________________\"
};")
   "The button for selecting/prompting the root directory.")

(defvar bee-c-bdb-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 3 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\"O	c #5949bd\",
\"#      c black\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"______________#__OO______________\",
\"___________#__#_OOOO_____________\",
\"____________#_#O_OOO_____________\",
\"_____________#OOOOO______________\",
\"_________###_OOOOO_______________\",
\"___________#O_OOO###_____________\",
\"__________O_OOOO#________________\",
\"_________OOOOOO__#_______________\",
\"_________OOOOO#___#______________\",
\"_________OOOOOO#_________________\",
\"_________OOOOO_#_________________\",
\"__________OOO__#_________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_______###______#__#_____________\",
\"_______#__#_____#__#_____________\",
\"_______#__#_____#__#_____________\",
\"_______####___###__###___________\",
\"_______#___#_#__#__#__#__________\",
\"_______#___#_#__#__#__#__________\",
\"_______#___#_#__#__#__#__________\",
\"_______####___###__###___________\",
\"_________________________________\",
\"_________________________________\"};")
   "*The bdb button")

(defvar bee-c-connect-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 3 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\"O	c #5949bd\",
\"#      c black\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"________________________#__OO____\",
\"_____________________#__#_OOOO___\",
\"___________#__________#_#O_OOO___\",
\"___________##__________#OOOOO____\",
\"__############_____###_OOOOO_____\",
\"__#############______#O_OOO###___\",
\"__############______O_OOOO#______\",
\"___________##______OOOOOO__#_____\",
\"___________#_______OOOOO#___#____\",
\"___________________OOOOOO#_______\",
\"___________________OOOOO_#_______\",
\"____________________OOO__#_______\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_____________________________#___\",
\"__##_________________________#___\",
\"_#____##__###__###___##___##_###_\",
\"_#___#__#_#__#_#__#_#__#_#___#___\",
\"_#___#__#_#__#_#__#_####_#___#___\",
\"_#___#__#_#__#_#__#_#____#___#___\",
\"__##__##__#__#_#__#__###__##__##_\",
\"_________________________________\",
\"_________________________________\"};")
  "*The connect button")
  
(defvar bee-c-info-button
  (toolbar-make-button-list
   "/* XPM */
static char * info[] = {
\"33 33 3 1\",
\"_      c Gray75 s backgroundToolBarColor\",
\"o      c #330099\",
\"#      c black\",
\"_________________________________\",
\"_______________ooo_______________\",
\"______________ooooo______________\",
\"_____________ooooooo_____________\",
\"_____________ooooooo_____________\",
\"_____________ooooooo_____________\",
\"______________ooooo______________\",
\"_______________ooo_______________\",
\"_________________________________\",
\"_____________oooooo______________\",
\"____________ooooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"_____________ooooooo_____________\",
\"____________ooooooooo____________\",
\"_________________________________\",
\"_________________________________\",
\"___________________#_____________\",
\"__________#_______#______________\",
\"__________#_###__###_##__________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#__##__________\",
\"_________________________________\",
\"_________________________________\"};"))

(defvar bee-c-help-button
  (toolbar-make-button-list
   "/* XPM */
static char *foo[] = {
/* width height num_colors chars_per_pixel */
\"    28    28        2            1\",
/* colors */
\"_ c #b0b0b0\",
\"# c #000000\",
/* pixels */
\"____________________________\",
\"____________________________\",
\"____________________________\",
\"____________________________\",
\"________##########__________\",
\"_______##_#_#_#_#_#_________\",
\"______##_#_#_#_#_#_#________\",
\"_____##_#_#######_#_#_______\",
\"_____#_#_#########_#_#______\",
\"_____##_###########_#_______\",
\"_____#_#_###_#_###_#_#______\",
\"_____##_#####_#####_#_#_____\",
\"_____#_#_###_#_###_#_#_#____\",
\"_____##_#_#_#_#####_#_#_#___\",
\"_____#_#_#_#_#####_#_####___\",
\"_____##_#_#_#####_#_#_______\",
\"______##_#_#####_#_#_#______\",
\"_______##_#_###_#_#_#_______\",
\"________##_#####_#_#________\",
\"_________##_#_#_#_#_#_______\",
\"_________#_#_#_#_#_#________\",
\"_________##_###_#___________\",
\"_________#_#####_#__________\",
\"_________##_###_#___________\",
\"________##_#_#_#_#__________\",
\"________#_#_#_#_#_#_________\",
\"____________________________\",
\"____________________________\"
};"))

;*---------------------------------------------------------------------*/
;*    bee-c-opened-toolbar-height ...                                  */
;*---------------------------------------------------------------------*/
(defun bee-c-opened-toolbar-height ()
  (+ 8 (glyph-height (car bee-c-close-toolbar-button))))

;*---------------------------------------------------------------------*/
;*    bee-c-closed-toolbar-height ...                                  */
;*---------------------------------------------------------------------*/
(defun bee-c-closed-toolbar-height ()
  (+ 8 (glyph-height (car bee-c-open-toolbar-button))))


