;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-compile.el                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed May 27 10:41:34 1998                          */
;*    Last change :  Thu Oct 15 15:15:09 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bee compilation                                              */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-compile)
(require 'bee-config)
(require 'bee-mode)
(require 'bee-toolbar)
(require 'compile)

;*---------------------------------------------------------------------*/
;*    Buffer local compile variables ...                               */
;*---------------------------------------------------------------------*/
(defvar compile-command nil)
(make-variable-buffer-local 'bee-compile-command)

(defvar bee-makefile-entries nil)
(make-variable-buffer-local 'bee-makefile-entries)

(defvar bee-makefile nil)
(make-variable-buffer-local 'bee-makefile)

(defvar bee-external-bmake-process '()
  "The external Bee bmake process")

(defvar bee-last-compile-buffer '()
  "The buffer used by the last compilation.")

(defvar bee-success-hook '()
  "A function to be called (once) on success.
Executed hooks are removed from that list before being executed.")

(defvar bee-failure-hook '()
  "A function to be called (once) on failure.
Executed hooks are removed from that list before being executed.")

;*---------------------------------------------------------------------*/
;*    bee-set-compile-command ...                                      */
;*---------------------------------------------------------------------*/
(defun bee-set-compile-command (string)
  (interactive "sCompile command: ")
  (setq compile-command string)) 

;*---------------------------------------------------------------------*/
;*    bee-get-compile-command ...                                      */
;*---------------------------------------------------------------------*/
(defun bee-get-compile-command ()
  compile-command)

;*---------------------------------------------------------------------*/
;*    bee-get-makefile ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-get-makefile ()
  bee-makefile)
 
;*---------------------------------------------------------------------*/
;*    bee-set-makefile ...                                             */
;*    -------------------------------------------------------------    */
;*    The user may call this function to change of Makefile.           */
;*---------------------------------------------------------------------*/
(defun bee-set-makefile (file)
  (interactive "fMakefile: ")
  (bee-compile-command-init file)
  (bee-modeline-init))

;*---------------------------------------------------------------------*/
;*    bee-get-makefile-entries ...                                     */
;*---------------------------------------------------------------------*/
(defun bee-get-makefile-entries ()
  bee-makefile-entries)

;*---------------------------------------------------------------------*/
;*    bee-compile-p ...                                                */
;*---------------------------------------------------------------------*/
(defvar bee-compile-p nil)

;*---------------------------------------------------------------------*/
;*    bee-compile ...                                                  */
;*---------------------------------------------------------------------*/
(defun bee-compile ()
  (interactive)
  ;; the mark that the compilation as been requested by BEE.
  ;; the consequence will be that a correct compilation will see
  ;; its buffer to disapear after 5 seconds
  (setq bee-compile-p t)
  (if (and (bufferp bee-last-compile-buffer)
	   (buffer-live-p bee-last-compile-buffer))
      (let ((window (get-buffer-window bee-last-compile-buffer t)))
	(if (not (and (windowp window)
		      (one-window-p window)
		      (framep (window-frame window))))
	    (kill-buffer bee-last-compile-buffer))))
  (let ((compilation-window-height bee-compilation-window-height)
	(compilation-read-command nil))
    (if (stringp bee-root-directory)
	(let ((default-directory bee-root-directory))
	  (call-interactively 'compile))
      (call-interactively 'compile))))

;*---------------------------------------------------------------------*/
;*    bee-success-hook ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-success-hook (fun)
  (setq bee-success-hook fun))

;*---------------------------------------------------------------------*/
;*    bee-failure-hook ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-failure-hook (fun)
  (setq bee-failure-hook fun))

;*---------------------------------------------------------------------*/
;*    bee-compilation-finish-function ...                              */
;*---------------------------------------------------------------------*/
(defun bee-compilation-finish-function (buffer msg)
  (setq bee-last-compile-buffer buffer)
  (if bee-compile-p
      (progn
	(if (string-match "abnormally" msg)
	    ;; an error has occured
	    (let* ((pop-up-frames t)
		   (height (save-excursion
			     (set-buffer buffer)
			     (+ 2 (count-lines (point-min) (point-max)))))
		   (cur-height (frame-height (selected-frame)))
		   (err-height (if (> height cur-height)
				   cur-height
				 height))
		   (default-frame-alist (cons 'height
					      (cons err-height
						    default-frame-alist)))
		   (window (get-buffer-window buffer)))
	      (set-buffer buffer)
	      (if (windowp window)
		  (progn
		    (if (not (one-window-p window))
			(progn
			  (delete-window window)
			  (pop-to-buffer buffer)))
		    (set-buffer buffer)
		    (beginning-of-buffer)
		    (if (re-search-forward bee-error-regexp (point-max) t)
			(goto-char (match-beginning 0)))))
	      (if (functionp bee-failure-hook)
		  (let ((hook bee-failure-hook))
		    (setq bee-failure-hook nil)
		    (funcall hook buffer msg))
		nil))
	  ;; the compilation is completed but we process that code
	  ;; only if compile has been called from BEE-COMPILE
	  (let ((window (get-buffer-window buffer)))
	    (if (functionp bee-success-hook)
		(let ((hook bee-success-hook))
		  (if (windowp window)
		      (delete-window window))
		  (setq bee-success-hook nil)
		  (funcall hook buffer msg))
	      (progn
		(sit-for 5)
		(if (windowp window)
		    (delete-window window))
		(message "compilation completed...")
		t))))
	(setq bee-compile-p nil))))

;*---------------------------------------------------------------------*/
;*    bee-compile-init ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-compile-init ()
  ;; we set the goto-error funtion
  (fset 'compilation-goto-locus 'bee-compilation-goto-locus)
  ;; we prepare the hook for toolbar setting
  (add-hook 'compilation-mode-hook 'bee-compilation-mode-hook)
  ;; we set the compile command
  (bee-compile-command-init)
  ;; Bigloo error messages looks like:
  ;;  File "foo.scm", line 5, character 38:
  (setq compilation-error-regexp-alist
	(cons (list bee-error-regexp 1 2)
	      compilation-error-regexp-alist))
  ;; The font for errors
  (setq compilation-font-lock-keywords
	(append '(("^#.*$" . font-lock-comment-face)
		  ("^[^: ]+:$" . font-lock-function-name-face)
		  ("^[^:]+:$" . font-lock-keyword-face)
		  ("[0-9]+ error.? occured, ending.*$" . bee-error-face)
		  ("Compilation finished.*$" . bee-ok-face))
		compilation-font-lock-keywords))
  ;; the finish function
  (custom-set-variables
   '(compilation-finish-function (quote bee-compilation-finish-function))))

;*---------------------------------------------------------------------*/
;*    bee-compile-command-init ...                                     */
;*---------------------------------------------------------------------*/
(defun bee-compile-command-init (&optional name)
  (let ((makefile (if (stringp name)
		      name
		    (concat bee-root-directory bee-makefile-name))))
    (setq bee-makefile makefile)
    (if (file-exists-p makefile)
	(progn
	  (bee-set-compile-command bee-make)
	  (setq bee-makefile-entries (bee-fetch-makefile-entries)))
      (let ((fname (buffer-file-name (current-buffer))))
	(setq bee-makefile-entries nil)
	(if (stringp fname)
	    (let* ((fname (file-name-nondirectory fname))
		   (a.out (file-name-sans-extension fname)))
	      (bee-set-compile-command
	       (format "%s %s -o %s" bee-bigloo fname a.out))))))))

;*---------------------------------------------------------------------*/
;*    bee-root-version ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-root-version ()
  (let ((res (bee-make-entry bee-makefile-version-entry)))
    (if (and (stringp res) (not (string= res "")))
	(format " (%s)" (let ((len (length res)))
			  (cond
			   ((> len 6)
			    (substring res 0 6))
			   ((eq (aref res (- len 1)) ?\n)
			    (substring res 0 (- len 1)))
			   (t
			    res))))
      "")))
 
;*---------------------------------------------------------------------*/
;*    bee-make-entry ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-make-entry (entry)
  (if (and (stringp bee-makefile) (file-exists-p bee-makefile))
      (if (member bee-makefile-version-entry bee-makefile-entries)
	  (let* ((res (exec-to-string (format "cd %s; %s -f %s %s"
					      bee-root-directory
					      bee-make
					      bee-makefile
					      entry))))
	    (if (and (stringp res) (not (string= res "")))
		res
	      ""))
	"")
    ""))
 
;*---------------------------------------------------------------------*/
;*    bee-fetch-makefile-entries ...                                   */
;*---------------------------------------------------------------------*/
(defun bee-fetch-makefile-entries ()
  (if (and (stringp bee-makefile) (file-exists-p bee-makefile))
      (let* ((cmd     (concat "grep -E -e \""
			      bee-makefile-entry-regexp
			      ":\" " bee-makefile))
	     (entries (exec-to-string cmd))
	     (pat     (concat "^\\(" bee-makefile-entry-regexp "\\):"))
	     (index   0)
	     (res     '()))
	(setq res '())
	(while (string-match pat entries index)
	  (let ((entry (substring entries
				  (match-beginning 1)
				  (match-end 1)))
		(end   (match-end 0)))
	    (if (not (string-match bee-makefile-entry-exclude-regexp entry))
		(setq res (cons entry res)))
	    (setq index (+ 1 end))))
	(if bee-sort-makefile-entries
	    (sort res 'string<)
	  (nreverse res)))))

;*---------------------------------------------------------------------*/
;*    bee-generate/update-makefile ...                                 */
;*---------------------------------------------------------------------*/
(defun bee-generate/update-makefile ()
  (interactive)
  (if (or (not (stringp bee-makefile)) (not (file-exists-p bee-makefile)))
      (bee-generate-appli-makefile)
    (bee-update-makefile)))

;*---------------------------------------------------------------------*/
;*    bee-compiler-buffer ...                                          */
;*---------------------------------------------------------------------*/
(defvar bee-compiler-buffer nil)

;*---------------------------------------------------------------------*/
;*    bee-generate-makefile ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-generate-makefile (kind entry)
  (interactive)
  (if (not (stringp bee-makefile))
      (progn
	(if (not (stringp bee-root-directory))
	    (call-interactively 'init-root-directory))
	(setq bee-makefile (concat bee-root-directory bee-makefile-name))))
  (if (stringp bee-makefile)
      (if (or (not (file-exists-p bee-makefile))
	      (yes-or-no-p-dialog-box
	       "Makefile already exists, overwrite it?"))
	  (let* ((def (if (stringp (buffer-file-name))
			  (buffer-file-name)
			nil))
		 (read (read-string
			(if (stringp def)
			    (format "Entry file (default %s): " def)
			  "Entry file: ")))
		 (main (cond
			((and (stringp read) (> (length read) 0))
			 read)
			((not (stringp def))
			 (error "Illegal entry file" def))
			(t
			 def))))
	    ;; we remember who request the compilation
	    (setq bee-compiler-buffer (current-buffer))
	    ;; we set the name of the Makefile
	    (bee-set-makefile bee-makefile)
	    (message "Generating %s Makefile" kind)
	    (if (not (file-exists-p main))
		(message "*** WARNING: Can't find entry file -- %s" main))
	    (let ((compile-command (format "%s -f%s -o %s %s %s; %s -f %s bee"
					   bee-bmake
					   kind
					   (file-relative-name
					    bee-makefile
					    bee-root-directory)
					   entry
					   (file-relative-name main
							       bee-root-directory)
					   bee-make
					   bee-makefile)))
	      ;; we prepare the success hook that will change
	      ;; the default compile command to use the Makefile
	      (bee-success-hook '(lambda (buffer msg)
				   (let ((buf (current-buffer)))
				     (set-buffer bee-compiler-buffer)
				     (bee-compile-command-init)
				     (set-buffer buf))))
	      ;; we do produce the Makefile
	      (bee-compile))))
    (error "Can't bind Makefile.")))

;*---------------------------------------------------------------------*/
;*    bee-generate-appli-makefile ...                                  */
;*---------------------------------------------------------------------*/
(defun bee-generate-appli-makefile ()
  (interactive)
  (bee-generate-makefile "application" "-main"))

;*---------------------------------------------------------------------*/
;*    bee-generate-library-makefile ...                                */
;*---------------------------------------------------------------------*/
(defun bee-generate-library-makefile ()
  (interactive)
  (bee-generate-makefile "library" "-heap"))

;*---------------------------------------------------------------------*/
;*    bee-update-makefile ...                                          */
;*---------------------------------------------------------------------*/
(defun bee-update-makefile (&optional name)
  (interactive)
  (if (and (stringp bee-makefile) (file-exists-p bee-makefile))
      (let ((compile-command (format "%s -f %s updatemakefile; %s -f %s bee"
			 bee-make
			 bee-makefile
			 bee-make
			 bee-makefile)))
	(bee-compile))
    (error "Can't find Makefile")))

;*---------------------------------------------------------------------*/
;*    bee-add-user-makefile-entry ...                                  */
;*    -------------------------------------------------------------    */
;*    The interactive wrapper for BEE-ADD-MAKEFILE-ENTRY               */
;*---------------------------------------------------------------------*/
(defun bee-add-user-makefile-entry (source)
  (interactive "sSource entry: ")
  (if (string-match "\\(.*\\)\\([.][^.]+\\)$" source)
      (let ((base (substring source (match-beginning 1) (match-end 1)))
	    (suf  (substring source (match-beginning 2) (match-end 2))))
	(bee-add-makefile-entry (concat base ".o") source))
    (bee-add-makefile-entry (read-string "object: ") source)))
  
;*---------------------------------------------------------------------*/
;*    bee-add-makefile-entry ...                                       */
;*---------------------------------------------------------------------*/
(defun bee-add-makefile-entry (object source)
  (interactive)
  (if (and (stringp bee-makefile) (file-exists-p bee-makefile))
      (progn
	(message "Adding entries to makefile")
	(let* ((cmd (format "%s -o %s %s %s; %s -f %s bee"
			    bee-bmake
			    bee-makefile
			    (if (stringp object)
				(concat "-object " (file-relative-name
						    object
						    bee-root-directory))
			      "")
			    (if (stringp source)
				(concat "-source " (file-relative-name
						    source
						    bee-root-directory))
			      "")
			    bee-make
			    bee-makefile))
	       (compilation-ask-about-save nil)
	       (compile-command cmd))
	  (bee-compile)))
    (error "Can't bind Makefile.")))

;*---------------------------------------------------------------------*/
;*    bee-view-last-compile-messages ...                               */
;*---------------------------------------------------------------------*/
(defun bee-view-last-compile-messages ()
  (interactive)
  (if (and (bufferp bee-last-compile-buffer)
	   (buffer-live-p bee-last-compile-buffer))
      (progn
	(set-buffer bee-last-compile-buffer)
	(let* ((pop-up-frames t)
	       (height (+ 2 (count-lines (point-min) (point-max))))
	       (cur-height (frame-height (selected-frame)))
	       (new-height (if (> height cur-height) cur-height height))
	       (default-frame-alist (cons 'height
					  (cons new-height
						default-frame-alist))))
	  (beginning-of-buffer)
	  (pop-to-buffer bee-last-compile-buffer)))))
  
;*---------------------------------------------------------------------*/
;*    bee-compile-messages-available-p ...                             */
;*---------------------------------------------------------------------*/
(defun bee-compile-messages-available-p ()
  (interactive)
  (and (bufferp bee-last-compile-buffer)
       (buffer-live-p bee-last-compile-buffer)))

;*---------------------------------------------------------------------*/
;*    bee-compilation-goto-locus ...    .                              */
;*    -------------------------------------------------------------    */
;*    Emacs lisp code is always so well written. Really sometime I     */
;*    don't understand. I'm not able to find a way to pop up an        */
;*    error buffer from compile without modification the source        */
;*    code of compile.el!                                              */
;*---------------------------------------------------------------------*/
(defun bee-compilation-goto-locus (next-error)
  "Jump to an error locus returned by `compilation-next-error-locus'.
Takes one argument, a cons (ERROR . SOURCE) of two markers.
Selects a window with point at SOURCE, with another window displaying ERROR."
;; XEmacs: this code is horrendous, and makes windows do all sorts of
;; weird things when you're using separate frames for the compilation
;; and source buffer.
;  (if (and (window-dedicated-p (selected-window))
;	   (eq (selected-window) (frame-root-window)))
;      (switch-to-buffer-other-frame (marker-buffer (cdr next-error)))
;    (switch-to-buffer (marker-buffer (cdr next-error))))
;  (goto-char (cdr next-error))
;  ;; If narrowing got in the way of
;  ;; going to the right place, widen.
;  (or (= (point) (marker-position (cdr next-error)))
;      (progn
;        (widen)
;        (goto-char (cdr next-error))))
;
;  ;; Show compilation buffer in other window, scrolled to this error.
;  (let* ((pop-up-windows t)
;	 (w (or (get-buffer-window (marker-buffer (car next-error)) 'visible)
;		(display-buffer (marker-buffer (car next-error))))))
;    (set-window-point w (car next-error))
;    (set-window-start w (car next-error))
;    (compilation-set-window-height w)))

  (let* ((pop-up-frames t)
	 (compilation-buffer (marker-buffer (car next-error)))
	 (source-buffer (marker-buffer (cdr next-error)))
	 ;; make sure compilation buffer is visible ...
	 (compilation-window
	 ;; Use an existing window if it is in a visible frame.
	  (or (get-buffer-window compilation-buffer 'visible)
	      ;; Pop up a window.
	      (pop-to-buffer compilation-buffer))))

    ;; now, make the compilation buffer **STAY WHERE IT IS** and
    ;; make sure the source buffer is visible

    (select-window compilation-window)
    (pop-to-buffer source-buffer)

    ;; now put things aright in the compilation window.
    (set-window-point compilation-window (car next-error))
    (set-window-start compilation-window (car next-error))
    (compilation-set-window-height compilation-window)

    ;; now put things aright in the source window.

    (set-buffer source-buffer)
    (goto-char (cdr next-error))
    ;; If narrowing got in the way of
    ;; going to the right place, widen.
    (or (= (point) (marker-position (cdr next-error)))
	(progn
	  (widen)
	  (goto-char (cdr next-error))))))

;*---------------------------------------------------------------------*/
;*    bee-compilation-opened-toolbar ...                               */
;*---------------------------------------------------------------------*/
(defvar bee-compilation-opened-toolbar 
  '(;;close button
    [bee-compilation-close-toolbar-button bee-compilation-close-toolbar t "bee-compilation toolbar"]
    [:style 2d :size 2]
    
    ;; the stop button
    [bee-compilation-stop-button kill-compilation t "Kill compilation"]
    [:style 2d :size 2]
    
    ;; the compile button
    [bee-compile-button bee-compile t "Compile"]
    ;; the root button
    [bee-root-button bee-set-root-directory t "Set Bee root directory"]
    ;; the bmake button
    [bee-bmake-button bee-generate/update-makefile t "Generate/update Makefile"]
    ;; the bmake entry button
    [bee-bmake-entry-button bee-add-user-makefile-entry t "Add Makefile entry"]
    [:style 2d :size 2]
    
    ;; flushing right
    nil
    [:style 2d :size 2]
    ;; the help action
    [bee-compilation-help-button bee-compilation-help t
    "The help for bee-compilation"]))

(fset 'bee-compilation-help 'describe-mode)

(defun xinfo-info ()
  (interactive)
  (Info-find-node "info" "Top"))

;*---------------------------------------------------------------------*/
;*    bee-compilation-closed-toolbar ...                               */
;*---------------------------------------------------------------------*/
(defvar bee-compilation-closed-toolbar
  '(;;open button
    [bee-compilation-open-toolbar-button
    bee-compilation-open-toolbar
    t
    "bee-compilation toolbar"]))

;*---------------------------------------------------------------------*/
;*    Opening/closing the toolbar ...                                  */
;*---------------------------------------------------------------------*/
(defun bee-compilation-close-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bee-compilation-closed-toolbar-height)))
    (set-specifier default-toolbar
		   bee-compilation-closed-toolbar
		   (current-buffer))
    (set-specifier default-toolbar-height new-height (current-buffer))))

(defun bee-compilation-open-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bee-compilation-opened-toolbar-height)))
    (set-specifier default-toolbar
		   bee-compilation-opened-toolbar
		   (current-buffer))
    (set-specifier default-toolbar-height new-height (current-buffer))))

;*---------------------------------------------------------------------*/
;*    bee-compilation-mode-hook ...                                    */
;*    -------------------------------------------------------------    */
;*    This hook simply set the BEE compilation toolbar for the buffer  */
;*---------------------------------------------------------------------*/
(defun bee-compilation-mode-hook ()
  (set-specifier default-toolbar-visible-p t)
  (set-specifier default-toolbar bee-compilation-opened-toolbar
		 (current-buffer)))
  
;*---------------------------------------------------------------------*/
;*    The bee-compilation toolbar buttons                              */
;*---------------------------------------------------------------------*/
(defvar bee-compilation-close-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *on[] = {
/* width height num_colors chars_per_pixel */
\"    8    30        9            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #808080\",
\"d c #c0c0c0\",
\"e c #e0e4e0\",
\"f c #f8e8c8\",
\"g c #f8fcf8\",
/* pixels */
\"eddddddd\",
\"eddddddd\",
\"e#####cd\",
\"ed#aagdd\",
\"edd#gddd\",
\"eddddddd\",
\"eddddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"eddddddd\",
};"))

(defvar bee-compilation-open-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *off[] = {
/* width height num_colors chars_per_pixel */
\"    27    5        8            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #c0c0c0\",
\"d c #e0e4e0\",
\"e c #f8e8c8\",
\"f c #f8fcf8\",
/* pixels */
\"ccc#ccccccfccfccfccfccfcccc\",
\"ccc##cccccc#cc#cc#cc#cc#ccc\",
\"ccc#a#cccccfccfccfccfccfccc\",
\"ccc#afcccccc.cc.cc.cc.cc.cc\",
\"ccc#fcccccccccccccccccccccc\",
};"))

(defvar bee-compilation-stop-button
  (toolbar-make-button-list
   "/* XPM */
static char *stop[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        4            1\",
/* colors */
\"_ c #c0c0c0 s backgroundToolBarColor\",
\". c #cc0000\",
\"a c #f0f4f0\",
\"# c black\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"____________.........____________\",
\"___________...........___________\",
\"__________.............__________\",
\"_________..............._________\",
\"________.................________\",
\"________.aaa.aaa.aaa.aaa.________\",
\"________.a....a..a.a.a.a.________\",
\"________.aaa..a..a.a.a.a.________\",
\"________...a..a..a.a.aaa.________\",
\"________.aaa..a..aaa.a...________\",
\"________.................________\",
\"_________..............._________\",
\"__________.............__________\",
\"___________...........___________\",
\"____________.........____________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_____________#___________________\",
\"________###__#___________________\",
\"_______#_____###___##__###_______\",
\"________###__#____#__#_#__#______\",
\"___________#_#____#__#_#__#______\",
\"_______#___#_#____#__#_#__#______\",
\"________###___###__##__###_______\",
\"_______________________#_________\",
\"_______________________#_________\"
};")
  "*The stop button")


(defvar bee-compilation-help-button
  (toolbar-make-button-list
   "/* XPM */
static char *foo[] = {
/* width height num_colors chars_per_pixel */
\"    28    28        2            1\",
/* colors */
\". c #b0b0b0\",
\"# c #000000\",
/* pixels */
\"............................\",
\"............................\",
\"............................\",
\"............................\",
\"........##########..........\",
\".......##.#.#.#.#.#.........\",
\"......##.#.#.#.#.#.#........\",
\".....##.#.#######.#.#.......\",
\".....#.#.#########.#.#......\",
\".....##.###########.#.......\",
\".....#.#.###.#.###.#.#......\",
\".....##.#####.#####.#.#.....\",
\".....#.#.###.#.###.#.#.#....\",
\".....##.#.#.#.#####.#.#.#...\",
\".....#.#.#.#.#####.#.####...\",
\".....##.#.#.#####.#.#.......\",
\"......##.#.#####.#.#.#......\",
\".......##.#.###.#.#.#.......\",
\"........##.#####.#.#........\",
\".........##.#.#.#.#.#.......\",
\".........#.#.#.#.#.#........\",
\".........##.###.#...........\",
\".........#.#####.#..........\",
\".........##.###.#...........\",
\"........##.#.#.#.#..........\",
\"........#.#.#.#.#.#.........\",
\"............................\",
\"............................\"
};"))

;*---------------------------------------------------------------------*/
;*    bee-compilation-opened-toolbar-height ...                        */
;*---------------------------------------------------------------------*/
(defun bee-compilation-opened-toolbar-height ()
  (+ 8 (glyph-height (car bee-compilation-close-toolbar-button))))

;*---------------------------------------------------------------------*/
;*    bee-compilation-closed-toolbar-height ...                        */
;*---------------------------------------------------------------------*/
(defun bee-compilation-closed-toolbar-height ()
  (+ 8 (glyph-height (car bee-compilation-open-toolbar-button))))
