;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-config.el                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon May 25 07:28:09 1998                          */
;*    Last change :  Sat Oct 10 13:21:16 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Bee configuration file.                                      */
;*=====================================================================*/
 
;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-config)

;; the binary files
(defcustom bee-bigloo "bigloo"
  "*The name of the Bigloo compiler."
  :group 'bee
  :type 'string)

(defcustom bee-cigloo "cigloo"
  "*The name of the Cigloo binary file."
  :group 'bee
  :type 'string)

(defcustom bee-make "make"
  "*The name of the Make binary file."
  :group 'bee
  :type 'string)

(defcustom bee-bmake "bmake"
  "*The name of the Bigloo Makefile generator."
  :group 'bee
  :type 'string)

(defcustom bee-external-indent "bpp"
  "*The external Bee indent program."
  :group 'bee
  :type 'string)

(defcustom bee-cc "gcc"
  "*The name of the C compiler."
  :group 'bee
  :type 'string)

(defcustom bee-c-manual-page-number 1
  "*The Unix section number for C function pages."
  :group 'bee
  :type 'number)

(defcustom bee-c-external-indent "indent"
  "*The external Bee C indent program."
  :group 'bee
  :type 'string)

(defcustom bee-prof "bprof"
  "*The name of the Bee profiler."
  :group 'bee
  :type 'string)

;; the path and file names
(defcustom bee-root-directory "./"
  "*A string that is the project root directory."
  :group 'bee
  :type 'string)

(defcustom bee-tags-table ".etags"
  "*The name for TAGS table files."
  :group 'bee
  :type 'string)

(defcustom bee-afile-name ".afile"
  "*The name of the Bee afile file."
  :group 'bee
  :type 'string)

(defcustom bee-makefile-name "Makefile"
  "*The name of the Makefile file."
  :group 'bee
  :type 'string)

(defcustom bee-root-search-depth 3
  "*The depth search for the Bee Root Directory."
  :group 'bee
  :type 'number)

;; window configuration
(defcustom bee-compilation-window-height window-min-height
  "*Number of lines in a compilation window.  If nil, use Emacs default."
  :group 'bee
  :type 'number)

(defcustom bee-font-lock-p t
  "*Non-nil means uses font lock for Bee buffers."
  :group 'bee
  :type 'boolean)

;; key bindings
(defcustom bee-mouse-binding [(button3)]
  "*The mouse binding for Bee mouse event."
  :group 'bee
  :type 'vector)

;; makefile and compilation flags
(defcustom bee-error-regexp "^File \"\\([^\"]+\\)\", line \\([0-9]+\\)"
  "*The expression that matches Bigloo errors."
  :group 'bee
  :type 'string)

(defcustom bee-sort-makefile-entries t
  "*Non-nil means Makefile entries are sorted for popup."
  :group 'bee
  :type 'boolean)

(defcustom bee-makefile-entry-regexp "^[^.:\t $][^ %$:]*[ \t]*"
  "*The regexp that matches Makefile entry declaration (without the :)."
  :group 'bee
  :type 'string)

(defcustom bee-makefile-entry-exclude-regexp "[.]o[ \t]*$\\|^%."
  "*The regexp for excluding makefile entry matches."
  :group 'bee
  :type 'string)

(defcustom bee-makefile-version-entry "version"
  "*The name of the Makefile entry that print the version number."
  :group 'bee
  :type 'string)

(defcustom bee-user-menu nil
  "*Non-nil means a popup menu user part."
  :group 'bee
  :type 'boolean)

(defcustom bee-bigloo-prompt "^(.?=>)"
  "*The regexp that matched Bigloo prompts."
  :group 'bee
  :type 'string)

(defcustom bee-tmp-dir (let ((dir (getenv "TMPDIR")))
		      (if (stringp dir)
			  dir
			"/tmp"))
  "*The tmp directory."
  :group 'bee
  :type 'string)

(defcustom bee-menu-max-width 30
  "*The maximum width of bee menus."
  :group 'bee
  :type 'number)

;; faces
(defcustom bee-ident-prof-face 'font-lock-function-name-face
  "The face to display Bigloo identifiers on profile."
  :group 'bee
  :type 'symbol)

(defcustom bee-module-prof-face 'bee-face-1
  "The face to display Bigloo modules on profile."
  :group 'bee
  :type 'symbol)

(defface bee-c-prof-face
  '((((class color)) (:foreground "gray75" :bold nil))
    (t (:bold t)))
  "The face to display C identifiers on profile."
  :group 'bee)

(defface bee-root-face
  (list (list '((class color))
	      (list ':foreground "red"
		    ':background (face-background 'modeline)
		    ':bold t))
	'(t (:bold t)))
  "Bee Root face."
  :group 'bee)

(defface bee-no-root-face
  (list (list '((class color))
	      (list ':foreground "blue"
		    ':background (face-background 'modeline)
		    ':bold t))
	'(t (:bold t)))
  "Bee No Root face."
  :group 'bee)

