;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-flock.el                   */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon May 25 08:00:32 1998                          */
;*    Last change :  Thu Oct 15 13:23:03 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Bee font-lockification                                       */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-flock)

;*---------------------------------------------------------------------*/
;*    font-lock-make-face ...                                          */
;*---------------------------------------------------------------------*/
(defun font-lock-make-face (l)
  (let ((fname (car l))
        (color (car (cdr l)))
        (italic (car (cdr (cdr l))))
        (bold (car (cdr (cdr (cdr l)))))
        (underline (car (cdr (cdr (cdr (cdr l)))))))
    (let ((face (copy-face 'default fname)))
      (if color
          (set-face-foreground face color))
      (if (and italic bold)
          (condition-case nil
              (make-face-bold-italic face)
            (error nil))
        (progn
          (if italic
              (condition-case nil
                  (make-face-italic face)
                (error nil)))
          (if bold
              (condition-case nil
                  (make-face-bold face)
                (error nil)))))
      (if underline
          (condition-case nil
              (set-face-underline-p face t)
            (error nil)))
      (if (not (x-display-color-p))
          (condition-case nil
              (make-face-bold face)
            (error nil)))
      face)))

;*---------------------------------------------------------------------*/
;*    bee colors ...                                                   */
;*---------------------------------------------------------------------*/
(defface bee-face-1
  '((((class color)) (:foreground "blue" :bold t))
    (t (:bold t)))
  "Bee face 1."
  :group 'bee)

(defface bee-face-2
  '((((class color)) (:foreground "tomato2" :bold t))
    (t (:bold t)))
  "Bee face 2."
  :group 'bee)

(defface bee-face-3
  '((((class color)) (:foreground "BlueViolet" :bold t))
    (t (:bold t)))
  "Bee face 3."
  :group 'bee)

(defface bee-face-4
  '((((class color)) (:foreground "green3" :bold t))
    (t (:bold t)))
  "Bee face 4."
  :group 'bee)

(defface bee-error-face
  '((((class color)) (:foreground "red" :bold t))
    (t (:bold t)))
  "Bee error face."
  :group 'bee)

(defface bee-ok-face
  '((((class color)) (:foreground "green3" :bold t))
    (t (:bold t)))
  "Bee ok face."
  :group 'bee)

(defface bee-italic-face
  '((((class color)) (:bold t :italic t))
    (t (:bold t)))
  "Bee face to display italic text."
  :group 'bee)

;*---------------------------------------------------------------------*/
;*    bee font-lock regexps ...                                        */
;*---------------------------------------------------------------------*/
(defvar bee-font-lock-keywords
  (list
   (list (concat "^\(\\(\\(?:define\\|define-inline\\|define-struct"
		 "\\|define-macro\\|define-generic\\|define-method"
		 "\\|define-syntax"
		 "\\|define-command\\)[ ]+\\(\(.*\)\\|[^ \n]+\\)\\)[ \n]")
	 1
	 'font-lock-function-name-face)
   (list "\(\\(\\(?:module\\|directives\\)[ ]+[^ \n]+\\)[ \n]"
         1
	 'bee-face-1)
   (cons "\\(?:::[^ \n)]+\\|#![a-zA-Z]+\\)"
	 'bee-face-4)
   (list (concat "\(\\(type\\|export\\|import\\|option\\|eval\\|main"
		 "\\|from\\|static\\|use\\|library"
		 "\\|include\\|foreign\\|extern\\|Cforeign\\)[ \n]")
	 1
	 'font-lock-type-face)
   (list (concat "\(\\(class\\|wide-class\\|final-class\\|generic"
		 "\\|inline\\|macro\\|infix[ ]macro\\)[ \n]")
	 1
	 'bee-face-3)
   (list (concat "\(\\(let\\|let[*]\\|letrec\\|define"
		 "\\|set[!]\\|with-access\\|instantiate\\|duplicate"
		 "\\|widen[!]\\|lambda\\|labels"
		 "\\|let-syntax\\|letrec-syntax"
		 "\\|regular-grammar\\|lalr-grammar"
		 "\\|if\\|begin\\|case\\|cond\\|else"
		 "\\|multiple-value-bind\\|values"
		 "\\|match-case\\|match-lambda"
		 "\\|string-case\\|syntax-rules\\)[ :\n]")
	 1
	 'font-lock-keyword-face)
   
   (cons "\\[assert[^\]]*\\]\\|\(assert[^\)]*\)"
	 'bee-face-2)
   (list (concat "\(\\(error\\|warning\\|pragma\\|trace\\|"
		 "bind-exit\\|call/cc\\|try\\|\unwind-protect\\)"
		 "[ \n:]")
	 1
	 'bee-face-2)
   (cons "\\(`\\|,\\|'\\|,@\\)"
	 'bee-face-1)))

;*---------------------------------------------------------------------*/
;*    bee-font-lock-get-info-keywords ...                              */
;*    -------------------------------------------------------------    */
;*    The Bee font lock keyword list for xinfo.                        */
;*---------------------------------------------------------------------*/
(defun bee-font-lock-get-info-keywords ()
  (append
   (list (cons "^ - \\(essential \\|library \\|bigloo \\)?\\(\\(rgc \\)?procedure\\|variable\\): .*$"
	       'font-lock-function-name-face)
	 (cons "^ - \\(essential \\|library \\|bigloo \\)?syntax: .*$"
	       'bee-face-2)
	 (cons "^ - bigloo module clause: .*$"
	       'bee-face-3)
	 (cons "^[*] [^:]+:" 'font-lock-keyword-face)
	 (cons "\\([^=]=>\\|-|\\) .+$" 'bee-italic-face)
	 (cons ";;.*$" 'font-lock-comment-face)
	 (cons "\"[^\"]+\"" 'font-lock-string-face))
   bee-font-lock-keywords))
  
