;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-keymap.el                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue May 26 07:34:07 1998                          */
;*    Last change :  Sat Oct 10 13:42:38 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bee menus                                                    */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-keymap)
(require 'bee-config)
(require 'bee-tools)
(require 'bee-autoload)
(require 'bee-mouse)
(require 'bdb-autoload)

;*---------------------------------------------------------------------*/
;*    bee-return ...                                                   */
;*---------------------------------------------------------------------*/
(defun bee-return (&optional dummy)
  "Indent on [return]"
  (interactive)
  (if (= (point) 1)
      (newline)
    (newline-and-indent)))
 
;*---------------------------------------------------------------------*/
;*    bee-ident-menu ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-ident-menu ()
  '(("Find..."
     ["Document" bee-docline t]
     ["Definition" bee-tags-find t]
     ["Usage" bee-usage-find t]
     ["Module" bee-tags-find-module t])))

;*---------------------------------------------------------------------*/
;*    bee-compile-last-entry ...                                       */
;*---------------------------------------------------------------------*/
(defvar bee-compile-last-entry nil)

;*---------------------------------------------------------------------*/
;*    bee-compile-menu ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-compile-menu ()
  (let ((mkmf (bee-get-makefile))
	(entries (bee-get-makefile-entries)))
    (if (and (stringp mkmf) (file-exists-p mkmf) (consp entries))
	(progn
	  (if (not (stringp bee-compile-last-entry))
	      (setq bee-compile-last-entry (car entries)))
	  (list
	   (list (let ((string "Compilation"))
		   (put-text-property 0 5 'face 'font-lock-function-name-face
				      string)
		   string)
		 ["View last messages" bee-view-last-compile-messages
		 (bee-compile-messages-available-p)]
		 (cons "make"
		       (mapcar
			'(lambda (entry)
			   (vector entry
				   (if (eq entry bee-compile-last-entry)
				       'bee-compile
				     (list 'progn
					   (list 'setq
						 'bee-compile-last-entry
						 entry)
					   (list 'bee-set-compile-command
						 (concat bee-make " " entry))
					   '(bee-compile)))
				   t))
			entries))
		 ["Kill compilation" kill-compilation compilation-in-progress]
		 ["Set compile command" bee-set-compile-command t])))
      (list
       (list "Compilation..."
	     (vector (bee-string-excerpt (bee-get-compile-command))
		     'bee-compile
		     t)
	     ["Kill compilation" kill-compilation compilation-in-progress]
	     ["Set compile command" bee-set-compile-command t])))))

;*---------------------------------------------------------------------*/
;*    bee-makefile-menu ...                                            */
;*---------------------------------------------------------------------*/
(defun bee-makefile-menu ()
  (let ((bee-makefile (bee-get-makefile)))
    (list (list (if (stringp bee-makefile)
		    (concat (file-name-nondirectory bee-makefile) "...")
		  "Makefile...")
		["Edit Makefile"
		bee-edit-makefile
		(and (stringp bee-makefile)
		     (file-exists-p bee-makefile))]
		(if (and (stringp bee-makefile) (file-exists-p bee-makefile))
		    (vector "Update" 'bee-update-makefile t)
		  (list "Generate Makefile"
			(vector "application" 'bee-generate-appli-makefile t)
			(vector "library" 'bee-generate-library-makefile t)))
		["Select other Makefile" bee-set-makefile t]))))

;*---------------------------------------------------------------------*/
;*    bee-bdb-menu ...                                                 */
;*---------------------------------------------------------------------*/
(defun bee-bdb-menu ()
  (list (list "Bdb..."
	      ["Bdb other frame" bdb-other-frame t]
	      (if (and (fboundp 'bdb-connected-buffer-p)
		       (bdb-connected-buffer-p))
		  (vector "Disconnect buffer" 'bee-toggle-connect-buffer t)
		(vector "Connect buffer" 'bee-toggle-connect-buffer t)))))

;*---------------------------------------------------------------------*/
;*    bee-indent-menu ...                                              */
;*---------------------------------------------------------------------*/
(defun bee-indent-menu ()
  (list (list "Indent..."
	      ["buffer" bee-external-indent t]
	      ["define"
	      bee-indent-define
	      (condition-case ()
		  (save-excursion
		    (end-of-defun)
		    (beginning-of-defun))
		(error nil))]
	      ["last sexp" bee-indent-last-sexp t]
	      ["toplevel sexp" bee-indent-toplevel-sexp t])))

;*---------------------------------------------------------------------*/
;*    bee-repl-menu ...                                                */
;*---------------------------------------------------------------------*/
(defun bee-repl-menu ()
  (list (list "Repl..."
	      ["other frame" bee-repl-other-frame t]
	      ["buffer" bee-repl-send-buffer t]
	      ["define"
	      bee-repl-send-define
	      (condition-case ()
		  (save-excursion
		    (end-of-defun)
		    (beginning-of-defun))
		(error nil))
	      ]
	      ["last sexp" bee-repl-send-last-sexp t]
	      ["toplevel sexp" bee-repl-send-toplevel-sexp t]
	      ["region"
	      bee-repl-send-region
	      (and (region-active-p) (region-exists-p))])))

;*---------------------------------------------------------------------*/
;*    bee-expand-menu ...                                              */
;*---------------------------------------------------------------------*/
(defun bee-expand-menu ()
  (list (list "Expand..."
	      ["buffer" bee-expand-buffer t]
	      ["define"
	      bee-expand-define
	      (condition-case ()
		  (save-excursion
		    (end-of-defun)
		    (beginning-of-defun))
		(error nil))
	      ]
	      ["last sexp" bee-expand-last-sexp t]
	      ["toplevel sexp" bee-expand-toplevel-sexp t]
	      ["region"
	      bee-expand-region
	      (and (region-active-p) (region-exists-p))])))

;*---------------------------------------------------------------------*/
;*    bee-profile-menu ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-profile-menu ()
  '(("Profile..."
     ["Compile for profile" bee-compile-for-profile t]
     ["Run for profile" bee-run-for-profile t])))
	      
;*---------------------------------------------------------------------*/
;*    bee-module-menu ...                                              */
;*---------------------------------------------------------------------*/
(defun bee-module-menu ()
  '(("Module..."
     ["Import binding" bee-import-binding t]
     ["Import C file" bee-import-c-file t]
     "-"
     ["Export binding" bee-export-binding t]
     ["Export function" bee-export-function t]
     ["Export variable" bee-export-variable t])))
  
;*---------------------------------------------------------------------*/
;*    bee-user-menu ...                                                */
;*---------------------------------------------------------------------*/
(defun bee-user-menu ()
  (if (listp bee-user-menu)
      bee-user-menu
    nil)) 

;*---------------------------------------------------------------------*/
;*    bee-make-menu ...                                                */
;*---------------------------------------------------------------------*/
(defun bee-make-menu ()
  (let* ((separator (list "--:shadowEtchedOut"))
	 (user-menu (bee-user-menu))
	 (menu      (append (bee-ident-menu)
			    (bee-compile-menu)
			    (bee-makefile-menu)
			    (bee-repl-menu)
			    (bee-bdb-menu)
			    (bee-indent-menu)
			    (bee-expand-menu)
			    (bee-module-menu)
			    (bee-profile-menu)
			    (if user-menu
				(append separator user-menu)
			      '()))))
    menu))

;*---------------------------------------------------------------------*/
;*    bee-make-version-menu ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-make-version-menu ()
  '(["Checkout file version" bee-checkout-file-version t]
     ["Compare to file version" bee-diff-file-version t]
     "--:shadowEtchedIn"
     ["Checkin project version" bee-checkin-project t]
     "--:shadowEtchedIn"
     ["Produce tar.gz" bee-tar-gz-project t]))

;*---------------------------------------------------------------------*/
;*    bee-popup-menu ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-popup-menu (event)
  (interactive "e")
  (popup-menu (cons "Bee" (bee-make-menu))))
  
;*---------------------------------------------------------------------*/
;*    bee-region-popup-entries ...                                     */
;*---------------------------------------------------------------------*/
(defvar bee-region-popup-entries 
  (list ["documentation" bee-docline t]
	["eval" bee-repl-send-region t]
	["expand" bee-expand-region t]
	"--:shadowDoubleEtchedOut"
	["kill region" kill-region t])
  "*The entry for the region menu.")
(make-variable-buffer-local 'bee-region-popup-entry)

;*---------------------------------------------------------------------*/
;*    bee-add-region-popup-entry ...                                   */
;*---------------------------------------------------------------------*/
(defun bee-add-region-popup-entry (entries)
  "Add ENTRIES (a list) to the region popup menu."
  (setq bee-region-popup-entries (cons entries bee-region-popup-entries)))

;*---------------------------------------------------------------------*/
;*    bee-remove-region-popup-entry ...                                */
;*---------------------------------------------------------------------*/
(defun bee-remove-region-popup-entry (entries)
  "Add ENTRIES (a list) to the region popup menu."
  (setq bee-region-popup-entries (delq entries bee-region-popup-entries)))

;*---------------------------------------------------------------------*/
;*    bee-region-popup-menu ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-region-popup-menu (event)
  (interactive "e")
  ;; we have to rebuild the cache first
  (let ((l bee-region-popup-entries)
	(menu '()))
    (while (consp l)
      (cond
       ((functionp (car l))
	(setq menu (append (reverse (funcall (car l) event)) menu)))
       ((consp (car l))
	(setq menu (append (car l) menu)))
       (t
	(setq menu (cons (car l) menu))))
      (setq l (cdr l)))
    (popup-menu (cons (concat
		       "`"
		       (bee-region-excerpt (region-beginning) (region-end))
		       "'")
		      (nreverse menu)))))

;*---------------------------------------------------------------------*/
;*    bee keymap ...                                                   */
;*---------------------------------------------------------------------*/
(defvar bee-mode-map (make-sparse-keymap))
(defvar bee-map-prefix ?\c)
(defvar bee-prefixed-map (make-sparse-keymap))

;*---------------------------------------------------------------------*/
;*    bee-keymap-init ...                                              */
;*---------------------------------------------------------------------*/
(defun bee-keymap-init ()
  
  ;; bee bindings
  (define-key bee-mode-map "\t"               'bee-indent-line)
  (define-key bee-mode-map "\e\C-q"           'bee-indent-sexp)
  (define-key bee-mode-map "\C-m"             'bee-return)
  (define-key bee-mode-map "\e\C-m"           'newline)
  (define-key bee-mode-map "\e\C- "           'mark-sexp)
  
  ;; C-' keymap
  (define-key bee-mode-map [(control \c)]     'bee-prefix)
  (fset 'bee-prefix bee-prefixed-map)

  ;; finders
  (define-key bee-prefixed-map "\C-di"        'bee-docline)
  (define-key bee-prefixed-map "\C-du"        'bee-usage-find)
  (define-key bee-prefixed-map "\C-df"        'bee-tags-find-variable)
  (define-key bee-prefixed-map "\C-dm"        'bee-tags-find-module)
  (define-key bee-mode-map "\C-x5."           'bee-tags-find)

  ;; compilation
  (define-key bee-prefixed-map "\C-c\C-c"     'bee-compile)
  (define-key bee-prefixed-map "\C-ck"        'kill-compilation)
  (define-key bee-prefixed-map "\C-cs"        'bee-set-compile-command)
  (define-key bee-prefixed-map "\C-cm"        'bee-view-last-compile-messages)

  ;; makefile
  (define-key bee-prefixed-map "\C-ce"        'bee-edit-makefile)
  (define-key bee-prefixed-map "\C-ca"        'bee-generate-appli-makefile)
  (define-key bee-prefixed-map "\C-cl"        'bee-generate-library-makefile)
  (define-key bee-prefixed-map "\C-cu"        'bee-update-makefile)
  (define-key bee-prefixed-map "\C-c\C-s"     'bee-set-makefile)

  ;; repl
  (define-key bee-prefixed-map "\C-r\C-r"     'bee-repl-other-frame)
  (define-key bee-prefixed-map "\C-rb"        'bee-repl-send-buffer)
  (define-key bee-prefixed-map "\C-rd"        'bee-repl-send-define)
  (define-key bee-prefixed-map "\C-rl"        'bee-repl-send-last-sexp)
  (define-key bee-prefixed-map "\C-rt"        'bee-repl-send-toplevel-sexp)
  (define-key bee-prefixed-map "\C-rr"        'bee-repl-send-region)

  ;; bdb
  (define-key bee-prefixed-map "\C-b\C-b"     'bdb-other-frame)
  (define-key bee-prefixed-map "\C-b\c"       'bee-toggle-connect-buffer)

  ;; indent
  (define-key bee-prefixed-map "\C-i\C-i"     'bee-external-indent)
  (define-key bee-prefixed-map "\C-i\C-d"     'bee-indent-define)
  (define-key bee-prefixed-map "\C-i\C-l"     'bee-indent-last-sexp)
  (define-key bee-prefixed-map "\C-i\C-t"     'bee-indent-toplevel-sexp)

  ;; expand
  (define-key bee-prefixed-map "\C-e\C-e"     'bee-expand-buffer)
  (define-key bee-prefixed-map "\C-e\C-d"     'bee-expand-define)
  (define-key bee-prefixed-map "\C-e\C-l"     'bee-expand-last-sexp)
  (define-key bee-prefixed-map "\C-e\C-t"     'bee-expand-toplevel-sexp)
  (define-key bee-prefixed-map "\C-e\C-r"     'bee-expand-region)

  ;; module
  (define-key bee-prefixed-map "\C-mi"        'bee-import-binding)
  (define-key bee-prefixed-map "\C-mc"        'bee-import-c-file)
  (define-key bee-prefixed-map "\C-mb"        'bee-export-binding)
  (define-key bee-prefixed-map "\C-mf"        'bee-export-function)
  (define-key bee-prefixed-map "\C-mv"        'bee-export-variable)

  ;; version
  (define-key bee-prefixed-map "\C-vi"        'bee-checkin-project)
  (define-key bee-prefixed-map "\C-vo"        'bee-checkout-file-version)
  (define-key bee-prefixed-map "\C-vd"        'bee-diff-file-version)

  ;; profile
  (define-key bee-prefixed-map "\C-pc"        'bee-compile-for-profile)
  (define-key bee-prefixed-map "\C-pr"        'bee-run-for-profile)
  
  ;; mouse bindings
  (define-key bee-mode-map bee-mouse-binding  'bee-predicate-mouse-event)
  
  ;; the popup menus
  (bee-add-menu '(lambda (event) t)
		'bee-popup-menu)
  (bee-add-menu '(lambda (event) (and (region-active-p) (region-exists-p)))
		'bee-region-popup-menu)
  ;; the menu bar
  (if (featurep 'menubar)
      (progn
	(set-buffer-menubar current-menubar)
	(add-submenu nil (cons "bee" (bee-make-menu)))
	(add-submenu nil (cons "version" (bee-make-version-menu))))))
