;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-mode.el                    */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon May 25 07:49:23 1998                          */
;*    Last change :  Sat Oct 10 13:06:21 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Bee mode declaration.                                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-mode)
(require 'bee-autoload)
(require 'bee-config)
(require 'bee-flock)
(require 'bee-keymap)
(require 'bee-toolbar)
(require 'bee-indent)
(require 'font-lock)
(require 'mic-paren)

;*---------------------------------------------------------------------*/
;*    bee-mode-syntax-table ...                                        */
;*---------------------------------------------------------------------*/
(defvar bee-mode-syntax-table (make-syntax-table) "")

;*---------------------------------------------------------------------*/
;*    bee-init-syntax-table ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-init-syntax-table ()
  (let ((i 0))
    ;; Default is atom-constituent.
    (while (< i 256)
      (modify-syntax-entry i "_   ")
      (setq i (1+ i)))

    ;; Word components.
    (setq i ?0)
    (while (<= i ?9)
      (modify-syntax-entry i "w   ")
      (setq i (1+ i)))
    (setq i ?A)
    (while (<= i ?Z)
      (modify-syntax-entry i "w   ")
      (setq i (1+ i)))
    (setq i ?a)
    (while (<= i ?z)
      (modify-syntax-entry i "w   ")
      (setq i (1+ i)))
    (modify-syntax-entry ?@ "w   ")
    (modify-syntax-entry ?- "w   ")
    (modify-syntax-entry ?_ "w   ")
    (modify-syntax-entry ?! "w   ")
    (modify-syntax-entry ?? "w   ")
    (modify-syntax-entry ?= "w   ")
    (modify-syntax-entry ?< "w   ")
    (modify-syntax-entry ?> "w   ")
    (modify-syntax-entry ?+ "w   ")
    (modify-syntax-entry ?* "w   ")
    (modify-syntax-entry ?/ "w   ")
    (modify-syntax-entry ?~ "w   ")
    (modify-syntax-entry ?$ "w   ")
    (modify-syntax-entry ?% "w   ")
    (modify-syntax-entry ?^ "w   ")
    (modify-syntax-entry ?\\ "w   ")
    (modify-syntax-entry ?. "w   ")

    ;; Whitespace
    (modify-syntax-entry ?\t "    ")
    (modify-syntax-entry ?\n ">   ")
    (modify-syntax-entry ?\f "    ")
    (modify-syntax-entry ?\r "    ")
    (modify-syntax-entry ?  "    ")

    ;; These characters are delimiters but otherwise undefined.
    ;; Brackets and braces balance for editing convenience.
    (modify-syntax-entry ?[ "(]  ")
    (modify-syntax-entry ?] ")[  ")
    (modify-syntax-entry ?{ "(}  ")
    (modify-syntax-entry ?} "){  ")
    (modify-syntax-entry ?\| "    ")

    ;; Other atom delimiters
    (modify-syntax-entry ?\( "()  ")
    (modify-syntax-entry ?\) ")(  ")
    (modify-syntax-entry ?\; "<   ")
    (modify-syntax-entry ?\" "\"    ")
    (modify-syntax-entry ?' "'   ")
    (modify-syntax-entry ?` "'   ")
    (modify-syntax-entry ?\: "'   ")

    ;; Special characters
    (modify-syntax-entry ?, "'   ")
    (modify-syntax-entry ?# "'   ")
    (modify-syntax-entry ?\\ "\\   ")))

;*---------------------------------------------------------------------*/
;*    bee-mode-abbrev-table ...                                        */
;*---------------------------------------------------------------------*/
(defvar bee-mode-abbrev-table nil "")
(define-abbrev-table 'bee-mode-abbrev-table ())

;*---------------------------------------------------------------------*/
;*    bee-mode-variables ...                                           */
;*---------------------------------------------------------------------*/
(defun bee-mode-variables ()
  (setq local-abbrev-table bee-mode-abbrev-table)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat "^$\\|" page-delimiter))
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start)
  (make-local-variable 'paragraph-ignore-fill-prefix)
  (setq paragraph-ignore-fill-prefix t)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'bee-indent-line)
  (make-local-variable 'comment-start)
  (setq comment-start ";")
  (make-local-variable 'comment-start-skip)
  (setq comment-start-skip ";+[ \t]*")
  (make-local-variable 'comment-column)
  (setq comment-column 40)
  (make-local-variable 'comment-indent-hook)
  (setq comment-indent-hook 'bee-comment-indent))

;*---------------------------------------------------------------------*/
;*    bee-paren-init ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-paren-init ()
  (setq paren-face
	(if (and (fboundp 'x-display-color-p)
		 (x-display-color-p))
	    (let ((fn 'paren-face))
	      (copy-face 'default fn)
	      (set-face-background fn "green")
	      fn)
	  'modeline))
  (setq paren-mismatch-face
	(if (and (fboundp 'x-display-color-p)
		 (x-display-color-p))
	    (let ((fn 'paren-mismatch-face))
	      (copy-face 'default fn)
	      (set-face-background fn "red")
	      fn)
	  'modeline))
  (setq paren-no-match-face
	(if (x-display-color-p)
	    (let ((fn 'paren-no-match-face))
	      (copy-face 'default fn)
	      (set-face-background fn "darkorange1")
	      fn)
	  'default)))

;*---------------------------------------------------------------------*/
;*    letn ...                                                         */
;*---------------------------------------------------------------------*/
(defmacro letn (name bdgs &rest body)
  (let ((args  (mapcar 'car bdgs))
	(inits (mapcar '(lambda (x) (car (cdr x))) bdgs)))
    `(let ((,name nil))
       (setq ,name (lambda ,args ,@body))
       (funcall ,name ,@inits))))

;*---------------------------------------------------------------------*/
;*    bee-find-root-directory ...                                      */
;*    -------------------------------------------------------------    */
;*    The root directory is the first directory where we are able      */
;*    to find at least one of the files in FILES.                      */
;*---------------------------------------------------------------------*/
(defun bee-find-root-directory (files)
  (letn loop-dir ((dir (if (stringp (buffer-file-name))
			   (file-name-directory (buffer-file-name))
			 default-directory))
		  (depth bee-root-search-depth))
	(cond
	 ((string= dir "//")
	  default-directory)
	 ((= depth 0)
	  default-directory)
	 (t
	  (letn loop-file ((fs files))
		(if (consp fs)
		    (if (file-exists-p (concat dir "/" (car fs)))
			dir
		      (funcall loop-file (cdr fs)))
		  (funcall loop-dir
			   (concat (expand-file-name (concat dir ".."))
				   "/")
			   (- depth 1))))))))

;*---------------------------------------------------------------------*/
;*    init-root-directory ...                                          */
;*---------------------------------------------------------------------*/
(defun init-root-directory (dir)
  (interactive "DRoot directory: ")
  (if (stringp dir)
      (progn
	(setq dir (expand-file-name dir))
	(if (not (eq (aref dir (- (length dir) 1)) ?/))
	    (setq bee-root-directory (concat dir "/"))
	  (setq bee-root-directory dir))
	(message "Setting bee-root-directory to %S" bee-root-directory))))

;*---------------------------------------------------------------------*/
;*    bee-set-root-directory ...                                       */
;*    -------------------------------------------------------------    */
;*    When the user request a new root directory we have to re-setup   */
;*    all the bee file (the makefile, the .afile and the .etags file). */
;*---------------------------------------------------------------------*/
(defun bee-set-root-directory ()
  (interactive)
  (call-interactively 'init-root-directory)
  (bee-set-makefile (concat bee-root-directory bee-makefile-name))
  (bee-tags-init)
  (bee-modeline-init)
  (bee-docline-init))

;*---------------------------------------------------------------------*/
;*    bee-mode ...                                                     */
;*---------------------------------------------------------------------*/
(defun bee-mode ()
  "Major mode for editing Bigloo code.

Commands:
Delete converts tabs to spaces as it moves back.
Blank lines separate paragraphs.  Semicolons start comments.
\\{bee-mode-map}
Entry to this mode calls the value of bee-mode-hook
if that value is non-nil."
  (interactive)
  (kill-all-local-variables)
  ;; mode declaration
  (setq major-mode 'bee-mode)
  (setq mode-name "Bee")
  (use-local-map bee-mode-map)
  ;; we setup the project root directory
  (init-root-directory 	(bee-find-root-directory (list bee-tags-table
						       bee-afile-name
						       bee-makefile-name)))
  (make-local-variable 'bee-root-directory)
  ;; syntax table
  (set-syntax-table bee-mode-syntax-table)
  (bee-init-syntax-table)
  (bee-mode-variables)
  ;; keymap bindings
  (bee-keymap-init)
  ;; parenthesis blinking init
  (bee-paren-init)
  ;; make init
  (bee-compile-init)
  ;; tags init
  (bee-tags-init)
  ;; starting font-lock
  (if bee-font-lock-p
      (progn
	(setq font-lock-keyword bee-font-lock-keywords)
	(font-lock-mode t)))
  ;; the doc source fontification
  (bee-fontify-doc-source (current-buffer))
  ;; the toolbar
  (bee-toolbar-init)
  ;; the modeline specific init
  (bee-modeline-init)
  ;; the initialize C hooking
  (bee-c-mode-init)
  ;; the bee hook
  (run-hooks 'bee-mode-hook))

;*---------------------------------------------------------------------*/
;*    bee-modeline-id ...                                              */
;*---------------------------------------------------------------------*/
(defvar bee-modeline-id nil)
(make-variable-buffer-local 'bee-modeline-id)
  
;*---------------------------------------------------------------------*/
;*    bee-modeline-init ...                                            */
;*    -------------------------------------------------------------    */
;*    This function change the left glyph for the modeline.            */
;*---------------------------------------------------------------------*/
(defun bee-modeline-init ()
  (let* ((id (if (stringp bee-root-directory)
		 (let* ((text (concat
			       (file-name-nondirectory
				(substring
				 bee-root-directory
				 0
				 (- (length bee-root-directory) 1)))
			       (bee-root-version)))
			(glyph (make-glyph text)))
		   (set-glyph-face glyph 'bee-root-face)
		   glyph)
	       (let ((glyph (make-glyph "no root !")))
		 (set-glyph-face glyph 'bee-no-root-face)
		 glyph))))
    (if (consp bee-modeline-id)
	(rplacd bee-modeline-id id)
      (progn
	(setq bee-modeline-id
	      (cons (copy-extent modeline-buffer-id-left-extent) id))
	(setq mode-line-buffer-identification
	      (cons bee-modeline-id
		    (mapcar '(lambda (prop)
			       (let ((name (cdr prop)))
				 (if (string= "XEmacs%N:" name)
				     (cons (car prop) "%N:")
				   prop)))
			    mode-line-buffer-identification)))))))

