;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-module.el                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Jun 11 11:58:51 1998                          */
;*    Last change :  Thu Sep 17 10:55:37 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Module handling                                                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-module)
(require 'bee-config)
(require 'bee-autoload)

;*---------------------------------------------------------------------*/
;*    bee-module-declaration-p ...                                     */
;*    -------------------------------------------------------------    */
;*    Is POS inside a module declaration?                              */
;*    CLAUSE is the kind of clause we want to parse (for instance it   */
;*    could be "(\\(import\\|use\\)[ \n]")                             */
;*---------------------------------------------------------------------*/
(defun bee-module-declaration-p (pos clause)
  (save-excursion
    (let ((sexp (id-select-sexp pos)))
      (and (consp sexp)
	   (let ((start (car sexp)))
	     (goto-char start)
	     (if (looking-at clause)
		 (let ((sexp (id-select-sexp start)))
		   (and (consp sexp)
			(let ((start (car sexp)))
			  (goto-char start)
			  (looking-at "(module[ \n]"))))))))))

;*---------------------------------------------------------------------*/
;*    bee-find-domestic-definition ...                                 */
;*    -------------------------------------------------------------    */
;*    This function search for a binding definition.                   */
;*---------------------------------------------------------------------*/
(defun bee-find-domestic-definition (ident)
  (save-excursion
    (goto-char (point-min))
    (let ((reg-exp (concat
		    "^(\\(define\\|define-generic\\|define-inline\\)[ \t\n]+(?"
		    (regexp-quote ident)
		    "\\(::\\w\\|\\>\\)")))
      (re-search-forward reg-exp (point-max) t))))

;*---------------------------------------------------------------------*/
;*    bee-export-binding/class ...                                     */
;*---------------------------------------------------------------------*/
(defun bee-export-binding/class (class tagname)
  (save-excursion
    (if (not (bee-find-domestic-definition tagname))
	;; we have found no definition
	(error (format "Can't find binding [%s]" tagname))
      (progn
	(goto-char (match-beginning 0))
	(cond
	 ((and (memq class '(binding function))
	       (re-search-forward "^(define-\\(generic\\|inline\\)[ \t\n]+("
				  (point-max) t))
	  ;; this is an inline or generic function definition
	  (let ((start (- (match-end 0) 1))
		(end   (condition-case ()
			   (progn
			     (backward-char 1)
			     (forward-sexp 1)
			     (point))
			 (error nil))))
	    (if end
		(bee-add-export (concat "("
					(buffer-substring (match-beginning 1)
							  (match-end 1))
					" "
					(buffer-substring (+ 1 start) end)))
	      (error (format "Incorrect definition for %s" tagname)))))
	 ((and (memq class '(binding function))
	       (re-search-forward "^(define[ \t\n]+(" (point-max) t))
	  ;; this is a function definition
	  (let ((start (- (match-end 0) 1))
		(end   (condition-case ()
			   (progn
			     (backward-char 1)
			     (forward-sexp 1)
			     (point))
			 (error nil))))
	    (if end
		(bee-add-export (buffer-substring start end))
	      (error (format "Incorrect definition for %s" tagname)))))
	 ((and (memq class '(binding variable))
	       (re-search-forward "^(define[ \t\n]+" (point-max) t))
	  ;; this is a variable definition
	  (let ((start (match-end 0))
		(end   (condition-case ()
			   (progn
			     (forward-sexp 1)
			     (point))
			 (error nil))))
	    (if end
		(bee-add-export (buffer-substring start end))
	      (error (format "Incorrect definition for %s" tagname)))))
	 (t
	  (error (format "Can't find binding [%s]" tagname))))))))

;*---------------------------------------------------------------------*/
;*    bee-export-binding ...                                           */
;*    -------------------------------------------------------------    */
;*    This function search for a global binding definition and it      */
;*    exports it as a function is the binding is a lambda form or      */
;*    it exports it as a variable if the definition looks like a       */
;*    variable definition.                                             */
;*---------------------------------------------------------------------*/
(defun bee-export-binding (tagname)
  (interactive (bee-interactive-ident (point) "Export binding: "))
  (bee-export-binding/class 'binding tagname))

;*---------------------------------------------------------------------*/
;*    bee-export-function ...                                          */
;*    -------------------------------------------------------------    */
;*    Looks for a function (or a generic function) and exports it.     */
;*---------------------------------------------------------------------*/
(defun bee-export-function (tagname)
  (interactive (bee-interactive-ident (point) "Export function: "))
  (bee-export-binding/class 'function tagname))

;*---------------------------------------------------------------------*/
;*    bee-export-variable ...                                          */
;*    -------------------------------------------------------------    */
;*    Looks for a variable definition and exports it.                  */
;*---------------------------------------------------------------------*/
(defun bee-export-variable (tagname)
  (interactive (bee-interactive-ident (point) "Export variable: "))
  (bee-export-binding/class 'variable tagname))

;*---------------------------------------------------------------------*/
;*    bee-insert-clause ...                                            */
;*---------------------------------------------------------------------*/
(defun bee-insert-clause (clause pos create-p definition)
  (goto-char pos)
  (insert "\n")
  (if create-p (insert (format "(%s " clause)))
  (indent-for-tab-command)
  (insert definition)
  (if create-p (insert ")"))
  (message "%S added to %s clauses" definition clause))

;*---------------------------------------------------------------------*/
;*    bee-add-clause ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-add-clause (clause definition)
  (goto-char (point-min))
  (if (re-search-forward "(module[ \t\n]" (point-max) t)
      (progn
	(goto-char (match-beginning 0))
	(let ((end (condition-case ()
		       (save-excursion
			 (forward-sexp 1)
			 (point))
		     (error nil))))
	  (if end
	      ;; we search for an exportation clause
	      (if (let ((eval t))
		    (while (and eval
				(re-search-forward
				 (format "(%s[ \t\n]" clause)
				 end
				 t))
		      (setq eval (bee-module-declaration-p (match-beginning 0)
							   "(eval[ \n]")))
		    (not eval))
		  (progn
		    (goto-char (match-beginning 0))
		    ;; it may be the export clause of an eval form...
		    (condition-case ()
			(progn
			  (forward-sexp 1)
			  (backward-char 1)
			  (bee-insert-clause clause (point) nil definition))
		      (error
		       (error (format "Illegal %s clause" clause)))))
		(progn
		  (goto-char end)
		  (backward-char 1)
		  (bee-insert-clause clause (point) t definition)))
	    (error "Illegal module clause"))))))

;*---------------------------------------------------------------------*/
;*    bee-add-export ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-add-export (definition)
  (bee-add-clause "export" definition))

;*---------------------------------------------------------------------*/
;*    bee-add-import ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-add-import (definition)
  (bee-add-clause "import" definition))

;*---------------------------------------------------------------------*/
;*    bee-export-definition ...                                        */
;*    -------------------------------------------------------------    */
;*    Export a definition.                                             */
;*---------------------------------------------------------------------*/
(defun bee-export-definition (ident)
  "Export a definition."
  (interactive (bee-interactive-ident (point) "Export: "))
  (if (and (stringp ident) (not (string= ident "")))
      (bee-export-binding/class 'binding ident)))
  
;*---------------------------------------------------------------------*/
;*    bee-import-c-file ...                                            */
;*---------------------------------------------------------------------*/
(defun bee-import-c-file (file)
  (interactive "FImport C file: ")
  (if (stringp file)
      (let* ((cmd (concat bee-cigloo " -s -no-directives " file))
	     (res (exec-to-string cmd))
	     (len (length res)))
	(if (string-match ")[ \n\t]*$" res)
	    ;; no error occured, we insert the result
	    (save-excursion
	      (goto-char (point-min))
	      (if (re-search-forward "(module[ \t\n]" (point-max) t)
		  (progn
		    (goto-char (match-end 0))
		    (condition-case ()
			(progn
			  (forward-sexp 1)
			  (insert "\n")
			  (save-excursion (insert res))
			  (indent-for-tab-command)
			  (bee-indent-sexp)
 			  (bee-add-makefile-entry
			   (concat (file-name-sans-extension file) ".o")
			   file))
		      (error "Illegal module clause" nil)))
		(error "Can't find module clause")))
	  (progn
	    (error (format "Error in C file %s" file))
	    (pop-to-buffer "*bee cigloo errors*")
	    (erase-buffer)
	    (insert res))))))

;*---------------------------------------------------------------------*/
;*    bee-module-declaration-find ...                                  */
;*    -------------------------------------------------------------    */
;*    This function search inside the module declaration a clause      */
;*    for VAR. If found, the clause and the kind (export or static)    */
;*    of the clause is returned.                                       */
;*---------------------------------------------------------------------*/
(defun bee-module-declaration-find (var)
  (save-excursion
    (beginning-of-buffer)
    ;; first we search for the module declaration
    (if (re-search-forward "(module[ \n\t]\\(\\w+\\)" (point-max) t)
	(let ((module (buffer-substring (match-beginning 1) (match-end 1)))
	      (sexp (id-select-sexp-start (match-beginning 0))))
	  (if (consp sexp)
	      ;; we have found the module clause
	      (let ((max-bound (cdr sexp))
		    (found     nil))
		(while (and (not found)
			    (re-search-forward "(\\(export\\|static\\)[ \n\t]"
					       max-bound
					       t))
		  ;; there is another defining clause
		  (let* ((start (match-beginning 0))
			 (clause (id-select-sexp-start start))
			 (kind (buffer-substring (match-beginning 1)
						 (match-end 1))))
		      (goto-char start)
		      (if (re-search-forward 
			   (format "(\\(\\w+[ \n\t]+\\)?%s\\(::\\|[ \n\t]\\)\\|%s"
				   var var)
			   max-bound t)
			  ;; we have found the definition
			  (setq found (cons kind (match-beginning 0)))
			;; no match, we skip
			(goto-char (match-end 0)))))
		(cons module found))
	    (cons module nil)))
      nil)))

;*---------------------------------------------------------------------*/
;*    bee-binding-imported-p ...                                       */
;*    -------------------------------------------------------------    */
;*    This predicate is true iff variable VAR from module MODULE is    */
;*    imported in the current buffer.                                  */
;*---------------------------------------------------------------------*/
(defun bee-binding-imported-p (module var)
  (save-excursion
    (beginning-of-buffer)
    ;; first we search for the module declaration
    (if (re-search-forward "(module[ \n\t]\\(\\w+\\)" (point-max) t)
	(let ((module (buffer-substring (match-beginning 1) (match-end 1)))
	      (sexp (id-select-sexp-start (match-beginning 0)))
	      (module-id (intern module))
	      (var-id (intern var)))
	  (if (consp sexp)
	      ;; we have found the module clause
	      (let ((max-bound (cdr sexp))
		    (found     nil))
		(goto-char (car sexp))
		(while (and (not found)
			    (re-search-forward "(\\(import\\|use\\)[ \n\t]"
					       max-bound
					       t))
		  ;; there is another import clause
		  (let* ((start (match-beginning 0))
			 (clause (id-select-sexp-start start)))
		    (if (consp clause)
			(let* ((end (cdr clause))
			       (str  (buffer-substring (car clause) end))
			       (sexp (car (read-from-string str))))
			  (let ((clauses (cdr sexp)))
			    (while (and (not found) (consp clauses))
			      (let ((clause (car clauses)))
				(cond
				 ((eq clause module-id)
				  (setq found t))
				 ((and (consp clause)
				       (eq (car clause) var-id)
				       (consp (cdr clause))
				       (eq (car (cdr clause)) module-id))
				  (setq found t))
				 (t
				  (setq clauses (cdr clauses)))))))
			  (goto-char end))
		      (error "Illegal import clause -- %S" module))))
		found)
	    (error "Illegal module clause -- %S" module)))
      (error "Illegal module clause -- %S" module))))

;*---------------------------------------------------------------------*/
;*    bee-get-module-name ...                                          */
;*---------------------------------------------------------------------*/
(defun bee-get-module-name (buffer)
  (save-excursion
    (set-buffer buffer)
    (beginning-of-buffer)
    ;; first we search for the module declaration
    (if (re-search-forward "(module[ \n\t]\\(\\w+\\)" (point-max) t)
	(buffer-substring (match-beginning 1) (match-end 1))
      nil)))
  
;*---------------------------------------------------------------------*/
;*    bee-assert-find ...                                              */
;*    -------------------------------------------------------------    */
;*    This function search for all assertion in SEXP.                  */
;*---------------------------------------------------------------------*/
(defun bee-assert-find (sexp)
  (let ((start  (car sexp))
	(end    (cdr sexp))
	(assert '()))
    (save-excursion
      (goto-char start)
      (while (re-search-forward "[(\\[]assert[ \n\t]" end t)
	(let ((sexp (id-select-sexp-start (match-beginning 0))))
	  (if (consp sexp)
	      (progn
		(setq assert (cons (buffer-substring (car sexp) (cdr sexp))
				   assert))
		(goto-char (cdr sexp))))))
      (nreverse assert))))

;*---------------------------------------------------------------------*/
;*    bee-usage-find ...                                               */
;*    -------------------------------------------------------------    */
;*    This function is not exact because it does not open the include  */
;*    file in order to read the directives clauses.                    */
;*---------------------------------------------------------------------*/
(defun bee-usage-find (ident)		     
  "Find documentation."
  (interactive (bee-interactive-ident (point) "Doc: "))
  (let ((buffer (bee-tags-find-variable-noselect ident)))
    (if (bufferp buffer)
	;; we have found the definition for this variable
	(progn
	  (set-buffer buffer)
	  ;; we fetch the s-expression that defines the variable
	  (let ((sexp (id-select-sexp-start (point))))
	    ;; if check for errors
	    (if (not (consp sexp))
		(error (format "Illegal definition -- %S" ident))
	      (let* ((assert (bee-assert-find sexp))
		     (define-point (point))
		     (module (bee-module-declaration-find ident))
		     (menu   (append
			      (list (make-module-entry module buffer)
				    (make-export-entry module)
				    (make-decl-entry module buffer)
				    (make-define-entry define-point buffer)
				    (make-use-entry ident module))
			      (make-assert-entry assert))))
		(popup-menu (cons ident menu)))))))))

;*---------------------------------------------------------------------*/
;*    make-define-entry ...                                            */
;*---------------------------------------------------------------------*/
(defun make-define-entry (pos buffer)
  (let ((define (buffer-substring pos
				  (save-excursion
				    (end-of-line)
				    (point)))))
    (vector (concat "def   : " define)
	    `(let ((pop-up-frames t))
	       (pop-to-buffer ,buffer)
	       (goto-char ,pos))
	    t)))

;*---------------------------------------------------------------------*/
;*    make-module-entry ...                                            */
;*---------------------------------------------------------------------*/
(defun make-module-entry (module buffer)
  (if (consp module)
      (vector (concat "module: " (car module))
	      `(let ((pop-up-frames t))
	       (pop-to-buffer ,buffer)
	       (beginning-of-buffer))
	      t)
    (vector "????" '() t)))

;*---------------------------------------------------------------------*/
;*    make-decl-entry ...                                              */
;*---------------------------------------------------------------------*/
(defun make-decl-entry (module buffer)
  (if (consp (cdr module))
      (let* ((start (cdr (cdr module)))
	     (decl (id-select-sexp-start start)))
	(vector (concat "decl  : " (buffer-substring (car decl) (cdr decl)))
		`(let ((pop-up-frames t))
		   (pop-to-buffer ,buffer)
		   (goto-char ,(car decl)))
		t))
    (vector "decl  : implicit" '() t)))

;*---------------------------------------------------------------------*/
;*    make-export-entry ...                                            */
;*---------------------------------------------------------------------*/
(defun make-export-entry (module)
  (if (consp (cdr module))
      (concat "scope : " (car (cdr module)))
    "scope : static"))

;*---------------------------------------------------------------------*/
;*    make-use-entry ...                                               */
;*---------------------------------------------------------------------*/
(defun make-use-entry (ident module)
  (let ((static (or (not (consp (cdr module)))
		    (string= (car (cdr module)) "static"))))
    (vector "Calls and refs..." `(bee-find-usage ,ident ,static) t)))
  
;*---------------------------------------------------------------------*/
;*    make-assert-entry ...                                            */
;*---------------------------------------------------------------------*/
(defun make-assert-entry (asserts)
  (if (null asserts)
      asserts
    (list (cons "Assertions..."
		(mapcar '(lambda (assert)
			   (vector (format "%S" assert) '() t))
			asserts)))))

;*---------------------------------------------------------------------*/
;*    bee-import-binding ...                                           */
;*---------------------------------------------------------------------*/
(defun bee-import-binding (var)
  ;; first we have to find in which module this variable is defined
  "Find documentation."
  (interactive (bee-interactive-ident (point) "Import binding: "))
  (let ((current (current-buffer))
	(buffer (bee-tags-find-variable-noselect var)))
    (if (bufferp buffer)
	;; we have found the definition for this variable
	(let ((current-module (bee-get-module-name (current-buffer))))
	  (set-buffer buffer)
	  ;; then we have to find the module clause for that variable
	  (let* ((module (bee-module-declaration-find var))
		 (import-module (if (consp module)
				    (car module)
				  (error
				   (format "Illegal module declaration -- %S"
					   var)))))
	    (cond
	     ((string= current-module import-module)
	      (error (format "The variable is defined in the module -- %S"
			     var)))
	     ((or (null (cdr module))
		  (not (string= (car (cdr module)) "export")))
	      (error (format "Variable %S is %S in %S"
			     var
			     (car (cdr module))
			     import-module)))
	     ((bee-binding-imported-p import-module var)
	      (error (format "Variable %S is already imported from %S"
			     var import-module)))
	     (t
	      (set-buffer current)
	      (save-excursion
		(bee-add-import (format "(%s %s)" var import-module))))))))))
	     
;*---------------------------------------------------------------------*/
;*    grep keymap                                                      */
;*---------------------------------------------------------------------*/
(defvar bee-grep-mouse-map (make-sparse-keymap))
(define-key bee-grep-mouse-map [(button2)]
  (function bee-grep-visit))

;*---------------------------------------------------------------------*/
;*    bee-find-usage ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-find-usage (ident static)
  (let ((files (if static
		   (file-relative-name (buffer-file-name (current-buffer))
				       bee-root-directory)
		 (bee-make-entry "getsources"))))
    (if (or (string= files "")
	    (string-match "getsources" files))
	(error "Can't find source files")
      (let* ((cmd (format "cd %s; grep --with-filename -n -E -e \"^[^;]+\\(?%s[ \t\n)]\" %s"
			  bee-root-directory
			  ident
			  files))
	     (res (exec-to-string cmd)))
	(if (not (string= res ""))
	    (let ((buffer (get-buffer-create (format "*Calls-and-Refs-%s*"
						     ident))))
	      (set-buffer buffer)
	      (erase-buffer)
	      (insert res)
	      (let* ((height (save-excursion
			       (+ 2 (count-lines (point-min) (point-max)))))
		     (cur-height (frame-height (selected-frame)))
		     (new-height (if (> height cur-height)
				     cur-height
				   height))
		     (default-frame-alist (cons 'height
						(cons new-height
						      default-frame-alist)))
		     (pop-up-frames t)
		     (regexp "^\\([a-zA-Z]?:?[^:( \t\n]+\\)[:( \t]+\\([0-9]+\\)[:) \t].*$"))
		(beginning-of-buffer)
		(while (re-search-forward regexp (point-max) t)
		  (let* ((file (buffer-substring (match-beginning 1)
						 (match-end 1)))
			 (line (buffer-substring (match-beginning 2)
						 (match-end 2)))
			 (end  (match-end 0))
			 (extent (make-extent (match-beginning 0) end)))
		    (set-extent-property extent
					 'grep (cons file line))
		    (set-extent-property extent
					 'mouse-face 'highlight)
		    (set-extent-property extent
					 'keymap bee-grep-mouse-map)
		    (goto-char end)))
		(pop-to-buffer buffer)))
	  (error (format "Nothing found about %S" ident)))))))

;*---------------------------------------------------------------------*/
;*    bee-grep-visit ...                                               */
;*---------------------------------------------------------------------*/
(defun bee-grep-visit (event)
  (interactive "e")
  (let* ((point  (event-closest-point event))
	 (buffer (event-buffer event))
	 (extent (extent-at point buffer 'grep))
	 (prop   (extent-property extent 'grep)))
    (if (consp prop)
	(let ((file (car prop))
	      (line (cdr prop)))
	  (if (stringp file)
	      (let* ((default-directory bee-root-directory)
		     (buffer (find-file-noselect file)))
		(if (bufferp buffer)
		    (progn
		      (set-buffer buffer)
		      (beginning-of-buffer)
		      (goto-line (string-to-int line))
		      (let ((pop-up-frames t))
			(pop-to-buffer buffer)))
		  (error "Can't find buffer for %S" file))))))))

