;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-prof.el                    */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Aug 12 08:40:54 1998                          */
;*    Last change :  Thu Oct 29 07:31:01 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The BEE profiler.                                                */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-prof)
(require 'bee-config)
(require 'bee-autoload)

;*---------------------------------------------------------------------*/
;*    bee-compile-for-profile ...                                      */
;*---------------------------------------------------------------------*/
(defun bee-compile-for-profile ()
  (interactive)
  (let ((compile-command (format "%s -f %s profile"
				 bee-make
				 (bee-get-makefile))))
    (bee-compile)))

;*---------------------------------------------------------------------*/
;*    bee-profile-default-args ...                                     */
;*---------------------------------------------------------------------*/
(defvar bee-profile-default-args "")

;*---------------------------------------------------------------------*/
;*    bee-run-for-profile ...                                          */
;*---------------------------------------------------------------------*/
(defun bee-run-for-profile (arg)
  (interactive
   (let ((arg (read-string (format "Run argument: [%s] "
				   bee-profile-default-args))))
     (list (if (and (stringp arg) (> (length arg) 0))
	       arg
	     bee-profile-default-args))))
  ;; we remember the run argument for later profile
  (setq bee-profile-default-args arg)
  ;; we setup success hook
  (bee-success-hook '(lambda (buffer msg)
		       (if (file-exists-p "PROF")
			   (let ((buf (find-buffer-visiting "PROF")))
			     (if (bufferp buf)
				 (let ((win (get-buffer-window buf t)))
				   (set-buffer buf)
				   (if (windowp win)
				       (progn
					 (if (not (one-window-p window))
					     (delete-window window)
					   (delete-frame (window-frame win)))))
				   (kill-buffer (buffer-name buf))))
			     (let ((buffer (find-file-other-frame "PROF")))
			       (bee-prof-highlight buffer))))))
  ;; we compile
  (let* ((compile-command (format "%s -f %s showprof PROFILE_ARGS=\"%s\""
				  bee-make
				  (bee-get-makefile)
				  arg))) 
    (bee-compile)))

;*---------------------------------------------------------------------*/
;*    doc source keymap                                                */
;*---------------------------------------------------------------------*/
(defvar bee-prof-mouse-map (make-sparse-keymap))
(define-key bee-prof-mouse-map [(button2)]
  (function bee-prof-find))

(defvar regexp "")

;*---------------------------------------------------------------------*/
;*    bee-prof-highlight ...                                           */
;*---------------------------------------------------------------------*/
(defun bee-prof-highlight (buffer)
  (set-buffer buffer)
  (let* ((scm-id-regexp "^[][0-9./ \t]+\\([A-Z_!?+.0-9@#$%^&*/<>-]+\\)[ \t]*\\(\\[[0-9]+\\]\\)?$")
	 (scm-mod-regexp "^[][0-9./ \t]+@\\([A-Z_!?+.0-9@#$%^&*/<>-]+\\)[ \t]*\\(\\[[0-9]+\\]\\)?$")
	 (c-regexp "^[][0-9./ \t]+[a-zA-Z_0-9]+[ t]*\\(\\[[0-9]+\\]\\)?$")
	 (union-regexp (concat "\\(" scm-id-regexp "\\)\\|\\(" c-regexp "\\)"))
	 (case-fold-search nil))
    (setq regexp scm-id-regexp)
    (beginning-of-buffer)
    (while (re-search-forward union-regexp (point-max) t)
      (goto-char (match-beginning 0))
      (let ((end (match-end 0)))
	(cond
	 ((re-search-forward scm-mod-regexp end t)
	  ;; a path addition
	  (let ((ident (buffer-substring (match-beginning 1) (match-end 1)))
		(extent (make-extent (match-beginning 0)
				     (match-end 0))))
	    (set-extent-face extent bee-module-prof-face)
	    (set-extent-property extent 'prof (cons 'module ident))
	    (set-extent-property extent 'mouse-face 'highlight)
	    (set-extent-property extent 'keymap bee-prof-mouse-map)))
	 ((re-search-forward scm-id-regexp end t)
	  ;; a path addition
	  (let ((ident (buffer-substring (match-beginning 1) (match-end 1)))
		(extent (make-extent (match-beginning 0)
				     (match-end 0))))
	    (set-extent-face extent bee-ident-prof-face)
	    (set-extent-property extent 'prof (cons 'var ident))
	    (set-extent-property extent 'mouse-face 'highlight)
	    (set-extent-property extent 'keymap bee-prof-mouse-map)))
	 ((re-search-forward c-regexp end t)
	  ;; a C function
	  (let ((extent (make-extent (match-beginning 0)
				     (match-end 0))))
	    (set-extent-face extent 'bee-c-prof-face)))
	 (t
	  (error (format "Illegal match %S" end))))
	(goto-char end)))
    (beginning-of-buffer)))

;*---------------------------------------------------------------------*/
;*    bee-prof-find ...                                                */
;*---------------------------------------------------------------------*/
(defun bee-prof-find (event)
  (interactive "e")
  (let* ((extent (event-glyph-extent event))
	 (point  (event-closest-point event))
	 (buffer (event-buffer event))
	 (extent (extent-at point buffer 'prof))
	 (prop   (extent-property extent 'prof)))
    (cond
     ((and (consp prop) (eq (car prop) 'var))
      (bee-tags-find-variable (cdr prop)))
     ((and (consp prop) (eq (car prop) 'module))
      (bee-tags-find-module (cdr prop))))))
     
      
    
