;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-repl.el                    */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon May 25 16:31:35 1998                          */
;*    Last change :  Fri Aug 21 09:32:36 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    This file implements a Bigloo repl process embedded in emacs.    */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-repl)
(require 'bee-config)
(require 'comint)

;*---------------------------------------------------------------------*/
;*    bee-repl-buffer ...                                              */
;*---------------------------------------------------------------------*/
(defvar bee-repl-buffer nil)

;*---------------------------------------------------------------------*/
;*    bee-repl ...                                                     */
;*---------------------------------------------------------------------*/
(defun bee-repl ()
  (let ((bufname (concat "*" bee-bigloo "*")))
    (switch-to-buffer-other-frame bufname)
    (comint-run bee-bigloo)
    (setq bee-repl-buffer (current-buffer))
    (make-variable-buffer-local 'comint-prompt-regexp)
    (setq comint-prompt-regexp bee-bigloo-prompt)
    (process-kill-without-query (get-buffer-process bee-repl-buffer))
    (set-process-sentinel (get-buffer-process bee-repl-buffer)
			  'bee-repl-sentinel)))

;*---------------------------------------------------------------------*/
;*    bee-repl-sentinel ...                                            */
;*    -------------------------------------------------------------    */
;*    This function is called when the bee running process changes     */
;*    of state. This means that the process has been killed or         */
;*    stopped or anything else like that.                              */
;*---------------------------------------------------------------------*/
(defun bee-repl-sentinel (proc msg)
  (cond
   ((null (buffer-name (process-buffer proc)))
    ;; the buffer has been killed. We stop displaying
    ;; arrow in the source files.
    (set-process-buffer proc nil))
   ((memq (process-status proc) '(signal exit))
    (let ((window (get-buffer-window bee-repl-buffer t)))
      (if (one-window-p window)
	  (let ((frame (window-frame window)))
	    (delete-frame frame))))
    (kill-buffer bee-repl-buffer)
    (setq bee-repl-buffer nil))))

;*---------------------------------------------------------------------*/
;*    bee-repl-other-frame ...                                         */
;*---------------------------------------------------------------------*/
(defun bee-repl-other-frame ()
  (interactive)
  (if (bufferp bee-repl-buffer)
      (let ((pop-up-frames t))
	(display-buffer bee-repl-buffer))
    (bee-repl)))

;*---------------------------------------------------------------------*/
;*    bee-repl-send-region ...                                         */
;*---------------------------------------------------------------------*/
(defun bee-repl-send-region (beg end)
  (interactive "r")
  (if (not (bufferp bee-repl-buffer))
      (let ((buffer (current-buffer)))
	(bee-repl)
	(set-buffer buffer)))
  (let ((command (buffer-substring beg end))
	(proc    (get-buffer-process bee-repl-buffer)))
    (set-buffer bee-repl-buffer)
    (goto-char (process-mark proc))
    (delete-region (point) (point-max))
    (insert command)
    (let* ((pmark (process-mark proc))
	   (intxt (if (>= (point) (marker-position pmark))
		      (progn (if comint-eol-on-send (end-of-line))
			     (buffer-substring pmark (point)))
		    nil)))
      (comint-send-input)
      (goto-char (point-max)))))

;*---------------------------------------------------------------------*/
;*    bee-repl-send-define ...                                         */
;*---------------------------------------------------------------------*/
(defun bee-repl-send-define ()
  "Send the current definition to the inferior Bigloo process."
  (interactive)
  (save-excursion
    (end-of-defun)
    (let ((end (point)))
      (if (beginning-of-defun)
	  (bee-repl-send-region (point) end)))))

;*---------------------------------------------------------------------*/
;*    bee-repl-send-last-sexp ...                                      */
;*---------------------------------------------------------------------*/
(defun bee-repl-send-last-sexp ()
  "Send the last s-expression to the inferior Bigloo process."
  (interactive)
  (save-excursion
    (forward-sexp -1)
    (let ((start (point)))
      (forward-sexp 1)
      (bee-repl-send-region start (point)))))

;*---------------------------------------------------------------------*/
;*    bee-repl-send-buffer ...                                         */
;*---------------------------------------------------------------------*/
(defun bee-repl-send-buffer ()
  "Send the whole buffer to the inferior Bigloo process."
  (interactive)
  (bee-repl-send-region (point-min) (point-max)))

;*---------------------------------------------------------------------*/
;*    bee-repl-send-toplevel-sexp ...                                  */
;*---------------------------------------------------------------------*/
(defun bee-repl-send-toplevel-sexp (pos)
  "Send the current at POS s-expression to the inferiror Bigloo process."
  (interactive "dPosition: ")
  (let ((sexp (bee-find-toplevel-sexp pos)))
    (if (consp sexp)
	(bee-repl-send-region (car sexp) (cdr sexp))
      (error "Corrupted toplevel sexp"))))
