;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-tags.el                    */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu May 28 12:00:29 1998                          */
;*    Last change :  Thu Aug 13 07:37:57 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bee tags (search) handling                                   */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-tags)
(require 'bee-tools)
(require 'bee-module)
(require 'bee-config)
(require 'etags)

;*---------------------------------------------------------------------*/
;*    Buffer local tags tables ...                                     */
;*---------------------------------------------------------------------*/
(defvar bee-tags-tags-file
  ".etags"
  "The tags file.")
(make-variable-buffer-local 'bee-tags-tags-file)

(defvar bee-tags-afile-file
  ".afile"
  "The afile file.")
(make-variable-buffer-local 'bee-tags-afile-file)

;*---------------------------------------------------------------------*/
;*    bee-tags-find-module ...                                         */
;*    -------------------------------------------------------------    */
;*    Find a module implementation.                                    */
;*---------------------------------------------------------------------*/
(defun bee-tags-find-module (module)
  "Find a module implementation."
  (interactive "sModule: ")
  (if (or (not (stringp bee-tags-afile-file))
	  (not (file-exists-p bee-tags-afile-file)))
      (error "Find module: no afile file loaded.")
    (let* ((cmd (concat "grep -i \"" module " \" " bee-tags-afile-file))
	   (res (exec-to-string cmd)))
      (if (> (length res) 0)
	  (let ((cell (read res)))
	    (if (consp cell)
		(let ((fname (car (cdr cell)))
		      (dir   (file-name-directory bee-tags-afile-file)))
		  (find-file-other-frame
		   (if dir (concat dir fname) fname)))
	      (error (format "Find module: corrupted afile -- %S [%S]"
			     bee-tags-afile-file
			     cmd))))
	(error "Find module: can't find module -- %S" module)))))
 
;*---------------------------------------------------------------------*/
;*    bee-tags-find-variable ...                                       */
;*    -------------------------------------------------------------    */
;*    Find a variable or class definition.                             */
;*---------------------------------------------------------------------*/
(defun bee-tags-find-variable (var)
  "Find a variable or class definition."
  (interactive "sVariable: ")
  (if (or (not (stringp bee-tags-tags-file))
	  (not (file-exists-p bee-tags-tags-file)))
      (error "Find variable: no tags file loaded.")
    (find-tag-other-frame var)))

;*---------------------------------------------------------------------*/
;*    bee-tags-find-variable-noselect ...                              */
;*    -------------------------------------------------------------    */
;*    Find a buffer defining VAR in a buffer that is not selected.     */
;*---------------------------------------------------------------------*/
(defun bee-tags-find-variable-noselect (var)
  "Find a variable or class definition."
  (interactive "sVariable: ")
  (if (or (not (stringp bee-tags-tags-file))
	  (not (file-exists-p bee-tags-tags-file)))
      (error "Find variable: no tags file loaded.")
    (find-tag-noselect var)))

;*---------------------------------------------------------------------*/
;*    bee-tags-find ...                                                */
;*    -------------------------------------------------------------    */
;*    This function is a dispatcher. It checks (with the current       */
;*    position) if it should look for a module or a variable           */
;*    definition. It looks for a module if (point) is inside an        */
;*    `import' or `use' module clause.                                 */
;*---------------------------------------------------------------------*/
(defun bee-tags-find ()
  "Find a variable/class/module definition."
  (interactive)
  (let* ((pos (point))
	 (ident (bee-fetch-identifier pos)))
    (if (stringp ident)
	(if (bee-module-declaration-p pos "(\\(import\\|use\\)[ \n]")
	    (bee-tags-find-module ident)
	  (bee-tags-find-variable ident))
      (let ((ident (read-string "Find definition: ")))
	(if (not (string= ident ""))
	    (bee-tags-find-variable ident))))))

;*---------------------------------------------------------------------*/
;*    bee-c-tags-find ...                                              */
;*---------------------------------------------------------------------*/
(defun bee-c-tags-find ()
  "Find a C definition."
  (interactive)
  (let ((ident (bee-get-c-identifier-at)))
    (if (stringp ident)
	(bee-tags-find-variable ident)
      (let ((ident (read-string "Find C definition: ")))
	(if (not (string= ident ""))
	    (bee-tags-find-variable ident))))))
  
;*---------------------------------------------------------------------*/
;*    bee-tags-init ...                                                */
;*    -------------------------------------------------------------    */
;*    This function search for a .etags file. We go up through         */
;*    the file tree. At the same time, we look for an .afile file.     */
;*---------------------------------------------------------------------*/
(defun bee-tags-init ()
  (setq bee-tags-afile-file (concat bee-root-directory bee-afile-name))
  (setq bee-tags-tags-file (concat bee-root-directory bee-tags-table))
  (setq tags-file-name bee-tags-tags-file)
  (setq tags-add-tables t))
