;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-toolbar.el                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Jun  5 21:03:27 1998                          */
;*    Last change :  Wed Sep 30 07:56:36 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bee toolbar system.                                          */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-toolbar)
(require 'bee-autoload)
(require 'bdb-autoload)

;*---------------------------------------------------------------------*/
;*    bee opened toolbar ...                                           */
;*---------------------------------------------------------------------*/
(defvar bee-opened-toolbar
  '(;;close button
    [bee-close-toolbar-button bee-close-toolbar t "bee toolbar"]
			      
    [:style 2d :size 2]
    ;; the tag button
    [bee-btag-button bee-tags-find t "Find definition"]
    ;; the doc buttom
    [bee-usage-button bee-usage-find t "Definition documentation"]
    [:style 2d :size 2]

    ;; the indent button
    [bee-indent-button bee-external-indent t "Indent whole buffer"]
    [:style 2d :size 2]
    
    ;; the import button
    [bee-import-button bee-import-binding t "Import definition"]
    ;; the export button
    [bee-export-button bee-export-definition t "Export definition"]
    ;; the C importation button
    [bee-c-import-button bee-import-c-file t "Import C definition"]
    [:style 2d :size 2]

    ;; the compile button
    [bee-compile-button bee-compile t "Compile"]
    ;; the root button
    [bee-root-button bee-set-root-directory t "Set Bee root directory"]
    ;; the bmake button
    [bee-bmake-button bee-generate/update-makefile t "Generate/update Makefile"]
    ;; the bmake entry button
    [bee-bmake-entry-button bee-add-user-makefile-entry t "Add Makefile entry"]
    [:style 2d :size 2]
    
    ;; the repl button
    [bee-repl-button bee-repl-other-frame t "Start a read-eval-print loop"]
    [:style 2d :size 2]
    ;; the bdb button
    [bee-bdb-button bdb-other-frame t "Bdb other frame"]
    ;; the bdb connect button
    [bee-connect-button bdb-toggle-connect-buffer t "Bdb connect buffer"]
    [:style 2d :size 2]
    
    nil
;*     ;; the help button                                              */
;*     [:style 2d :size 2]                                             */
;*     [bee-help-button bee-compiler-help t                            */
;*     "Summary of the compiler invokation options"]                   */
    ;; the info button
    [bee-info-button bee-docline t
    "The online documentation for Bee"]))

;*---------------------------------------------------------------------*/
;*    bee-closed-toolbar ...                                           */
;*---------------------------------------------------------------------*/
(defvar bee-closed-toolbar
  '(;;open button
    [bee-open-toolbar-button bee-open-toolbar t "bee toolbar"]))

;*---------------------------------------------------------------------*/
;*    Opening/closing the toolbar ...                                  */
;*---------------------------------------------------------------------*/
(defun bee-close-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bee-closed-toolbar-height)))
    (set-specifier default-toolbar bee-closed-toolbar (current-buffer))
    (set-specifier default-toolbar-height new-height) 
    (bee-shrink-frame-for-toolbar (- old-height new-height))))

(defun bee-open-toolbar ()
  (let ((old-height (specifier-instance default-toolbar-height))
	(new-height (bee-opened-toolbar-height)))
    (set-specifier default-toolbar bee-opened-toolbar (current-buffer))
    (set-specifier default-toolbar-height new-height)
    (bee-shrink-frame-for-toolbar (- old-height new-height))))

;*---------------------------------------------------------------------*/
;*    bee-shrink-frame-for-toolbar ...                                 */
;*---------------------------------------------------------------------*/
(defun bee-shrink-frame-for-toolbar (delta)
  (let* ((frame (selected-frame))
	 (fr-height (frame-pixel-height frame)))
    (if (< delta 0)
	(while (> (- (frame-pixel-height frame) fr-height) delta)
	  (set-frame-height frame (- (frame-height frame) 1)))
      (while (< (- (frame-pixel-height frame) fr-height) delta)
	(set-frame-height frame (+ (frame-height frame) 1))))))

;*---------------------------------------------------------------------*/
;*    bee-toolbar-init ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-toolbar-init ()
  (set-specifier default-toolbar-visible-p t)
  (set-specifier default-toolbar bee-opened-toolbar (current-buffer)))

;*---------------------------------------------------------------------*/
;*    The bee toolbar buttons                                          */
;*---------------------------------------------------------------------*/
(defvar bee-close-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *on[] = {
/* width height num_colors chars_per_pixel */
\"    8    30        9            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #808080\",
\"d c #c0c0c0\",
\"e c #e0e4e0\",
\"f c #f8e8c8\",
\"g c #f8fcf8\",
/* pixels */
\"eddddddd\",
\"eddddddd\",
\"e#####cd\",
\"ed#aagdd\",
\"edd#gddd\",
\"eddddddd\",
\"eddddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"edddgddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"egddd#dd\",
\"ed#ddddd\",
\"eddddddd\",
\"eddddddd\",
};"))

(defvar bee-open-toolbar-button
  (toolbar-make-button-list
   "/* XPM */
static char *off[] = {
/* width height num_colors chars_per_pixel */
\"    27    5        8            1\",
/* colors */
\". c #000000\",
\"# c #303060\",
\"a c #6064c8\",
\"b c #686868\",
\"c c #c0c0c0\",
\"d c #e0e4e0\",
\"e c #f8e8c8\",
\"f c #f8fcf8\",
/* pixels */
\"ccc#ccccccfccfccfccfccfcccc\",
\"ccc##cccccc#cc#cc#cc#cc#ccc\",
\"ccc#a#cccccfccfccfccfccfccc\",
\"ccc#afcccccc.cc.cc.cc.cc.cc\",
\"ccc#fcccccccccccccccccccccc\",
};"))

(defvar bee-btag-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",                  
\"       c #b6b6b2b2b6b6 s backgroundToolBarColor\",      
\".      c #B6B6B2B2B6B6\",      
\"X      c #000000001010\",      
\"o      c #A6A6A2A2A6A6\",      
\"O      c #000000000000\",      
\"+      c #FFFFFBFBCFCF\",      
\"@      c #D7D7B6B68E8E\",      
\"#      c #86867D7D7171\",      
\"%      c #FFFF00000000\",                   
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"     OOOOOO  %%%  %   %%         \",
\"              %  % % %           \",
\"     OOOOOO   %  %%% % %         \",
\"              %  % %  %%         \",
\"     OOOOOO                      \",
\"                   O             \",
\"     OOOOOO        O             \",
\"                   O             \",
\"     OOOOOO   O    O             \",
\"             O    O              \",
\"     %%%%%% OOOOOO               \",
\"             O                   \",
\"     OOOOOO   O                  \",
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"     OOOOOO                      \",
\"                                 \",
\"                                 \"
\"      OOOO O         O           \",
\"      O              O           \",
\"      O    O O O   OOO           \",
\"      OOOO O OO O O  O           \",
\"      O    O O  O O  O           \",
\"      O    O O  O O  O           \",
\"      O    O O  O  OOO           \",
\"                                 \",
\"                                 \",
};")
  "*The btag button.")

(defvar bee-usage-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",                  
\"       c #b6b6b2b2b6b6 s backgroundToolBarColor\",      
\".      c #B6B6B2B2B6B6\",      
\"X      c #000000001010\",      
\"o      c #A6A6A2A2A6A6\",      
\"O      c #000000000000\",      
\"+      c #FFFFFBFBCFCF\",      
\"@      c #D7D7B6B68E8E\",      
\"#      c #86867D7D7171\",      
\"%      c #ff0000\",                   
\"                                 \",
\"     OOOOOO       %%             \",
\"                 %%%%            \",
\"     OOOOOO      %%%%            \",
\"                  %%             \",
\"     OOOOOO                      \",
\"                %%%%%            \",
\"     OOOOOO      %%%%            \",
\"                 %%%%            \",
\"     OOOOOO      %%%%            \",
\"                 %%%%            \",
\"     OOOOOO      %%%%            \",
\"                %%%%%%           \",
\"     OOOOOO                      \",
\"                   O             \",
\"     OOOOOO        O             \",
\"                   O             \",
\"     OOOOOO   O    O             \",
\"             O    O              \",
\"     %%%%%% OOOOOO               \",
\"             O                   \",
\"     OOOOOO   O                  \",
\"                                 \",
\"                                 \",
\"          OOO                    \",
\"          O  O                   \",
\"          O  O  OO   OOO         \",
\"          O  O O  O O            \",
\"          O  O O  O O            \",
\"          O  O O  O O            \",
\"          OOO   OO   OOO         \",
\"                                 \",
\"                                 \",
};")
  "*The doc button.")

(defvar bee-export-button
  (toolbar-make-button-list
   "/* XPM */
static char *down[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        8            1\",
/* colors */
\". c #330099\",
\"+ c #888888\",
\"a c #aaaaaa\",
\"_ c #b2b2b2 s backgroundToolBarColor\",
\"c c #cccccc\",
\"d c #eeeeee\",
\"e c #ffffff\",
\"# c black\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_______________._a_______________\",
\"______________.e.+a______________\",
\"_____________.edd.+a_____________\",
\"____________.edccd.+a____________\",
\"___________.edccccd.+a___________\",
\"__________....dcc....+a__________\",
\"_____________.dcc.+++++a_________\",
\"_____________.dcc.+______________\",
\"_____________.dcc.+______________\",
\"_____________.dcc.+______________\",
\"_____________.dcc.+______________\",
\"_____________.....+______________\",
\"_______________a+++______________\",
\"_____________a...a_______________\",
\"_____________.+d+.+______________\",
\"_____________.ddc.+______________\",
\"_____________.+c+.+______________\",
\"_____________a...+a______________\",
\"______________+++aa______________\",
\"______________aaaaa______________\",
\"_________________________________\"
\"_________________________________\",
\"__####_____________________#_____\",
\"__#________________________#_____\",
\"__#____#__#_###___##__###__#_____\",
\"__###__#__#_#__#_#__#_#__#_###___\",
\"__#_____##__#__#_#__#_#____#_____\",
\"__#____#__#_#__#_#__#_#____#_____\",
\"__####_#__#_###___##__#_____###__\",
\"____________#____________________\",
\"____________#____________________\",
};")
  "*The export button.")

(defvar bee-import-button
  (toolbar-make-button-list
   "/* XPM */
static char *down[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        8            1\",
/* colors */
\". c #330099\",
\"+ c #888888\",
\"a c #aaaaaa\",
\"_ c #b2b2b2 s backgroundToolBarColor\",
\"c c #cccccc\",
\"d c #eeeeee\",
\"e c #ffffff\",
\"# c black\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_____________.....+______________\",
\"_____________.dcc.+______________\",
\"_____________.dcc.+______________\",
\"_____________.dcc.+______________\",
\"_____________.dcc.+______________\",
\"_____________.dcc.+++++a_________\",
\"__________....dcc....+a__________\",
\"___________.edccccd.+a___________\",
\"____________.edccd.+a____________\",
\"_____________.edd.+a_____________\",
\"______________.e.+a______________\",
\"_______________._a_______________\",
\"______________aaaaa______________\",
\"______________+++aa______________\",
\"_____________a...+a______________\",
\"_____________.+c+.+______________\",
\"_____________.ddc.+______________\",
\"_____________.+d+.+______________\",
\"_____________a...a_______________\",
\"_______________a+________________\",
\"_________________________________\"
\"_________________________________\",
\"___________________________#_____\",
\"____#______________________#_____\",
\"____#__###__###___##__###__#_____\",
\"____#_#_#_#_#__#_#__#_#__#_###___\",
\"____#_#_#_#_#__#_#__#_#____#_____\",
\"____#_#_#_#_#__#_#__#_#____#_____\",
\"____#_#_#_#_###___##__#_____###__\",
\"____________#____________________\",
\"____________#____________________\",
};")
  "*The import button.")

(defvar bee-c-import-button
  (toolbar-make-button-list
   "/* XPM */
static char *reld-up[] = {
/* width height num_colors chars_per_pixel */
\"    33    33       18            1\",
/* colors */
\"# c #000000\",
\"# c #252525\",
\"a c #330099\",
\"b c #4c4c4c\",
\"c c #555555\",
\"d c #6666cc\",
\"e c #737373\",
\"f c #777777\",
\"g c #888888\",
\"h c #999999\",
\"i c #9999ff\",
\"j c #aaaaaa\",
\"_ c #b2b2b2 s backgroundToolBarColor\",
\"l c #bbbbbb\",
\"m c #cccccc\",
\"n c #e1e1e1\",
\"o c #eeeeee\",
\"p c #ffffff\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_____________jfdadfj_____________\",
\"____________gadfjfdag____________\",
\"___________dajoommmjad___________\",
\"__________gapofdadfmmag__________\",
\"_________jajofajjjadmjaj_________\",
\"_________fdpfajjjjjafmaf_________\",
\"_________dfpafj__adjommgda_______\",
\"_________ajpaj____admmmdajj______\",
\"_________gaaag_____admdajj_______\",
\"_________ggggg______adajj________\",
\"_____________________djj_________\",
\"_________#####_____jaaaj_________\",
\"________##___##____amoga_________\",
\"________##_________aoomaj________\",
\"________##_________agmgag________\",
\"________##_________jaaagg________\",
\"________##___##_____jgggj________\",
\"_________#####___________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"__####______#____________________\",
\"__#_________#____________________\",
\"__#____#__#_#_____##__###__###___\",
\"__####_#__#_###__#__#_#__#_#__#__\",
\"__#_____##__#____####_#____#__#__\",
\"__#____#__#_#____#____#____#__#__\",
\"__####_#__#__###__###_#____#__#__\",
\"_________________________________\",
\"_________________________________\"
};")
  "*The C importation button.")

(defvar bee-indent-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",
\"       c #c0c0c0c0c0c0 s backgroundToolBarColor\",
\".      c #B6B6B2B2B6B6\",
\"X      c #000000001010\",
\"o      c #A6A6A2A2A6A6\",
\"O      c #000000000000\",
\"+      c #FFFFFBFBCFCF\",
\"L      c #FFFF00000000\",
\"@      c #D7D7B6B68E8E\",
\"#      c #86867D7D7171\",
\"                                 \",
\"     OOOOOOOOOOOOOOOOOOO         \",
\"     O+++++++++++++++++OO        \",
\"     O+L+++L++LL++LL+++O@O       \",
\"     O+L+++L+L++L+L+L++O@@O      \",
\"     O+L+++L++L+++L+L++O@@@O     \",
\"     O+L+++L+++L++LL+++O@@@@O    \",
\"     O+L+++L+L++L+L++++OOOOOO#   \",
\"     O+LLL+L++LL++L+++++@@@@O#   \",
\"     O++++++++++++++++++@@@@O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OOOOO+O++++++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OOOOOO+++++++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OO++++OO+++OO++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O++OO+++OO+++OO++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     OOOO++++OO++OO+++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O++OOOO+++OO+++O+++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     OOOO+OO+O++++++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OOOO++OOO++OO++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+++++++OOOO+++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OOOO+++OOOO++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     OOOOOOOOOOOOOOOOOOOOOOOO#   \",
\"      ########################   \",
\"                                 \"
};"
   "/* XPM */
static char * image_name[] = {
\"33 33 9 1\",
\"       c #c0c0c0c0c0c0 s backgroundToolBarColor\",
\".      c #B6B6B2B2B6B6\",
\"X      c #000000001010\",
\"o      c #A6A6A2A2A6A6\",
\"O      c #000000000000\",
\"+      c #FFFFFBFBCFCF\",
\"L      c #FFFF00000000\",
\"@      c #D7D7B6B68E8E\",
\"#      c #86867D7D7171\",
\"                                 \",
\"     OOOOOOOOOOOOOOOOOOO         \",
\"     O+++++++++++++++++OO        \",
\"     O+L+++L++LL++LL+++O@O       \",
\"     O+L+++L+L++L+L+L++O@@O      \",
\"     O+L+++L++L+++L+L++O@@@O     \",
\"     O+L+++L+++L++LL+++O@@@@O    \",
\"     O+L+++L+L++L+L++++OOOOOO#   \",
\"     O+LLL+L++LL++L+++++@@@@O#   \",
\"     O++++++++++++++++++@@@@O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OOOOO+O++++++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OOOOOO+++++++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+++OO+OOO+O+++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O++++OO+OO+OO++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+++OOO+OO++OO+++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O++++OOOO+OO+O+++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+++++OOO+OO++O++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+++OOOO+OOO+OO++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O++OOOO++++++++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     O+OOOO+OOOO++++++++++++O#   \",
\"     O++++++++++++++++++++++O#   \",
\"     OOOOOOOOOOOOOOOOOOOOOOOO#   \",
\"      ########################   \",
\"                                 \"
};")
  "*The indent button.")

(defvar bee-print-button
  (toolbar-make-button-list
   "/* XPM */
static char *print-up[] = {
/* width height num_colors chars_per_pixel */
\"    32    30       18            1\",
/* colors */
\". c #000000\",
\"# c #252525\",
\"a c #330099\",
\"b c #4c4c4c\",
\"c c #555555\",
\"d c #6666cc\",
\"e c #737373\",
\"f c #777777\",
\"g c #888888\",
\"h c #999999\",
\"i c #9999ff\",
\"j c #aaaaaa\",
\"k c #b2b2b2 s backgroundToolBarColor\",
\"l c #bbbbbb\",
\"m c #cccccc\",
\"n c #e1e1e1\",
\"o c #eeeeee\",
\"p c #ffffff\",
/* pixels */
\"kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk\",
\"kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk\",
\"kkkkkkkkkkkaaaaaaaaakkkkkkkkkkkk\",
\"kkkkkkkkkkkapppppppakkkkkkkkkkkk\",
\"kkkkkkkkkkkapfmfoooakkkkkkkkkkkk\",
\"kkkkkkkkkkkapooooooakkkkkkkkkkkk\",
\"kkkkkkkkkkkapfmfmfoakkkkkkkkkkkk\",
\"kkkkkkkkkkkapooooooakkkkkkkkkkkk\",
\"kkkkkkkkkkkapfffmfoakkkkkkkkkkkk\",
\"kkkkkkkkkkkapooooooakkkkkkkkkkkk\",
\"kkkkkkkkaaaaaaaaaaaaaaakkkkkkkkk\",
\"kkkkkkkkapppppppppppppajkkkkkkkk\",
\"kkkkkkkkapkkmmmmmmmmmmagjkkkkkkk\",
\"kkkkkkkkapmmmmmmmmmmmmagjkkkkkkk\",
\"kkkkkkkkapmmmmmmmmmmmmagjkkkkkkk\",
\"kkkkkkkkaaaaaaaaaaaaaaagjkkkkkkk\",
\"kkkkkkkkkamgmgmgmgmgmaggjkkkkkkk\",
\"kkkkkkkkkaaaaaaaaaaaaagjjkkkkkkk\",
\"kkkkkkkkkkjggggggggggggjkkkkkkkk\",
\"kkkkkkkkkkkjjjjjjjjjjjjjkkkkkkkk\",
\"kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk\",
\"kkkkkkk...kkkkk.kkkkkkkkkkkkkkkk\",
\"kkkkkkk.kk.kkkkkkkkkkk.kkkkkkkkk\",
\"kkkkkkk.kk.k.k..k.k.kk..kkkkkkkk\",
\"kkkkkkk...kk..k.k..k.k.kkkkkkkkk\",
\"kkkkkkk.kkkk.kk.k.kk.k.kkkkkkkkk\",
\"kkkkkkk.kkkk.kk.k.kk.k.kkkkkkkkk\",
\"kkkkkkk.kkkk.kk.k.kk.kk.kkkkkkkk\",
\"kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk\",
\"kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk\"
};"))

(defvar bee-compile-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 6 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\".	c #836fff\",
\"o	c #000010\",
\"+	c #000000\",
\"@	c #9898f8\",
\"#      c black\",
\"_________________________________\",
\"_________________________________\",
\"____________+++__________________\",
\"__________+++_+++________________\",
\"__________+..+..+________________\",
\"_________++.o++.o+_______________\",
\"_________o..+....+_______________\",
\"_________++..o..o+_______________\",
\"__________o..._.++++_____________\",
\"__________+o+.++++@+__+__________\",
\"____________+++@+@@@++@+_________\",
\"____________+@@@@@@@@@@++________\",
\"_____________+@@@+++@@@+_________\",
\"_____________+@@+@+_+@@+_________\",
\"___________++@@+@+___+@@++_______\",
\"___________+@@@++____+@@@+_______\",
\"___________+++@+@+___+@+++_______\",
\"_____________+@@+@+_+@@+_________\",
\"_____________+@@@+++@@@+_________\",
\"____________+@@@@@@@@@@@+________\",
\"____________++@++@@+++++_________\",
\"_____________++_+@@+__+__________\",
\"________________++++_____________\",
\"_________________________________\",
\"________________________#________\",
\"__###___________________#________\",
\"_#_____##___#_#__###__#_#___##___\",
\"_#____#__#_#_#_#_#__#_#_#__#__#__\",
\"_#____#__#_#_#_#_#__#_#_#__####__\",
\"_#____#__#_#_#_#_#__#_#_#__#_____\",
\"__##_#_##__#_#_#_###__#__##_###__\",
\"_________________#_______________\",
\"_________________#_______________\",
};\")")
  "*The compile button.")

(defvar bee-bmake-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 6 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\".	c #836fff\",
\"o	c #000010\",
\"+	c #000000\",
\"@	c #9898f8\",
\"#      c black\",

\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"______________++++_______________\",
\"___________++_+@@+__+____________\",
\"___________++@+@@@++@+___________\",
\"__________+@@@@@@@@@@++__________\",
\"___________+@@@+++@@@+___________\",
\"___________+@@+@+_+@@+___________\",
\"_________++@@+@+___+@@++_________\",
\"_________+@@@++____+@@@+_________\",
\"_________+++@+@+___+@+++_________\",
\"___________+@@+@+_+@@+___________\",
\"___________+@@@+++@@@+___________\",
\"__________+@@@@@@@@@@@+__________\",
\"__________++@++@@+++++___________\",
\"___________++_+@@+__+____________\",
\"______________++++_______________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"____________#___________###______\",
\"______##_##_#_#________#_________\",
\"______#_#_#_#_#___#_#__#_________\",
\"______#_#_#_##___#_#_#_###_______\",
\"______#_#_#_#_#__#_#_#_#_________\",
\"______#___#_#__#_#_#_#_#_________\",
\"______#___#_#__#_#___#_#_________\",
\"_________________________________\",
\"_________________________________\"};")
   "The button for the Makefile generator.")

(defvar bee-bmake-entry-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 6 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\".	c #836fff\",
\"o	c #000010\",
\"+	c #000000\",
\"@	c #9898f8\",
\"#      c black\",

\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_____________________++++________\",
\"__________________++_+@@+__+_____\",
\"__________________++@+@@@++@+____\",
\"_________________+@@@@@@@@@@++___\",
\"__________#_______+@@@+++@@@+____\",
\"__________##______+@@+@+_+@@+____\",
\"__###########___++@@+@+___+@@++__\",
\"__############__+@@@++____+@@@+__\",
\"__###########___+++@+@+___+@+++__\",
\"__________##______+@@+@+_+@@+____\",
\"__________#_______+@@@+++@@@+____\",
\"_________________+@@@@@@@@@@@+___\",
\"_________________++@++@@+++++____\",
\"__________________++_+@@+__+_____\",
\"_____________________++++________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_____####______#_________________\",
\"_____#_________#_________________\",
\"_____#____###__###_###__#__#_____\",
\"_____####_#__#_#___#__#_#__#_____\",
\"_____#____#__#_#___#____#__#_____\",
\"_____#____#__#_#___#_____###_____\",
\"_____####_#__#__##_#_______#_____\",
\"_________________________##______\",
\"_________________________________\"};")
   "The button for the Makefile entry adding.")

(defvar bee-root-button
  (toolbar-make-button-list
   "/* XPM */
static char *dir[] = {
/* width height num_colors chars_per_pixel */
\"    33    33        5            1\",
/* colors */
\"_ c #c0c0c0 s backgroundToolBarColor\",
\"# c black\",
\"b c #836fff\",
\"c c #080408\",
\"d c #9898f8\",
/* pixels */
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________######__________\",
\"________________#dddddd#_________\",
\"_______________#dbbbbbbd#________\",
\"________#################________\",
\"_______cdddddddddddddddddc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______cdbbbbbbbbbbbbbbbbc_______\",
\"_______ccccccccccccccccccc_______\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_______###_____________#_________\",
\"_______#__#____________#_________\",
\"_______###____##___##__#_________\",
\"_______#__#__#__#_#__#_###_______\",
\"_______#___#_#__#_#__#_#_________\",
\"_______#___#_#__#_#__#_#_________\",
\"_______#___#__##___##___##_______\",
\"_________________________________\",
\"_________________________________\"
};")
   "The button for selecting/prompting the root directory.")

(defvar bee-repl-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 16 1\",
\" 	c #F7F7F3F3F7F7\",
\".	c #B6B6B2B2B6B6\",
\"X	c #000000000000\",
\"o	c #A6A6A2A2A6A6\",
\"O	c #61618282B6B6\",
\"+	c #969692929696\",
\"@	c #696971718E8E\",
\"#	c #080804040808\",
\"$	c #000000001010\",
\"%	c #B6B6AEAEB6B6\",
\"&	c #96969A9AAEAE\",
\"*	c #49498A8AB6B6\",
\"=	c #AEAEAEAEB6B6\",
\"-	c #A6A69E9EA6A6\",
\";	c #8E8E8E8E9696\",
\":	c #F7F7F7F7F7F7\",
\".................................\",
\".O+OO +O+O+O+O+O+O+OOO OOOX OOOX.\",
\".+O+O ++O+O+O+O+O+O+OO OOOX OOOX.\",
\".O+OO +O+O+O+O+O+O+OOX OOOX OOOX.\",
\".XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX.\",
\".X......................... +O+X.\",
\".X......................... O+OX.\",
\".X....X........   ......... +O+X.\",
\".X...XX..X+....   ......... O+OX.\",
\".X....X..+X+...   ......... +OOX.\",
\".X....X..+X+...   ......... O+OX.\",
\".X....X..X+....   ......... +OOX.\",
\".X....+.................... O+OX.\",
\".X......................... +OOX.\",
\".X......................... OOO#.\",
\".X......................... ;OOX.\",
\".X......................... OOOX.\",
\".X......................... OOOX.\",
\".X......................... OOOX.\",
\".X......................... OOOX.\",
\".X                          OOOX.\",
\".XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX.\",
\".................................\",
\".................................\",
\".......XXX.............X.........\",
\".......X..X............X.........\",
\".......XXX....XX..XXX..X.........\",
\".......X..X..X..X.X..X.X.........\",
\".......X...X.XXXX.X..X.X.........\",
\".......X...X.X....X..X.X.........\",
\".......X...X..XXX.XXX...XX.......\",
\"..................X..............\",
\"..................X..............\"};")
  "*The Repl button.")

(defvar bee-bdb-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 3 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\"O	c #5949bd\",
\"#      c black\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"______________#__OO______________\",
\"___________#__#_OOOO_____________\",
\"____________#_#O_OOO_____________\",
\"_____________#OOOOO______________\",
\"_________###_OOOOO_______________\",
\"___________#O_OOO###_____________\",
\"__________O_OOOO#________________\",
\"_________OOOOOO__#_______________\",
\"_________OOOOO#___#______________\",
\"_________OOOOOO#_________________\",
\"_________OOOOO_#_________________\",
\"__________OOO__#_________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_______###______#__#_____________\",
\"_______#__#_____#__#_____________\",
\"_______#__#_____#__#_____________\",
\"_______####___###__###___________\",
\"_______#___#_#__#__#__#__________\",
\"_______#___#_#__#__#__#__________\",
\"_______#___#_#__#__#__#__________\",
\"_______####___###__###___________\",
\"_________________________________\",
\"_________________________________\"};")
   "*The bdb button")

(defvar bee-connect-button
  (toolbar-make-button-list
   "/* XPM */
static char * image_name[] = {
\"33 33 3 1\",
\"_	c #c0c0c0 s backgroundToolBarColor\",
\"O	c #5949bd\",
\"#      c black\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"________________________#__OO____\",
\"_____________________#__#_OOOO___\",
\"___________#__________#_#O_OOO___\",
\"___________##__________#OOOOO____\",
\"__############_____###_OOOOO_____\",
\"__#############______#O_OOO###___\",
\"__############______O_OOOO#______\",
\"___________##______OOOOOO__#_____\",
\"___________#_______OOOOO#___#____\",
\"___________________OOOOOO#_______\",
\"___________________OOOOO_#_______\",
\"____________________OOO__#_______\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_________________________________\",
\"_____________________________#___\",
\"__##_________________________#___\",
\"_#____##__###__###___##___##_###_\",
\"_#___#__#_#__#_#__#_#__#_#___#___\",
\"_#___#__#_#__#_#__#_####_#___#___\",
\"_#___#__#_#__#_#__#_#____#___#___\",
\"__##__##__#__#_#__#__###__##__##_\",
\"_________________________________\",
\"_________________________________\"};")
  "*The connect button")
  
(defvar bee-info-button
  (toolbar-make-button-list
   "/* XPM */
static char * info[] = {
\"33 33 3 1\",
\"_      c Gray75 s backgroundToolBarColor\",
\"o      c #330099\",
\"#      c black\",
\"_________________________________\",
\"_______________ooo_______________\",
\"______________ooooo______________\",
\"_____________ooooooo_____________\",
\"_____________ooooooo_____________\",
\"_____________ooooooo_____________\",
\"______________ooooo______________\",
\"_______________ooo_______________\",
\"_________________________________\",
\"_____________oooooo______________\",
\"____________ooooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"______________ooooo______________\",
\"_____________ooooooo_____________\",
\"____________ooooooooo____________\",
\"_________________________________\",
\"_________________________________\",
\"___________________#_____________\",
\"__________#_______#______________\",
\"__________#_###__###_##__________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#_#__#_________\",
\"__________#_#__#__#__##__________\",
\"_________________________________\",
\"_________________________________\"};"))

(defvar bee-help-button
  (toolbar-make-button-list
   "/* XPM */
static char *foo[] = {
/* width height num_colors chars_per_pixel */
\"    28    28        2            1\",
/* colors */
\"_ c #b0b0b0\",
\"# c #000000\",
/* pixels */
\"____________________________\",
\"____________________________\",
\"____________________________\",
\"____________________________\",
\"________##########__________\",
\"_______##_#_#_#_#_#_________\",
\"______##_#_#_#_#_#_#________\",
\"_____##_#_#######_#_#_______\",
\"_____#_#_#########_#_#______\",
\"_____##_###########_#_______\",
\"_____#_#_###_#_###_#_#______\",
\"_____##_#####_#####_#_#_____\",
\"_____#_#_###_#_###_#_#_#____\",
\"_____##_#_#_#_#####_#_#_#___\",
\"_____#_#_#_#_#####_#_####___\",
\"_____##_#_#_#####_#_#_______\",
\"______##_#_#####_#_#_#______\",
\"_______##_#_###_#_#_#_______\",
\"________##_#####_#_#________\",
\"_________##_#_#_#_#_#_______\",
\"_________#_#_#_#_#_#________\",
\"_________##_###_#___________\",
\"_________#_#####_#__________\",
\"_________##_###_#___________\",
\"________##_#_#_#_#__________\",
\"________#_#_#_#_#_#_________\",
\"____________________________\",
\"____________________________\"
};"))

;*---------------------------------------------------------------------*/
;*    bee-opened-toolbar-height ...                                    */
;*---------------------------------------------------------------------*/
(defun bee-opened-toolbar-height ()
  (+ 8 (glyph-height (car bee-close-toolbar-button))))

;*---------------------------------------------------------------------*/
;*    bee-closed-toolbar-height ...                                    */
;*---------------------------------------------------------------------*/
(defun bee-closed-toolbar-height ()
  (+ 8 (glyph-height (car bee-open-toolbar-button))))

