;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/bee-tools.el                   */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon May 25 17:22:26 1998                          */
;*    Last change :  Thu Sep 17 10:51:37 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Some global bee tools. Amongst them:                             */
;*       - bee-region-excerpt                                          */
;*       - bee-string-excerpt                                          */
;*       - bee-quit-frame-or-emacs                                     */
;*       - bee-fetch-identifier                                        */
;*       - bee-fetch-then-request-identifier                           */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-tools)
(require 'bee-config)
 
;*---------------------------------------------------------------------*/
;*    bee-region-excerpt ...                                           */
;*---------------------------------------------------------------------*/
(defun bee-region-excerpt (min max)
  (if (< max min)
      (let ((old min))
	(setq min max)
	(setq max old)))
  (if (> (- max min) bee-menu-max-width)
      (let ((half (- (/ bee-menu-max-width 4) 1)))
	(concat (buffer-substring min (+ min half))
		" .. "
		(buffer-substring max (- max half))))
    (buffer-substring min max)))

;*---------------------------------------------------------------------*/
;*    bee-string-excerpt ...                                           */
;*---------------------------------------------------------------------*/
(defun bee-string-excerpt (string)
  (let ((len (length string)))
    (if (> len bee-menu-max-width)
	(let ((half (- (/ bee-menu-max-width 2) 1)))
	  (concat (substring string 0 half)
		  " .. "
		  (substring string (- len half) len)))
      string)))

;*---------------------------------------------------------------------*/
;*    bee-quit-frame-or-emacs ...                                      */
;*---------------------------------------------------------------------*/
(defun bee-quit-frame-or-emacs ()
  (interactive)
  (if (eq (next-frame) (selected-frame))
      (save-buffers-kill-emacs)
    (condition-case nil
        (delete-frame (selected-frame) t)
      (error (save-buffers-kill-emacs)))))

;*---------------------------------------------------------------------*/
;*    bee-fetch-identifier ...                                         */
;*    -------------------------------------------------------------    */
;*    Fetch the Bigloo identifier at POS.                              */
;*---------------------------------------------------------------------*/
(defun bee-fetch-identifier (pos)
  "Fetch the Bigloo identifier at POS."
  (interactive "dPosition: ")
  (save-excursion
    ;; we start skipping left until we found a separator
    (let* ((min (save-excursion (beginning-of-line) (point)))
	   (max (save-excursion (end-of-line) (point))))
      (if (looking-at "\\w")
	  ;; we could be located on a identifier we go left until we found
	  ;; a non identifier character
	  (progn
	    (or (save-excursion
		  (backward-char 1)
		  (looking-at "\\W"))
		(backward-word 1))
	    (if (or (= (point) min)
		    (progn
		      (backward-char 1)
		      (looking-at "\\s-\\|(")))
		(let ((start (+ (point) 1)))
		  ;; we have to look right
		  (goto-char pos)
		  (forward-word 1)
		  (if (or (= (point) max) (looking-at "\\s-\\|)\\|::"))
		      ;; we are not sure yet we have to check that it
		      ;; is not a number
		      (let ((end (point)))
			(goto-char start)
			(skip-chars-forward "0-9")
			(if (= (point) end)
			    ;; this is a number
			    nil
			  ;; yes, this is correct
			  (progn
			    (goto-char start)
			    (if (and (> (- end 2) (point))
				     (search-forward "::" (- end 2) t))
				(buffer-substring start (match-beginning 0))
			      (buffer-substring start end)))))
		    nil))
	      nil))
	nil))))

;*---------------------------------------------------------------------*/
;*    bee-interactive-ident ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-interactive-ident (pos prompt)
  (list (bee-fetch-then-request-identifier pos prompt)))
				  
;*---------------------------------------------------------------------*/
;*    bee-fetch-then-request-identifier ...                            */
;*    -------------------------------------------------------------    */
;*    This function try to fetch an identifier. If it fails, the       */
;*    function ask the user for an identifier.                         */
;*---------------------------------------------------------------------*/
(defun bee-fetch-then-request-identifier (pos &optional prompt) 
  "Fetch the Bigloo identifier at POS and when failure, ask for an identifier."
  (interactive "dPosition: ")
  (let ((ident (bee-fetch-identifier pos)))
    (if (stringp ident)
	ident
      (if (not prompt)
	  (read-string "Find: ")
	(read-string prompt)))))

;*---------------------------------------------------------------------*/
;*    bee-fetch-then-request-c-identifier ...                          */
;*    -------------------------------------------------------------    */
;*    This function try to fetch an identifier. If it fails, the       */
;*    function ask the user for an identifier.                         */
;*---------------------------------------------------------------------*/
(defun bee-fetch-then-request-c-identifier (pos &optional prompt) 
  "Fetch the C identifier at POS and when failure, ask for an identifier."
  (interactive "dPosition: ")
  (let ((ident (save-excursion (goto-char pos) (bee-get-c-identifier-at))))
    (if (stringp ident)
	ident
      (if (not prompt)
	  (read-string "Find: ")
	(read-string prompt)))))

;*---------------------------------------------------------------------*/
;*    bee-get-identifier-at ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-get-identifier-at ()
  (save-excursion
    ;; we start skipping left until we found a separator
    (let* ((pos (point))
	   (min (save-excursion (beginning-of-line) (point)))
	   (max (save-excursion (end-of-line) (point))))
      (if (looking-at "\\w")
	  ;; we could be located on a identifier we go left until we found
	  ;; a non identifier character
	  (progn
	    (or (save-excursion
		  (backward-char 1)
		  (looking-at "\\W"))
		(backward-word 1))
	    (if (or (= (point) min)
		    (progn
		      (backward-char 1)
		      (looking-at "\\s-\\|(")))
		(let ((start (+ (point) 1)))
		  ;; we have to look right
		  (goto-char pos)
		  (forward-word 1)
		  (if (or (= (point) max) (looking-at "\\s-\\|[:)]"))
		      ;; we are not sure yet we have to check that it
		      ;; is not a number
		      (let ((end (point)))
			(goto-char start)
			(skip-chars-forward "0-9")
			(if (= (point) end)
			    ;; this is a number
			    nil
			  ;; yes, this is correct
			  (progn
			    (goto-char start)
			    (if (and (> (- end 2) (point))
				     (search-forward "::" (- end 2) t))
				(buffer-substring start (match-beginning 0))
			      (buffer-substring start end)))))
		    nil))
	      nil))
	nil))))

;*---------------------------------------------------------------------*/
;*    bee-request-identifier-at ...                                    */
;*---------------------------------------------------------------------*/
(defun bee-request-identifier-at (&optional prompt)
  (if (not prompt)
      (setq prompt "Find: "))
  (let ((res (bee-get-identifier-at)))
    (if res
	res
      (read-string prompt))))

;*---------------------------------------------------------------------*/
;*    bee-shrink-frame ...                                             */
;*---------------------------------------------------------------------*/
(defun bee-shrink-frame (&optional delta)
  (interactive)
  (if (not delta)
      (setq delta 2))
  (let ((frame (selected-frame)))
    (set-frame-height frame (- (frame-height frame) 3))))

;*---------------------------------------------------------------------*/
;*    bee-enlarge-frame ...                                            */
;*---------------------------------------------------------------------*/
(defun bee-enlarge-frame (&optional delta)
  (interactive)
  (if (not delta)
      (setq delta 2))
  (let ((frame (selected-frame)))
    (set-frame-height frame (+ (frame-height frame) 3))))
	 
;*---------------------------------------------------------------------*/
;*    bee-find-toplevel-sexp ...                                       */
;*    -------------------------------------------------------------    */
;*    Find the toplevel POS is in.                                     */
;*---------------------------------------------------------------------*/
(defun bee-find-toplevel-sexp (pos)
  (save-excursion
    (let* ((sexp nil)
	   (res  (id-select-sexp pos))
	   (old  pos))
      (while (and (consp res) (not (eq (car res) old)))
	(setq sexp res)
	(setq old (car res))
	(setq res (id-select-sexp (car res))))
      sexp)))
  
;*---------------------------------------------------------------------*/
;*    bee-get-c-identifier-at ...                                      */
;*---------------------------------------------------------------------*/
(defun bee-get-c-identifier-at ()
  (save-excursion
    (let ((pos (point))
	  (buffer (current-buffer)))
      ;; we start skipping left until we found a separator
      (let ((id  "0-9a-zA-Z$_")
	    (min (save-excursion (beginning-of-line) (point)))
	    (max (save-excursion (end-of-line) (point))))
	(if (looking-at (concat "[" id "]"))
	    ;; we could be located on a identifier we go left until we found
	    ;; a non identifier character
	    (progn
	      (skip-chars-backward id min)
	      (if (or (= (point) min)
		      (progn
			(backward-char 1)
			(looking-at "\\s-\\|[^\"']")))
		  (let ((start (+ (point) 1)))
		    ;; we have to look right
		    (goto-char pos)
		    (skip-chars-forward id max)
		    (if (or (= (point) max) (looking-at "\\s-\\|[^\"']"))
			;; we are not sure yet we have to check that it
			;; is not a number
			(let ((end (point)))
			  (goto-char start)
			  (skip-chars-forward "0-9")
			  (if (= (point) end)
			      ;; this is a number
			      nil
			    ;; yes, this is correct
			    (buffer-substring start end)))
		      nil))
		nil))
	  nil)))))
