;*=====================================================================*/
;*    serrano/prgm/project/bigloo/emacs/bee-version.el                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Aug 10 16:10:24 1998                          */
;*    Last change :  Wed Aug 12 09:39:13 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Bee versionning                                                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'bee-version)
(require 'bee-config)
(require 'bee-autoload)

;*---------------------------------------------------------------------*/
;*    bee-checkin-project ...                                          */
;*---------------------------------------------------------------------*/
(defun bee-checkin-project ()
  (interactive)
  (if (yes-or-no-p-dialog-box "Checkin new version?")
      (let ((compile-command (format "%s -f %s revision"
				     bee-make
				     (bee-get-makefile))))
	(bee-compile))))

;*---------------------------------------------------------------------*/
;*    bee-tar-gz-project ...                                           */
;*---------------------------------------------------------------------*/
(defun bee-tar-gz-project ()
  (interactive)
  (let ((compile-command (format "%s -f %s tar.gz"
				 bee-make
				 (bee-get-makefile))))
    (bee-compile)))

;*---------------------------------------------------------------------*/
;*    bee-version-highlight-buffer ...                                 */
;*---------------------------------------------------------------------*/
(defun bee-version-highlight-buffer (buffer msg mkmf file regexp keymap)
  (set-buffer buffer)
  (erase-buffer)
  (insert msg)
  (let* ((height (save-excursion (+ 2 (count-lines (point-min) (point-max)))))
	 (cur-height (frame-height (selected-frame)))
	 (new-height (if (> height cur-height) cur-height height))
	 (default-frame-alist (cons 'height
				    (cons new-height default-frame-alist)))
	 (pop-up-frames t))
    (beginning-of-buffer)
    (while (re-search-forward regexp (point-max) t)
      (let* ((version (buffer-substring (match-beginning 1) (match-end 1)))
	     (end  (match-end 0))
	     (extent (make-extent (match-beginning 0) end)))
	(set-extent-property extent 'version (list mkmf file version))
	(set-extent-property extent 'mouse-face 'highlight)
	(set-extent-property extent 'keymap keymap)
	(goto-char end)))
    (pop-to-buffer buffer)
    buffer))

;*---------------------------------------------------------------------*/
;*    checkout map                                                     */
;*---------------------------------------------------------------------*/
(defvar bee-checkout-map (make-sparse-keymap))
(define-key bee-checkout-map [(button2)]
  (function bee-checkout-file))

;*---------------------------------------------------------------------*/
;*    bee-checkout-file-version ...                                    */
;*---------------------------------------------------------------------*/
(defun bee-checkout-file-version ()
  (interactive)
  (let* ((file (file-relative-name (buffer-file-name (current-buffer))
				   bee-root-directory))
	 (cmd (format "cd %s; %s -f %s infofile FILE=%s"
		      bee-root-directory
		      bee-make
		      (bee-get-makefile)
		      file))
	 (res (exec-to-string cmd)))
    (if (not (string= res ""))
	(let* ((regexp "^[^ \t]+[ \t]+\\([^ \t]+\\).*$")
	       (buffer (get-buffer-create (format "*%s-versions*" file))))
	  (bee-version-highlight-buffer buffer
					res
					(bee-get-makefile)
					file
					regexp
					bee-checkout-map))
      (error (format "No version of %S found" file)))))

;*---------------------------------------------------------------------*/
;*    bee-checkout-file ...                                            */
;*---------------------------------------------------------------------*/
(defun bee-checkout-file (event)
  (interactive "e")
  (let* ((point  (event-closest-point event))
	 (buffer (event-buffer event))
	 (extent (extent-at point buffer 'version))
	 (prop   (extent-property extent 'version)))
    (if (consp prop)
	(let* ((mkmf    (car prop))
	       (file    (car (cdr prop)))
	       (version (car (cdr (cdr prop))))
	       (curname (concat file ".current")))
	  ;; we first have to rename the current file
	  (rename-file file curname)
	  (unwind-protect
	      (let* ((cmd (format "cd %s; %s -f %s checkoutfile FILE=%s FILEVERSION=%s"
				  bee-root-directory
				  bee-make
				  mkmf
				  file
				  version))
		     (res (exec-to-string cmd)))
		;; we rename the checked out file
		(if (file-exists-p file)
		    (let* ((name (file-name-sans-extension file))
			   (oldname (if (string= name file)
					(concat file "-" version)
				      (concat file
					      "-" version
					      (substring file
							 (length name)
							 (length file))))))
		      (if (file-exists-p oldname)
			  (delete-file oldname))
		      (rename-file file oldname)
		      ;; we open a buffer for the checked out file
		      (let* ((default-directory bee-root-directory)
			     (buffer (find-file-other-frame oldname)))
			(delete-file oldname)
			buffer))
		  (error (format "Can't restore version %S for file %S because of %S"
				 version
				 file
				 res))))
	    (rename-file curname file))))))
		
;*---------------------------------------------------------------------*/
;*    diff map                                                         */
;*---------------------------------------------------------------------*/
(defvar bee-diff-map (make-sparse-keymap))
(define-key bee-diff-map [(button2)]
  (function bee-diff-file))

;*---------------------------------------------------------------------*/
;*    bee-diff-file-version ...                                        */
;*---------------------------------------------------------------------*/
(defun bee-diff-file-version ()
  (interactive)
  (let* ((file (file-relative-name (buffer-file-name (current-buffer))
				   bee-root-directory))
	 (cmd (format "cd %s; %s -f %s infofile FILE=%s"
		      bee-root-directory
		      bee-make
		      (bee-get-makefile)
		      file))
	 (res (exec-to-string cmd)))
    (if (not (string= res ""))
	(let* ((regexp "^[^ \t]+[ \t]+\\([^ \t]+\\).*$")
	       (buffer (get-buffer-create (format "*%s-versions*" file))))
	  (bee-version-highlight-buffer buffer
					res
					(bee-get-makefile)
					file
					regexp
					bee-diff-map))
      (error (format "No version of %S found" file)))))

;*---------------------------------------------------------------------*/
;*    bee-diff-file ...                                                */
;*---------------------------------------------------------------------*/
(defun bee-diff-file (event)
  (interactive "e")
  (let* ((point   (event-closest-point event))
	 (buffer  (event-buffer event))
	 (obuffer (bee-checkout-file event))
	 (extent  (extent-at point buffer 'version))
	 (prop    (extent-property extent 'version))
	 (file    (car (cdr prop)))
	 (fbuffer (get-buffer file)))
    (if (bufferp obuffer)
	(unwind-protect
	    (progn
	      (set-buffer fbuffer)
	      (ediff-buffers fbuffer obuffer))))))

	
